#include "distcdataset.h"
#include "getdistscommand.h"
#include "listseqscommand.h"
#include "getseqscommand.h"

/***********************************************************************/
DistCDataSet::DistCDataSet() {
    m = MothurOut::getInstance();
}
/***********************************************************************/
vector<string> DistCDataSet::getFiles(int numSeqs) {
    vector<string> newFiles;
    
    if (numSeqs > 2550) { m->mothurOut("[ERROR]: too many seqs requested in DistCDataSet::getFiles\n"); }
    else {
        string countfile = writeCountFile();
        string columnfile = writeColumnFile();
        
        string inputString = "count=" + countfile;
        m->mothurOut("/******************************************/"); m->mothurOutEndLine();
        m->mothurOut("Running command: list.seqs(" + inputString + ")"); m->mothurOutEndLine();
        m->mothurCalling = true;
        
        Command* listCommand = new ListSeqsCommand(inputString);
        listCommand->execute();
        
        map<string, vector<string> > filenames = listCommand->getOutputFiles();
        
        delete listCommand;
        m->mothurCalling = false;
        
        string accnosfile = filenames["accnos"][0];
        m->mothurOut("/******************************************/"); m->mothurOutEndLine();
        
        ifstream in;
        m->openInputFile(accnosfile, in);
        
        ofstream out;
        m->openOutputFile("temp.accnos", out);
        
        int count = 0; string name;
        while(!in.eof()) {
            if (m->control_pressed) { break; }
            
            in >> name; m->gobble(in);
            out << name << endl;
            count++;
            
            if (count >= numSeqs) { break; }
        }
        in.close();
        out.close();
        m->mothurRemove(accnosfile);
        
        inputString = "count=" + countfile + ", accnos=temp.accnos";
        m->mothurOut("/******************************************/"); m->mothurOutEndLine();
        m->mothurOut("Running command: get.seqs(" + inputString + ")"); m->mothurOutEndLine();
        m->mothurCalling = true;
        
        Command* getCommand = new GetSeqsCommand(inputString);
        getCommand->execute();
        
        filenames = getCommand->getOutputFiles();
        
        delete getCommand;
        m->mothurCalling = false;
        
        string newCountfile = filenames["count"][0];
        m->mothurOut("/******************************************/"); m->mothurOutEndLine();
        
        inputString = "column=" + columnfile + ", accnos=temp.accnos";
        m->mothurOut("/******************************************/"); m->mothurOutEndLine();
        m->mothurOut("Running command: get.dists(" + inputString + ")"); m->mothurOutEndLine();
        m->mothurCalling = true;
        
        Command* getDCommand = new GetDistsCommand(inputString);
        getDCommand->execute();
        
        filenames = getDCommand->getOutputFiles();
        
        delete getDCommand;
        m->mothurCalling = false;
        
        string newColumnfile = filenames["column"][0];
        m->mothurOut("/******************************************/"); m->mothurOutEndLine();
        
        newFiles.push_back(newColumnfile); newFiles.push_back(newCountfile);
    }
    
    return newFiles;
}
/***********************************************************************/
string DistCDataSet::writeColumnFile() {
    
    ofstream out;
    m->openOutputFile("column.temp", out);


out << "M00967_43_000000000-A3JHG_1_1101_20384_5679 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16061_3516 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16061_3516 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16061_3516 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12395_11489 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17084_15866 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17084_15866 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17084_15866 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20653_21248 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24493_5303 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24493_5303 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28036_9037 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28036_9037 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28036_9037 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13731_17325 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16780_2259 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15701_17582 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15701_17582 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15994_22947 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15994_22947 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16041_22886 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17289_26831 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17289_26831 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17289_26831 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26066_8122 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26066_8122 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26066_8122 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26066_8122 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26066_8122 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24688_14454 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24688_14454 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20031_8045 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13449_7439 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15591_4696 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11725_18144 M00967_43_000000000-A3JHG_1_1102_27488_14514 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22051_11604 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22051_11604 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22051_11604 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22051_11604 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22051_11604 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22085_12796 M00967_43_000000000-A3JHG_1_1102_27189_17647 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18313_3175 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18313_3175 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13421_28822 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13421_28822 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13421_28822 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_5231_15971 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21369_2858 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23238_24359 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25418_11257 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25962_10907 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8082_6010 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7659_11830 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13682_16758 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20235_16372 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_5171_14027 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6552_16619 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6552_16619 M00967_43_000000000-A3JHG_1_1103_5171_14027 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16815_14755 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16815_14755 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16815_14755 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9943_10858 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25695_18492 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_6956_16009 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9234_4159 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15381_10377 M00967_43_000000000-A3JHG_1_1107_3735_12375 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19875_9916 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19875_9916 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13777_28036 M00967_43_000000000-A3JHG_1_1107_3735_12375 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13777_28036 M00967_43_000000000-A3JHG_1_2103_15381_10377 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7075_7100 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27485_13416 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24599_6936 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24599_6936 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11673_24359 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10609_18161 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10609_18161 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21243_9500 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20070_20418 M00967_43_000000000-A3JHG_1_1101_11856_9674 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18770_23200 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21793_13370 M00967_43_000000000-A3JHG_1_1102_27189_17647 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21793_13370 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21793_13370 M00967_43_000000000-A3JHG_1_1113_6044_19045 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21793_13370 M00967_43_000000000-A3JHG_1_1103_5171_14027 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21793_13370 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20139_17113 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13053_11087 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21375_27109 M00967_43_000000000-A3JHG_1_1113_13777_28036 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12761_22353 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10606_2663 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10606_2663 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10606_2663 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10606_2663 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26838_20823 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_26838_20823 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_21303_18168 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17560_17233 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17560_17233 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10943_19850 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25888_23585 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9553_14094 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9553_14094 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9553_14094 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7408_16842 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7408_16842 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18278_3345 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18278_3345 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18278_3345 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18278_3345 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18278_3345 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18143_13375 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18143_13375 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18143_13375 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6259_8307 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6259_8307 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4298_19710 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4298_19710 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4298_19710 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_27537_17929 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16038_11947 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16038_11947 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16038_11947 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14336_9476 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14336_9476 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10492_14599 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10492_14599 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8332_13243 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21352_15666 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21352_15666 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21352_15666 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18397_7516 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14457_3580 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12406_27672 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26773_14017 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26773_14017 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26066_19401 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15899_17037 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15899_17037 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15899_17037 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10826_19542 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10826_19542 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6782_8678 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5664_6671 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5664_6671 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_29227_17392 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_29227_17392 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16376_7553 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16376_7553 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16376_7553 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16376_7553 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12338_23302 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12338_23302 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_6116_13104 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23478_21034 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23478_21034 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23478_21034 M00967_43_000000000-A3JHG_1_1104_13731_17325 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10107_20150 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10107_20150 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7804_9298 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7804_9298 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7804_9298 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7804_9298 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7804_9298 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7674_17990 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7674_17990 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5077_12621 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_24812_22312 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_24812_22312 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_24812_22312 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_24812_22312 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19840_26409 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19840_26409 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19840_26409 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19840_26409 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19309_25091 M00967_43_000000000-A3JHG_1_1113_6044_19045 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10435_17390 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26817_14749 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26817_14749 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26817_14749 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26817_14749 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26555_14955 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22891_19654 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22891_19654 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19750_8249 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19750_8249 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19750_8249 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16743_9576 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16743_9576 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14305_11884 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14305_11884 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14305_11884 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14305_11884 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14305_11884 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12968_23126 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10758_21336 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10758_21336 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_8155_16465 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_8155_16465 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_8155_16465 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_8155_16465 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_8155_16465 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22135_19871 M00967_43_000000000-A3JHG_1_2105_9234_4159 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17957_13444 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17320_11249 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17320_11249 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16910_23126 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16910_23126 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16910_23126 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16910_23126 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16910_23126 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9975_21883 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20685_7431 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15263_18242 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7701_13077 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22514_6355 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22514_6355 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22029_15973 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22029_15973 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18861_20938 M00967_43_000000000-A3JHG_1_1106_21375_27109 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17221_27256 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17221_27256 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17221_27256 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17221_27256 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14234_20506 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14234_20506 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14234_20506 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13201_17677 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17019_10818 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17019_10818 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17019_10818 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15542_26928 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15542_26928 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15542_26928 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12930_14637 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12930_14637 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12696_27241 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12696_27241 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12263_16584 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12263_16584 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16955_14005 M00967_43_000000000-A3JHG_1_1104_9801_4846 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13466_27462 M00967_43_000000000-A3JHG_1_1101_11856_9674 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13466_27462 M00967_43_000000000-A3JHG_1_1108_20070_20418 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8603_11968 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8603_11968 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8603_11968 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5902_6841 M00967_43_000000000-A3JHG_1_1101_11666_7824 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3131_16050 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3131_16050 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3131_16050 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3131_16050 M00967_43_000000000-A3JHG_1_2102_17019_10818 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25590_23883 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25590_23883 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23055_10307 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22804_12324 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22804_12324 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22804_12324 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22681_21130 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22681_21130 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_1863_15569 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_1863_15569 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13397_2211 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13397_2211 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13397_2211 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12471_18325 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12471_18325 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12471_18325 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8139_17897 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8139_17897 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8139_17897 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8139_17897 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23502_11596 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23502_11596 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19532_4899 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19532_4899 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19532_4899 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19266_19695 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18958_10702 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18958_10702 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18958_10702 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8821_13863 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8821_13863 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8821_13863 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8821_13863 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20884_13058 M00967_43_000000000-A3JHG_1_1104_7522_5556 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17696_15281 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17696_15281 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17696_15281 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17696_15281 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17696_15281 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12844_3769 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12844_3769 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12844_3769 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12844_3769 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10194_20788 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17002_7999 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13451_28512 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13451_28512 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13451_28512 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9936_12702 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9936_12702 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9902_7407 M00967_43_000000000-A3JHG_1_1104_17315_20406 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17220_22884 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17220_22884 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17220_22884 M00967_43_000000000-A3JHG_1_2111_20158_16824 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14717_26485 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14717_26485 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13033_21990 M00967_43_000000000-A3JHG_1_1105_14835_21938 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8014_5010 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8014_5010 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8014_5010 M00967_43_000000000-A3JHG_1_2107_26555_14955 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7996_16802 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7996_16802 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7996_16802 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7996_16802 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7996_16802 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20401_24219 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20401_24219 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20401_24219 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20401_24219 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20401_24219 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14359_12484 M00967_43_000000000-A3JHG_1_1111_22890_9747 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4844_10914 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4844_10914 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4844_10914 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18996_9436 M00967_43_000000000-A3JHG_1_1106_22417_19200 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21038_8569 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21038_8569 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21038_8569 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21038_8569 M00967_43_000000000-A3JHG_1_2102_17019_10818 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21038_8569 M00967_43_000000000-A3JHG_1_1114_3131_16050 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26087_11342 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26087_11342 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26087_11342 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20287_11044 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12555_24141 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12231_13452 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11240_22282 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9179_20196 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9179_20196 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9179_20196 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9179_20196 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14547_12843 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14547_12843 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14547_12843 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14547_12843 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14252_27883 M00967_43_000000000-A3JHG_1_1113_18548_4292 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_5754_14689 M00967_43_000000000-A3JHG_1_2104_12505_3424 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3864_17599 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3864_17599 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3864_17599 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14518_16099 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6990_17751 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6990_17751 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6990_17751 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6990_17751 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5880_14023 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5880_14023 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_2114_15227 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_2114_15227 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_2114_15227 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17816_13048 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17816_13048 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17816_13048 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15058_11924 M00967_43_000000000-A3JHG_1_1113_20207_16728 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10827_17300 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10827_17300 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10827_17300 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10827_17300 M00967_43_000000000-A3JHG_1_1114_3131_16050 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10827_17300 M00967_43_000000000-A3JHG_1_1107_21038_8569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9620_19745 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9620_19745 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18044_1900 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14364_8401 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12695_21368 M00967_43_000000000-A3JHG_1_1101_11856_9674 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12695_21368 M00967_43_000000000-A3JHG_1_1108_20070_20418 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12695_21368 M00967_43_000000000-A3JHG_1_2101_13466_27462 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11035_15765 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11035_15765 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11035_15765 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11035_15765 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10331_23332 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17613_16563 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.003922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17613_16563 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14280_26421 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14280_26421 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14280_26421 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14280_26421 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14280_26421 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23612_18072 M00967_43_000000000-A3JHG_1_1109_9629_6769 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12567_15271 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12567_15271 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20527_14577 M00967_43_000000000-A3JHG_1_1102_11115_19075 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18492_19103 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18492_19103 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18492_19103 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21792_3253 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10121_22720 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4962_7612 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8537_7072 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4696_10935 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4696_10935 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7742_10390 M00967_43_000000000-A3JHG_1_1104_17583_11048 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17763_13764 M00967_43_000000000-A3JHG_1_2111_6887_10490 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3758_10627 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22451_22884 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22451_22884 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12446_11570 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12446_11570 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12446_11570 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11924_15441 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11924_15441 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11757_15493 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11757_15493 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11757_15493 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11757_15493 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11757_15493 M00967_43_000000000-A3JHG_1_1114_22804_12324 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22954_15571 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22954_15571 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22954_15571 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18460_8495 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18460_8495 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13923_24399 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13923_24399 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13923_24399 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13923_24399 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8291_8111 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5139_21514 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5139_21514 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5139_21514 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5139_21514 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23078_24856 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23078_24856 M00967_43_000000000-A3JHG_1_1102_22375_5719 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21066_7506 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21066_7506 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21066_7506 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19248_7551 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13579_10781 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13579_10781 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12166_13729 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12166_13729 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12166_13729 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12166_13729 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12166_13729 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11932_12993 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11932_12993 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9416_24199 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9416_24199 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9416_24199 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9416_24199 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9416_24199 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6882_14781 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6882_14781 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6882_14781 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6882_14781 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6882_14781 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20252_6302 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20252_6302 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20252_6302 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14530_23810 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5052_15488 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5052_15488 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5052_15488 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5052_15488 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5052_15488 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18358_2270 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18358_2270 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18358_2270 M00967_43_000000000-A3JHG_1_1103_14518_16099 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18358_2270 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11790_16027 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11790_16027 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11790_16027 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11790_16027 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10122_16816 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10122_16816 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8153_5482 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8153_5482 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8153_5482 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_8153_5482 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27195_14074 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27195_14074 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27195_14074 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27195_14074 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27195_14074 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23336_11976 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23336_11976 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23336_11976 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23336_11976 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23336_11976 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14104_6500 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14104_6500 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14104_6500 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14104_6500 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14104_6500 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14096_15016 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14096_15016 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14096_15016 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14096_15016 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14096_15016 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5552_10582 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5552_10582 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5552_10582 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5552_10582 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19668_20827 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19668_20827 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19668_20827 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18899_8185 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18899_8185 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18899_8185 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18899_8185 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18899_8185 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7856_11382 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7856_11382 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7856_11382 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7856_11382 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6640_14822 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6640_14822 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6640_14822 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6640_14822 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_24666_20800 M00967_43_000000000-A3JHG_1_1113_20770_14973 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22604_4240 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_28391_18939 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_28391_18939 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_28391_18939 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_28391_18939 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_27710_20218 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_27710_20218 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17292_21391 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17292_21391 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17292_21391 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13551_2912 M00967_43_000000000-A3JHG_1_1103_5171_14027 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21549_4010 M00967_43_000000000-A3JHG_1_1112_10024_21424 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4318_16941 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21002_11725 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21002_11725 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21002_11725 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21002_11725 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12649_18573 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12649_18573 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12649_18573 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12649_18573 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12649_18573 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12062_8139 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12062_8139 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10719_18379 M00967_43_000000000-A3JHG_1_1105_18475_22342 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9943_15231 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9943_15231 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9943_15231 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9141_21408 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9141_21408 M00967_43_000000000-A3JHG_1_1112_10194_20788 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27130_19681 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27130_19681 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24306_21531 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24306_21531 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24306_21531 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22274_20191 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22274_20191 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22274_20191 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11506_10347 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11506_10347 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11506_10347 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11506_10347 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4607_20709 M00967_43_000000000-A3JHG_1_2103_14393_8923 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14408_20028 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14408_20028 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14408_20028 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14408_20028 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11386_7041 M00967_43_000000000-A3JHG_1_2108_8220_23281 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9431_18167 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9431_18167 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9431_18167 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9431_18167 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4449_10146 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4449_10146 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4449_10146 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4449_10146 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3713_15410 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3713_15410 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3713_15410 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3713_15410 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3713_15410 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13879_1994 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13879_1994 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11254_14780 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11254_14780 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11254_14780 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11254_14780 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_5351_11203 M00967_43_000000000-A3JHG_1_1113_20770_14973 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_5351_11203 M00967_43_000000000-A3JHG_1_2102_24666_20800 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18126_7545 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18126_7545 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18126_7545 M00967_43_000000000-A3JHG_1_1103_14518_16099 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18126_7545 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18126_7545 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27354_15573 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_2102_17019_10818 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_1114_3131_16050 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_1107_21038_8569 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696 M00967_43_000000000-A3JHG_1_1102_10827_17300 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26583_8053 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26583_8053 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26583_8053 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26583_8053 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13967_11089 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13967_11089 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13967_11089 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13967_11089 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_9733_12197 M00967_43_000000000-A3JHG_1_1109_11007_22305 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22881_7086 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22881_7086 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22881_7086 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15090_18340 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15090_18340 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15090_18340 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19278_14733 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19278_14733 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19278_14733 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19278_14733 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19278_14733 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11900_6285 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11900_6285 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11900_6285 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11900_6285 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11900_6285 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12896_14027 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_3890_17221 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_3890_17221 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8995_24296 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8995_24296 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12242_23443 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12242_23443 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12242_23443 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7382_19629 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7382_19629 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_9258_26812 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20023_3542 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14628_22758 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14628_22758 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14628_22758 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14628_22758 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24706_13976 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15635_4811 M00967_43_000000000-A3JHG_1_1105_15878_26239 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14329_23174 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14329_23174 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10076_22271 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10076_22271 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7671_14378 M00967_43_000000000-A3JHG_1_2103_14393_8923 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7671_14378 M00967_43_000000000-A3JHG_1_2113_25899_22034 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7671_14378 M00967_43_000000000-A3JHG_1_1111_4607_20709 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4146_15034 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4146_15034 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4146_15034 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4146_15034 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12741_23025 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13473_22310 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13473_22310 M00967_43_000000000-A3JHG_1_1104_10943_19850 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20678_18094 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8186_13301 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8186_13301 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8186_13301 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8186_13301 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8186_13301 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6114_18144 M00967_43_000000000-A3JHG_1_1105_18475_22342 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14170_11158 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14170_11158 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14170_11158 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13298_25616 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13298_25616 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9657_11455 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9657_11455 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15602_23047 M00967_43_000000000-A3JHG_1_1101_13431_10294 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10641_20956 M00967_43_000000000-A3JHG_1_1106_25828_15956 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27497_14930 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27497_14930 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27497_14930 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27497_14930 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13445_6827 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13445_6827 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13445_6827 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13445_6827 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17498_8085 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7936_8291 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6909_9337 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6909_9337 M00967_43_000000000-A3JHG_1_2107_26555_14955 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6909_9337 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27160_14501 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25337_11987 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25337_11987 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25337_11987 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16818_12005 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16818_12005 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16818_12005 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16818_12005 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_16818_12005 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19677_24897 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19677_24897 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18261_8446 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18261_8446 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19057_10430 M00967_43_000000000-A3JHG_1_2107_12720_21761 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14570_24653 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14570_24653 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11954_7925 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11954_7925 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_6445_10536 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_6445_10536 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_6445_10536 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26498_14617 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26498_14617 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26498_14617 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26498_14617 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26498_14617 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25094_24210 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25094_24210 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25094_24210 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17535_7182 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17535_7182 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22108_9150 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22108_9150 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19154_8327 M00967_43_000000000-A3JHG_1_1102_22375_5719 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18190_18406 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18190_18406 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18190_18406 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18190_18406 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13389_13295 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13389_13295 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13389_13295 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1102_29181_14456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13883_23943 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13883_23943 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13883_23943 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13883_23943 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13883_23943 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12104_26590 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11232_14725 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9549_5158 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9549_5158 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9549_5158 M00967_43_000000000-A3JHG_1_1104_10875_20179 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27414_10900 M00967_43_000000000-A3JHG_1_1106_25798_21572 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26682_16357 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26682_16357 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26682_16357 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26682_16357 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19666_21477 M00967_43_000000000-A3JHG_1_1109_15029_15883 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18757_14747 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18757_14747 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18757_14747 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16536_4799 M00967_43_000000000-A3JHG_1_1109_11007_22305 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16536_4799 M00967_43_000000000-A3JHG_1_1106_9733_12197 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21774_11978 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21774_11978 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21774_11978 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18950_6153 M00967_43_000000000-A3JHG_1_1106_24475_12056 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18818_19657 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18818_19657 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18818_19657 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11611_23800 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11611_23800 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11611_23800 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_11611_23800 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10693_14739 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10693_14739 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10588_13620 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10033_25797 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11299_13989 M00967_43_000000000-A3JHG_1_1106_25798_21572 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11299_13989 M00967_43_000000000-A3JHG_1_1112_27414_10900 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_15027_8794 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_15027_8794 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_15027_8794 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_15027_8794 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8149_24158 M00967_43_000000000-A3JHG_1_2105_11723_5649 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7357_12797 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7357_12797 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7357_12797 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17692_2367 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17692_2367 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17692_2367 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17692_2367 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17692_2367 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15774_18346 M00967_43_000000000-A3JHG_1_1102_27189_17647 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20176_26018 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20176_26018 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20176_26018 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20176_26018 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27578_17650 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27578_17650 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22959_4225 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22959_4225 M00967_43_000000000-A3JHG_1_1110_20816_14150 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11324_6735 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11324_6735 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11324_6735 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11324_6735 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22735_22577 M00967_43_000000000-A3JHG_1_1101_13431_10294 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22735_22577 M00967_43_000000000-A3JHG_1_2109_15602_23047 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16001_2026 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16001_2026 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16001_2026 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16001_2026 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16001_2026 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14563_13874 M00967_43_000000000-A3JHG_1_1105_27806_20120 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26933_11956 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26933_11956 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26933_11956 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19584_26279 M00967_43_000000000-A3JHG_1_1102_22375_5719 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19584_26279 M00967_43_000000000-A3JHG_1_1104_17583_11048 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19584_26279 M00967_43_000000000-A3JHG_1_1110_7742_10390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19584_26279 M00967_43_000000000-A3JHG_1_1114_19154_8327 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18198_2609 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18198_2609 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18198_2609 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18198_2609 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15350_5623 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4725_7023 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24913_16194 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24913_16194 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24913_16194 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23687_14507 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23687_14507 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23687_14507 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23687_14507 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18844_7914 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18844_7914 M00967_43_000000000-A3JHG_1_2111_19248_7551 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15802_4098 M00967_43_000000000-A3JHG_1_1110_27249_9297 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22673_23252 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17219_14317 M00967_43_000000000-A3JHG_1_1102_29181_14456 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8189_7507 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8189_7507 M00967_43_000000000-A3JHG_1_2111_14170_11158 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22020_21434 M00967_43_000000000-A3JHG_1_2107_12720_21761 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22020_21434 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14290_10968 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14290_10968 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14290_10968 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14290_10968 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10304_14471 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10304_14471 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10304_14471 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6844_12008 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6844_12008 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6844_12008 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6844_12008 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6844_12008 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17326_26211 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17326_26211 M00967_43_000000000-A3JHG_1_2108_22020_21434 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25610_6877 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25610_6877 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25610_6877 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25610_6877 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11759_16153 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11759_16153 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11759_16153 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11759_16153 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20113_16810 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20113_16810 M00967_43_000000000-A3JHG_1_2106_27160_14501 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9084_14502 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9084_14502 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9084_14502 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9084_14502 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9084_14502 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654 M00967_43_000000000-A3JHG_1_1106_12555_24141 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20989_19235 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20989_19235 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20989_19235 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14259_12215 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14259_12215 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14259_12215 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13584_18783 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13584_18783 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13584_18783 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13584_18783 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13584_18783 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13545_27181 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13545_27181 M00967_43_000000000-A3JHG_1_1104_10943_19850 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13545_27181 M00967_43_000000000-A3JHG_1_2113_13473_22310 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27215_10406 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23401_8348 M00967_43_000000000-A3JHG_1_1101_11666_7824 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23401_8348 M00967_43_000000000-A3JHG_1_1114_5902_6841 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19958_8814 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5118_11366 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5118_11366 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26033_17779 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26033_17779 M00967_43_000000000-A3JHG_1_2111_19248_7551 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_26033_17779 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17916_7193 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17916_7193 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17916_7193 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17916_7193 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16159_23387 M00967_43_000000000-A3JHG_1_2112_17281_21576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16159_23387 M00967_43_000000000-A3JHG_1_2107_6821_13209 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16159_23387 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11192_11762 M00967_43_000000000-A3JHG_1_1113_18548_4292 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11192_11762 M00967_43_000000000-A3JHG_1_1104_14252_27883 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10124_4953 M00967_43_000000000-A3JHG_1_1101_13431_10294 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10124_4953 M00967_43_000000000-A3JHG_1_2109_15602_23047 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10124_4953 M00967_43_000000000-A3JHG_1_1113_22735_22577 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18224_14666 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18224_14666 M00967_43_000000000-A3JHG_1_1104_10943_19850 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18224_14666 M00967_43_000000000-A3JHG_1_2113_13473_22310 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18224_14666 M00967_43_000000000-A3JHG_1_1114_13545_27181 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675 M00967_43_000000000-A3JHG_1_2101_9084_14502 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11925_5550 M00967_43_000000000-A3JHG_1_1109_11007_22305 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11925_5550 M00967_43_000000000-A3JHG_1_1106_9733_12197 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11925_5550 M00967_43_000000000-A3JHG_1_1112_16536_4799 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5736_9217 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_2111_14170_11158 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083 M00967_43_000000000-A3JHG_1_2110_8189_7507 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10006_21831 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10006_21831 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10006_21831 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_27552_22078 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_27552_22078 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_27552_22078 M00967_43_000000000-A3JHG_1_1110_10693_14739 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1102_29181_14456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_15844_3660 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_15844_3660 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10911_24264 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10911_24264 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_2468_14102 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_2468_14102 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_2468_14102 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20644_25868 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20644_25868 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20644_25868 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16624_8157 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16624_8157 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16624_8157 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16624_8157 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16624_8157 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15636_27192 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15636_27192 M00967_43_000000000-A3JHG_1_2106_27160_14501 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15636_27192 M00967_43_000000000-A3JHG_1_2103_20113_16810 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15318_9427 M00967_43_000000000-A3JHG_1_2102_6705_15205 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17535_11802 M00967_43_000000000-A3JHG_1_2114_13884_15208 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25170_16709 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25170_16709 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25170_16709 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25170_16709 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_3871_8294 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_3871_8294 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_3871_8294 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_3871_8294 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050 M00967_43_000000000-A3JHG_1_1103_14518_16099 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050 M00967_43_000000000-A3JHG_1_2106_16818_12005 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13714_14338 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13714_14338 M00967_43_000000000-A3JHG_1_1113_4318_16941 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7484_9022 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7484_9022 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7484_9022 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15973_8357 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15973_8357 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15973_8357 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15973_8357 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15973_8357 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20575_21363 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20575_21363 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20575_21363 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20575_21363 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15255_5535 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15255_5535 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15255_5535 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15255_5535 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15255_5535 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16554_11081 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16554_11081 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16554_11081 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8686_5552 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869 M00967_43_000000000-A3JHG_1_1109_12129_4985 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26547_22871 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26547_22871 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16087_14724 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16087_14724 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16087_14724 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16087_14724 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_22495_21331 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21713_13835 M00967_43_000000000-A3JHG_1_2105_20695_7710 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_29278_13914 M00967_43_000000000-A3JHG_1_2107_6821_13209 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_29278_13914 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16551_3455 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16551_3455 M00967_43_000000000-A3JHG_1_1110_5736_9217 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27757_15813 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27757_15813 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_27757_15813 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23831_12272 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23831_12272 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23831_12272 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23831_12272 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23831_12272 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19460_3735 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19460_3735 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19460_3735 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19460_3735 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15738_17268 M00967_43_000000000-A3JHG_1_1114_26968_12650 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14094_18415 M00967_43_000000000-A3JHG_1_2103_21597_21834 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12578_10682 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12578_10682 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24712_15313 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24712_15313 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24712_15313 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24712_15313 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24712_15313 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23104_8731 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23104_8731 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23104_8731 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21230_21464 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21230_21464 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21230_21464 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21230_21464 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21230_21464 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13757_8649 M00967_43_000000000-A3JHG_1_1103_8515_3790 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7887_6353 M00967_43_000000000-A3JHG_1_1105_15878_26239 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7887_6353 M00967_43_000000000-A3JHG_1_1107_15635_4811 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19578_6645 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19578_6645 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19578_6645 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16319_7131 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16319_7131 M00967_43_000000000-A3JHG_1_1104_10943_19850 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16319_7131 M00967_43_000000000-A3JHG_1_2113_13473_22310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16319_7131 M00967_43_000000000-A3JHG_1_1114_13545_27181 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16319_7131 M00967_43_000000000-A3JHG_1_1111_18224_14666 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10169_13566 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10169_13566 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10169_13566 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10169_13566 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12937_4366 M00967_43_000000000-A3JHG_1_1110_27249_9297 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12937_4366 M00967_43_000000000-A3JHG_1_2113_15802_4098 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26736_16107 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9301_13162 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9301_13162 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9301_13162 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9301_13162 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9301_13162 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_3066_12463 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_3066_12463 M00967_43_000000000-A3JHG_1_2108_29278_13914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21256_12063 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21256_12063 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21256_12063 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_9466_21791 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_9466_21791 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_9466_21791 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_9466_21791 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_9466_21791 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15312_18978 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15312_18978 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15312_18978 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15312_18978 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15463_23056 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15463_23056 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15463_23056 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15463_23056 M00967_43_000000000-A3JHG_1_1107_2468_14102 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15335_17802 M00967_43_000000000-A3JHG_1_2103_21597_21834 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15335_17802 M00967_43_000000000-A3JHG_1_2106_14094_18415 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14183_13016 M00967_43_000000000-A3JHG_1_2103_12830_18492 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10946_20858 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10946_20858 M00967_43_000000000-A3JHG_1_2109_22495_21331 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12715_19004 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12715_19004 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12715_19004 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12715_19004 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_7641_20331 M00967_43_000000000-A3JHG_1_2103_21597_21834 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_7641_20331 M00967_43_000000000-A3JHG_1_2106_14094_18415 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_7641_20331 M00967_43_000000000-A3JHG_1_1110_15335_17802 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14909_4503 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14909_4503 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14909_4503 M00967_43_000000000-A3JHG_1_2101_17535_7182 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12975_13764 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12975_13764 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12975_13764 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12975_13764 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10730_20249 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10730_20249 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10730_20249 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10730_20249 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10730_20249 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20748_16898 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20748_16898 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20748_16898 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20748_16898 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20748_16898 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17173_6497 M00967_43_000000000-A3JHG_1_2103_12830_18492 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_17173_6497 M00967_43_000000000-A3JHG_1_1110_14183_13016 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14437_24723 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14437_24723 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14437_24723 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_9232_9214 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_9232_9214 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458 M00967_43_000000000-A3JHG_1_2113_24913_16194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27543_11258 M00967_43_000000000-A3JHG_1_1102_21382_27245 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25410_11631 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25410_11631 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25410_11631 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25410_11631 M00967_43_000000000-A3JHG_1_1107_2468_14102 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25410_11631 M00967_43_000000000-A3JHG_1_1110_15463_23056 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18988_15558 M00967_43_000000000-A3JHG_1_1101_13158_12636 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15675_15142 M00967_43_000000000-A3JHG_1_1110_12738_11413 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23257_9242 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23257_9242 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23257_9242 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23257_9242 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12675_27249 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12675_27249 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12222_5327 M00967_43_000000000-A3JHG_1_1106_22417_19200 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12222_5327 M00967_43_000000000-A3JHG_1_1108_18996_9436 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6362_13340 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6362_13340 M00967_43_000000000-A3JHG_1_2111_14170_11158 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5575_6810 M00967_43_000000000-A3JHG_1_2103_14393_8923 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5575_6810 M00967_43_000000000-A3JHG_1_1111_4607_20709 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5575_6810 M00967_43_000000000-A3JHG_1_1103_7671_14378 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.003922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_2101_9084_14502 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644 M00967_43_000000000-A3JHG_1_1111_12899_5675 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8303_10880 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8303_10880 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12364_4557 M00967_43_000000000-A3JHG_1_1102_14677_6391 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18085_11993 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18085_11993 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18085_11993 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18085_11993 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18085_11993 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16796_6663 M00967_43_000000000-A3JHG_1_1102_21382_27245 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16796_6663 M00967_43_000000000-A3JHG_1_1105_27543_11258 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_2104_17991_24804 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22672_12392 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22672_12392 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22672_12392 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22672_12392 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22672_12392 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20183_27184 M00967_43_000000000-A3JHG_1_2103_21597_21834 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20183_27184 M00967_43_000000000-A3JHG_1_2106_14094_18415 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20183_27184 M00967_43_000000000-A3JHG_1_1110_15335_17802 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20183_27184 M00967_43_000000000-A3JHG_1_1108_7641_20331 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11899_4014 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11899_4014 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11899_4014 M00967_43_000000000-A3JHG_1_1106_12555_24141 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11899_4014 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11899_4014 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5740_11365 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19731_20280 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19731_20280 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19731_20280 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19731_20280 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19731_20280 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14117_4744 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11446_24358 M00967_43_000000000-A3JHG_1_1101_13431_10294 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11446_24358 M00967_43_000000000-A3JHG_1_2109_15602_23047 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11446_24358 M00967_43_000000000-A3JHG_1_1113_22735_22577 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11446_24358 M00967_43_000000000-A3JHG_1_1112_10124_4953 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6233_13205 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6233_13205 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6233_13205 M00967_43_000000000-A3JHG_1_2109_22495_21331 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6233_13205 M00967_43_000000000-A3JHG_1_1110_10946_20858 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19943_26491 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19943_26491 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19943_26491 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19943_26491 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17025_20887 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22959_20262 M00967_43_000000000-A3JHG_1_2112_17281_21576 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9558_19765 M00967_43_000000000-A3JHG_1_1105_18475_22342 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9558_19765 M00967_43_000000000-A3JHG_1_2111_6114_18144 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19149_16619 M00967_43_000000000-A3JHG_1_1112_14780_8078 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_28446_16991 M00967_43_000000000-A3JHG_1_1102_10011_20138 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606 M00967_43_000000000-A3JHG_1_2106_12578_10682 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_1110_10693_14739 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_1103_22959_4225 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19821_2207 M00967_43_000000000-A3JHG_1_1102_14677_6391 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18788_21055 M00967_43_000000000-A3JHG_1_2114_13884_15208 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18788_21055 M00967_43_000000000-A3JHG_1_1104_17535_11802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13258_26764 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13258_26764 M00967_43_000000000-A3JHG_1_2111_19248_7551 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13258_26764 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13258_26764 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13258_26764 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24501_12692 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24501_12692 M00967_43_000000000-A3JHG_1_1113_26736_16107 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_2128_16290 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_2128_16290 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.03488" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_2128_16290 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_2128_16290 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11017_18921 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11017_18921 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11017_18921 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11017_18921 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6751_17524 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6751_17524 M00967_43_000000000-A3JHG_1_1112_10194_20788 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6751_17524 M00967_43_000000000-A3JHG_1_1112_9141_21408 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_6751_17524 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_1110_13879_1994 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.004" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.01195" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10808_11763 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4974_8377 M00967_43_000000000-A3JHG_1_2102_15778_12420 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16200_26616 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16200_26616 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16200_26616 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16200_26616 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16200_26616 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26976_19838 M00967_43_000000000-A3JHG_1_1102_22375_5719 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26976_19838 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26976_19838 M00967_43_000000000-A3JHG_1_1114_19154_8327 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26976_19838 M00967_43_000000000-A3JHG_1_1110_19584_26279 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17368_18443 M00967_43_000000000-A3JHG_1_2102_7041_13746 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9197_9746 M00967_43_000000000-A3JHG_1_1104_13731_17325 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9197_9746 M00967_43_000000000-A3JHG_1_2110_23478_21034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23081_16781 M00967_43_000000000-A3JHG_1_1106_25798_21572 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23081_16781 M00967_43_000000000-A3JHG_1_1112_27414_10900 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23081_16781 M00967_43_000000000-A3JHG_1_1109_11299_13989 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11260_22540 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11260_22540 M00967_43_000000000-A3JHG_1_2108_29278_13914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11260_22540 M00967_43_000000000-A3JHG_1_1112_3066_12463 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20352_16896 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20352_16896 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20352_16896 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18906_15466 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18906_15466 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18906_15466 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18906_15466 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_2293_12053 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_2293_12053 M00967_43_000000000-A3JHG_1_2105_17326_26211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25210_11850 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25210_11850 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25210_11850 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15005_21488 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15005_21488 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15005_21488 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15005_21488 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15005_21488 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8311_8476 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8311_8476 M00967_43_000000000-A3JHG_1_1110_5736_9217 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8311_8476 M00967_43_000000000-A3JHG_1_2108_16551_3455 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27444_10708 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27444_10708 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27444_10708 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27444_10708 M00967_43_000000000-A3JHG_1_1110_10588_13620 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27444_10708 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_2359_11998 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_2359_11998 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_2359_11998 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_2359_11998 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_2359_11998 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22955_21067 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22955_21067 M00967_43_000000000-A3JHG_1_2104_15263_18242 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22477_7326 M00967_43_000000000-A3JHG_1_1105_15878_26239 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22477_7326 M00967_43_000000000-A3JHG_1_2103_7887_6353 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17961_23432 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17961_23432 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17961_23432 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_17961_23432 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12732_20169 M00967_43_000000000-A3JHG_1_1102_14677_6391 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12732_20169 M00967_43_000000000-A3JHG_1_2114_12364_4557 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12732_20169 M00967_43_000000000-A3JHG_1_1114_19821_2207 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_2105_9234_4159 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830 M00967_43_000000000-A3JHG_1_1114_12898_16926 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13819_4746 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13819_4746 M00967_43_000000000-A3JHG_1_1110_5736_9217 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13819_4746 M00967_43_000000000-A3JHG_1_2108_16551_3455 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13819_4746 M00967_43_000000000-A3JHG_1_1103_8311_8476 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16506_20291 M00967_43_000000000-A3JHG_1_1113_20207_16728 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16506_20291 M00967_43_000000000-A3JHG_1_1102_15058_11924 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7782_10984 M00967_43_000000000-A3JHG_1_2112_17281_21576 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7782_10984 M00967_43_000000000-A3JHG_1_1112_16159_23387 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7782_10984 M00967_43_000000000-A3JHG_1_2107_22959_20262 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268 M00967_43_000000000-A3JHG_1_1107_14088_12072 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13414_6546 M00967_43_000000000-A3JHG_1_2107_12720_21761 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13414_6546 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13414_6546 M00967_43_000000000-A3JHG_1_2108_22020_21434 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13414_6546 M00967_43_000000000-A3JHG_1_2105_17326_26211 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11875_9361 M00967_43_000000000-A3JHG_1_2114_21674_11680 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8019_22340 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8019_22340 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8019_22340 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8019_22340 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8019_22340 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19314_26014 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19314_26014 M00967_43_000000000-A3JHG_1_1110_5736_9217 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19314_26014 M00967_43_000000000-A3JHG_1_2108_16551_3455 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19314_26014 M00967_43_000000000-A3JHG_1_1103_8311_8476 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19314_26014 M00967_43_000000000-A3JHG_1_2111_13819_4746 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13356_9691 M00967_43_000000000-A3JHG_1_2104_14359_14505 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11085_20271 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11085_20271 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11085_20271 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11085_20271 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11085_20271 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20096_14197 M00967_43_000000000-A3JHG_1_2109_24829_22616 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1103_5171_14027 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1110_13879_1994 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118 M00967_43_000000000-A3JHG_1_1112_25336_20130 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11302_8835 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11302_8835 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11302_8835 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7943_14658 M00967_43_000000000-A3JHG_1_1105_14835_21938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20224_25526 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20224_25526 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20224_25526 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20224_25526 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14859_14822 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14859_14822 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18687_19607 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18687_19607 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18687_19607 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18687_19607 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18687_19607 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10730_3162 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10730_3162 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10730_3162 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10730_3162 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10730_3162 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25789_8271 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25789_8271 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_25789_8271 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_1107_14088_12072 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23355_23413 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23355_23413 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23355_23413 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23355_23413 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23355_23413 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988 M00967_43_000000000-A3JHG_1_2105_10730_3162 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12503_10753 M00967_43_000000000-A3JHG_1_2111_6887_10490 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12503_10753 M00967_43_000000000-A3JHG_1_1102_17763_13764 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11460_19125 M00967_43_000000000-A3JHG_1_2103_12830_18492 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11460_19125 M00967_43_000000000-A3JHG_1_1110_14183_13016 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11460_19125 M00967_43_000000000-A3JHG_1_1107_17173_6497 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_1107_14088_12072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291 M00967_43_000000000-A3JHG_1_2104_13048_23330 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_24708_11365 M00967_43_000000000-A3JHG_1_1101_13431_10294 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_24708_11365 M00967_43_000000000-A3JHG_1_2109_15602_23047 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_24708_11365 M00967_43_000000000-A3JHG_1_1113_22735_22577 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_24708_11365 M00967_43_000000000-A3JHG_1_1112_10124_4953 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_24708_11365 M00967_43_000000000-A3JHG_1_2109_11446_24358 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_1103_14518_16099 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_2106_16818_12005 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599 M00967_43_000000000-A3JHG_1_1102_18759_22050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19287_23968 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19287_23968 M00967_43_000000000-A3JHG_1_2108_22020_21434 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19287_23968 M00967_43_000000000-A3JHG_1_2105_17326_26211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19287_23968 M00967_43_000000000-A3JHG_1_1106_2293_12053 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466 M00967_43_000000000-A3JHG_1_1101_13714_14338 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28031_16211 M00967_43_000000000-A3JHG_1_1106_25798_21572 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28031_16211 M00967_43_000000000-A3JHG_1_1112_27414_10900 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28031_16211 M00967_43_000000000-A3JHG_1_1109_11299_13989 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28031_16211 M00967_43_000000000-A3JHG_1_1109_23081_16781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21845_13793 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21845_13793 M00967_43_000000000-A3JHG_1_1110_10946_20858 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21845_13793 M00967_43_000000000-A3JHG_1_2108_6233_13205 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14958_19894 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14958_19894 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14958_19894 M00967_43_000000000-A3JHG_1_2101_17535_7182 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14958_19894 M00967_43_000000000-A3JHG_1_1108_14909_4503 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7560_26006 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7560_26006 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7560_26006 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7560_26006 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_2104_17991_24804 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4667_8023 M00967_43_000000000-A3JHG_1_1102_21382_27245 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4667_8023 M00967_43_000000000-A3JHG_1_1105_27543_11258 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4667_8023 M00967_43_000000000-A3JHG_1_2113_16796_6663 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_2106_12578_10682 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19384_17999 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19384_17999 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19384_17999 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12249_8143 M00967_43_000000000-A3JHG_1_1102_14677_6391 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12249_8143 M00967_43_000000000-A3JHG_1_2114_12364_4557 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12249_8143 M00967_43_000000000-A3JHG_1_1101_12732_20169 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17805_7443 M00967_43_000000000-A3JHG_1_1105_14835_21938 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17805_7443 M00967_43_000000000-A3JHG_1_2106_7943_14658 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10420_3770 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10420_3770 M00967_43_000000000-A3JHG_1_2105_17326_26211 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10420_3770 M00967_43_000000000-A3JHG_1_1106_2293_12053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10420_3770 M00967_43_000000000-A3JHG_1_2102_19287_23968 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9752_18363 M00967_43_000000000-A3JHG_1_1102_10011_20138 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9752_18363 M00967_43_000000000-A3JHG_1_2102_28446_16991 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23963_16017 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23963_16017 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23963_16017 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23963_16017 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610 M00967_43_000000000-A3JHG_1_1113_2128_16290 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11559_26077 M00967_43_000000000-A3JHG_1_1101_9331_5806 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1114_12898_16926 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24903_5978 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24903_5978 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24903_5978 M00967_43_000000000-A3JHG_1_1108_27354_15573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24903_5978 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22280_22887 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22280_22887 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14488_13179 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14488_13179 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14488_13179 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_14488_13179 M00967_43_000000000-A3JHG_1_1105_25210_11850 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_4106_18467 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_4106_18467 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_4106_18467 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_4106_18467 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_4106_18467 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11497_20872 M00967_43_000000000-A3JHG_1_2111_6887_10490 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11497_20872 M00967_43_000000000-A3JHG_1_1102_17763_13764 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11497_20872 M00967_43_000000000-A3JHG_1_2103_12503_10753 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7070_12571 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7070_12571 M00967_43_000000000-A3JHG_1_1113_26736_16107 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7070_12571 M00967_43_000000000-A3JHG_1_1113_24501_12692 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7913_12679 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7913_12679 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7913_12679 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7913_12679 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26542_17589 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26542_17589 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26542_17589 M00967_43_000000000-A3JHG_1_1110_10033_25797 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26542_17589 M00967_43_000000000-A3JHG_1_2101_7254_13419 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13411_22875 M00967_43_000000000-A3JHG_1_1109_22380_20532 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.04" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478 M00967_43_000000000-A3JHG_1_1114_22804_12324 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478 M00967_43_000000000-A3JHG_1_2113_11757_15493 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478 M00967_43_000000000-A3JHG_1_1110_10693_14739 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14299_17220 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14299_17220 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14299_17220 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14299_17220 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9649_19293 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9649_19293 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9649_19293 M00967_43_000000000-A3JHG_1_1114_12898_16926 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9649_19293 M00967_43_000000000-A3JHG_1_1107_7195_18281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17680_15930 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17680_15930 M00967_43_000000000-A3JHG_1_1110_10033_25797 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17680_15930 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17680_15930 M00967_43_000000000-A3JHG_1_2101_7254_13419 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17680_15930 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28102_18945 M00967_43_000000000-A3JHG_1_2103_12830_18492 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28102_18945 M00967_43_000000000-A3JHG_1_1110_14183_13016 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28102_18945 M00967_43_000000000-A3JHG_1_1107_17173_6497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_28102_18945 M00967_43_000000000-A3JHG_1_2103_11460_19125 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9766_20824 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7083_22842 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7083_22842 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7083_22842 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7083_22842 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24996_25271 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24996_25271 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24996_25271 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24996_25271 M00967_43_000000000-A3JHG_1_1103_7913_12679 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23848_13201 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23848_13201 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23848_13201 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23848_13201 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21662_22858 M00967_43_000000000-A3JHG_1_2105_20695_7710 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21662_22858 M00967_43_000000000-A3JHG_1_2109_21713_13835 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_2113_24913_16194 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_2104_13048_23330 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682 M00967_43_000000000-A3JHG_1_2102_8893_14291 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920 M00967_43_000000000-A3JHG_1_1101_17961_23432 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19140_21811 M00967_43_000000000-A3JHG_1_1110_20816_14150 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16325_18649 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16325_18649 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16325_18649 M00967_43_000000000-A3JHG_1_1106_9232_9214 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16325_18649 M00967_43_000000000-A3JHG_1_1106_8303_10880 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10607_6385 M00967_43_000000000-A3JHG_1_2103_21597_21834 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10607_6385 M00967_43_000000000-A3JHG_1_2106_14094_18415 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10607_6385 M00967_43_000000000-A3JHG_1_1110_15335_17802 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10607_6385 M00967_43_000000000-A3JHG_1_1108_7641_20331 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10607_6385 M00967_43_000000000-A3JHG_1_2112_20183_27184 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_2104_13048_23330 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_2102_8893_14291 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486 M00967_43_000000000-A3JHG_1_2112_25988_9682 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20493_16450 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20493_16450 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20493_16450 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20493_16450 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20493_16450 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_2111_20158_16824 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_2112_11899_4014 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888 M00967_43_000000000-A3JHG_1_1101_17961_23432 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20871_13331 M00967_43_000000000-A3JHG_1_1103_22959_4225 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14743_23547 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14743_23547 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14743_23547 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_14743_23547 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902 M00967_43_000000000-A3JHG_1_1105_4106_18467 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9090_14716 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22380_17554 M00967_43_000000000-A3JHG_1_2111_8444_24070 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10074_5728 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10074_5728 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10074_5728 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10074_5728 M00967_43_000000000-A3JHG_1_1107_2468_14102 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10074_5728 M00967_43_000000000-A3JHG_1_1110_15463_23056 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1113_2128_16290 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13953_7091 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13953_7091 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13953_7091 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13953_7091 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5207_13479 M00967_43_000000000-A3JHG_1_1110_12295_19699 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395 M00967_43_000000000-A3JHG_1_2101_13130_17466 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17730_26068 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17730_26068 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17730_26068 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17730_26068 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17730_26068 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7150_17375 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7150_17375 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6303_20947 M00967_43_000000000-A3JHG_1_2103_18975_9767 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11788_5032 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11788_5032 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11788_5032 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11788_5032 M00967_43_000000000-A3JHG_1_1105_25210_11850 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11788_5032 M00967_43_000000000-A3JHG_1_1107_14488_13179 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504 M00967_43_000000000-A3JHG_1_1105_4106_18467 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14398_12314 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14398_12314 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14398_12314 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14398_12314 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14398_12314 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292 M00967_43_000000000-A3JHG_1_2111_19248_7551 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10298_23586 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10298_23586 M00967_43_000000000-A3JHG_1_1112_10194_20788 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10298_23586 M00967_43_000000000-A3JHG_1_1112_9141_21408 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10298_23586 M00967_43_000000000-A3JHG_1_1112_6751_17524 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26230_18106 M00967_43_000000000-A3JHG_1_1101_11666_7824 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26230_18106 M00967_43_000000000-A3JHG_1_1114_5902_6841 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26230_18106 M00967_43_000000000-A3JHG_1_1113_23401_8348 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19468_25966 M00967_43_000000000-A3JHG_1_1102_22375_5719 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19468_25966 M00967_43_000000000-A3JHG_1_1110_19584_26279 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_19468_25966 M00967_43_000000000-A3JHG_1_1110_26976_19838 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25457_17518 M00967_43_000000000-A3JHG_1_1106_17565_8490 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19250_12721 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19250_12721 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19250_12721 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18362_23908 M00967_43_000000000-A3JHG_1_1109_9629_6769 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18362_23908 M00967_43_000000000-A3JHG_1_2108_5077_12621 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18362_23908 M00967_43_000000000-A3JHG_1_1105_23612_18072 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13402_18823 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13402_18823 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13402_18823 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13402_18823 M00967_43_000000000-A3JHG_1_2113_24996_25271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27199_15268 M00967_43_000000000-A3JHG_1_1113_6044_19045 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27199_15268 M00967_43_000000000-A3JHG_1_2108_19309_25091 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_1107_14088_12072 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2104_13048_23330 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2102_8893_14291 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2112_25988_9682 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18001_27099 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18001_27099 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18001_27099 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_13721_7517 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_13721_7517 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_13721_7517 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_13721_7517 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_13721_7517 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2104_13048_23330 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2112_25988_9682 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468 M00967_43_000000000-A3JHG_1_1107_26400_19526 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9515_26311 M00967_43_000000000-A3JHG_1_1106_24475_12056 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9515_26311 M00967_43_000000000-A3JHG_1_1110_18950_6153 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19059_19452 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19059_19452 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742 M00967_43_000000000-A3JHG_1_1106_13721_7517 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11335_21750 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11335_21750 M00967_43_000000000-A3JHG_1_1113_4318_16941 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11335_21750 M00967_43_000000000-A3JHG_1_1101_13714_14338 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10830_22855 M00967_43_000000000-A3JHG_1_1104_13731_17325 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10830_22855 M00967_43_000000000-A3JHG_1_2110_23478_21034 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10830_22855 M00967_43_000000000-A3JHG_1_1105_13597_20997 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_10830_22855 M00967_43_000000000-A3JHG_1_1109_9197_9746 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8524_16122 M00967_43_000000000-A3JHG_1_1108_11048_4561 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5327_17123 M00967_43_000000000-A3JHG_1_1112_10024_21424 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_5327_17123 M00967_43_000000000-A3JHG_1_1114_21549_4010 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504 M00967_43_000000000-A3JHG_1_1106_18001_27099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4346_14940 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4346_14940 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11754_11390 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11754_11390 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11754_11390 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11754_11390 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11754_11390 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539 M00967_43_000000000-A3JHG_1_2105_19668_20827 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_22245_24849 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_22245_24849 M00967_43_000000000-A3JHG_1_2105_9234_4159 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_22245_24849 M00967_43_000000000-A3JHG_1_2105_22135_19871 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_22245_24849 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_22245_24849 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7562_15957 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7562_15957 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7562_15957 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_7562_15957 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12184_22923 M00967_43_000000000-A3JHG_1_1104_13731_17325 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12184_22923 M00967_43_000000000-A3JHG_1_2110_23478_21034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12184_22923 M00967_43_000000000-A3JHG_1_1109_9197_9746 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_12184_22923 M00967_43_000000000-A3JHG_1_1103_10830_22855 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_2105_10730_3162 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16686_25858 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16686_25858 M00967_43_000000000-A3JHG_1_2108_29278_13914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16686_25858 M00967_43_000000000-A3JHG_1_1112_3066_12463 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16686_25858 M00967_43_000000000-A3JHG_1_1109_11260_22540 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16230_18061 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16230_18061 M00967_43_000000000-A3JHG_1_1101_3758_10627 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16230_18061 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808 M00967_43_000000000-A3JHG_1_1103_7913_12679 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808 M00967_43_000000000-A3JHG_1_2113_24996_25271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808 M00967_43_000000000-A3JHG_1_1108_13402_18823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_28218_15620 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24635_10372 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24635_10372 M00967_43_000000000-A3JHG_1_1113_4318_16941 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24635_10372 M00967_43_000000000-A3JHG_1_1101_13714_14338 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24635_10372 M00967_43_000000000-A3JHG_1_1104_11335_21750 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24618_11376 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24618_11376 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24618_11376 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24618_11376 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24618_11376 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24274_20575 M00967_43_000000000-A3JHG_1_2112_17281_21576 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24274_20575 M00967_43_000000000-A3JHG_1_1112_16159_23387 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24274_20575 M00967_43_000000000-A3JHG_1_2107_22959_20262 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24274_20575 M00967_43_000000000-A3JHG_1_1104_7782_10984 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530 M00967_43_000000000-A3JHG_1_1104_11751_17981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18566_5886 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18566_5886 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18566_5886 M00967_43_000000000-A3JHG_1_1110_10588_13620 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18566_5886 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18566_5886 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_1107_2468_14102 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_1110_15463_23056 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_1105_25410_11631 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462 M00967_43_000000000-A3JHG_1_2103_10074_5728 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20783_5301 M00967_43_000000000-A3JHG_1_2111_5740_11365 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18673_6259 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18673_6259 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18673_6259 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18673_6259 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18673_6259 M00967_43_000000000-A3JHG_1_1112_21256_12063 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_11145_23790 M00967_43_000000000-A3JHG_1_1110_9258_26812 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1114_12898_16926 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1107_7195_18281 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256 M00967_43_000000000-A3JHG_1_1109_9649_19293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13463_16561 M00967_43_000000000-A3JHG_1_2103_18975_9767 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13463_16561 M00967_43_000000000-A3JHG_1_1113_6303_20947 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_1105_13597_20997 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383 M00967_43_000000000-A3JHG_1_2104_15288_13724 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6981_7811 M00967_43_000000000-A3JHG_1_1101_11856_9674 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6981_7811 M00967_43_000000000-A3JHG_1_1108_20070_20418 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6981_7811 M00967_43_000000000-A3JHG_1_2101_13466_27462 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_6981_7811 M00967_43_000000000-A3JHG_1_1101_12695_21368 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25701_15655 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25701_15655 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25701_15655 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081 M00967_43_000000000-A3JHG_1_1105_25210_11850 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081 M00967_43_000000000-A3JHG_1_1107_14488_13179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081 M00967_43_000000000-A3JHG_1_1112_11788_5032 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15422_19940 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15422_19940 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15422_19940 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15422_19940 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15422_19940 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10993_5398 M00967_43_000000000-A3JHG_1_1102_10011_20138 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10993_5398 M00967_43_000000000-A3JHG_1_2102_28446_16991 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10993_5398 M00967_43_000000000-A3JHG_1_1109_9752_18363 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16565_10623 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16565_10623 M00967_43_000000000-A3JHG_1_2106_27160_14501 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16565_10623 M00967_43_000000000-A3JHG_1_2103_20113_16810 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16565_10623 M00967_43_000000000-A3JHG_1_1105_15636_27192 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328 M00967_43_000000000-A3JHG_1_1101_13431_10294 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328 M00967_43_000000000-A3JHG_1_2109_15602_23047 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328 M00967_43_000000000-A3JHG_1_1113_22735_22577 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328 M00967_43_000000000-A3JHG_1_1112_10124_4953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328 M00967_43_000000000-A3JHG_1_2109_11446_24358 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328 M00967_43_000000000-A3JHG_1_2102_24708_11365 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1106_23697_7097 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7207_7404 M00967_43_000000000-A3JHG_1_1106_17565_8490 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7207_7404 M00967_43_000000000-A3JHG_1_1108_25457_17518 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2104_17991_24804 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20718_23357 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20718_23357 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20718_23357 M00967_43_000000000-A3JHG_1_1109_12129_4985 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20718_23357 M00967_43_000000000-A3JHG_1_1108_10911_24264 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_2104_17991_24804 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1106_23697_7097 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1106_23697_7097 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_1105_13597_20997 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17671_25995 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17671_25995 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17671_25995 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17671_25995 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17671_25995 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10211_20812 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9368_14850 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9368_14850 M00967_43_000000000-A3JHG_1_2101_13130_17466 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9368_14850 M00967_43_000000000-A3JHG_1_1114_23025_24395 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_27637_18457 M00967_43_000000000-A3JHG_1_1110_12738_11413 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_27637_18457 M00967_43_000000000-A3JHG_1_1105_15675_15142 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294 M00967_43_000000000-A3JHG_1_1101_18693_22396 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21648_16922 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21648_16922 M00967_43_000000000-A3JHG_1_2110_19140_21811 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21648_16922 M00967_43_000000000-A3JHG_1_2113_11145_23790 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21648_16922 M00967_43_000000000-A3JHG_1_2105_10211_20812 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18926_9248 M00967_43_000000000-A3JHG_1_1102_21382_27245 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18926_9248 M00967_43_000000000-A3JHG_1_1105_27543_11258 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18926_9248 M00967_43_000000000-A3JHG_1_1111_4667_8023 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7671_11355 M00967_43_000000000-A3JHG_1_2105_9234_4159 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7671_11355 M00967_43_000000000-A3JHG_1_2105_22135_19871 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7671_11355 M00967_43_000000000-A3JHG_1_2109_22245_24849 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22381_10329 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22381_10329 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22381_10329 M00967_43_000000000-A3JHG_1_1106_8303_10880 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22381_10329 M00967_43_000000000-A3JHG_1_2110_16325_18649 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697 M00967_43_000000000-A3JHG_1_2104_21630_21997 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10627_9330 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10627_9330 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10627_9330 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10627_9330 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10627_9330 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10158_13191 M00967_43_000000000-A3JHG_1_2104_15263_18242 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10158_13191 M00967_43_000000000-A3JHG_1_1102_22955_21067 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290 M00967_43_000000000-A3JHG_1_2107_6821_13209 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290 M00967_43_000000000-A3JHG_1_2108_29278_13914 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290 M00967_43_000000000-A3JHG_1_1112_3066_12463 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290 M00967_43_000000000-A3JHG_1_1109_11260_22540 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290 M00967_43_000000000-A3JHG_1_1112_16686_25858 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_28033_9425 M00967_43_000000000-A3JHG_1_1112_14868_10194 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_1108_10911_24264 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_1107_7195_18281 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191 M00967_43_000000000-A3JHG_1_2107_20718_23357 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_2112_11899_4014 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1113_2128_16290 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182 M00967_43_000000000-A3JHG_1_2102_16879_22327 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12081_2719 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12081_2719 M00967_43_000000000-A3JHG_1_1113_26736_16107 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12081_2719 M00967_43_000000000-A3JHG_1_1113_24501_12692 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12081_2719 M00967_43_000000000-A3JHG_1_1104_7070_12571 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636 M00967_43_000000000-A3JHG_1_1114_24119_8383 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_1106_23697_7097 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2105_10730_3162 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524 M00967_43_000000000-A3JHG_1_1104_10943_19850 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524 M00967_43_000000000-A3JHG_1_2113_13473_22310 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524 M00967_43_000000000-A3JHG_1_1114_13545_27181 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524 M00967_43_000000000-A3JHG_1_1111_18224_14666 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524 M00967_43_000000000-A3JHG_1_2103_16319_7131 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1114_10011_16636 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1113_27906_8733 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8225_24102 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8225_24102 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8225_24102 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8225_24102 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781 M00967_43_000000000-A3JHG_1_2104_6580_11453 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781 M00967_43_000000000-A3JHG_1_2108_29278_13914 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781 M00967_43_000000000-A3JHG_1_1112_3066_12463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781 M00967_43_000000000-A3JHG_1_1109_11260_22540 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781 M00967_43_000000000-A3JHG_1_1112_16686_25858 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781 M00967_43_000000000-A3JHG_1_2101_6613_14290 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_2101_9084_14502 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_1111_12899_5675 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369 M00967_43_000000000-A3JHG_1_1104_5274_12644 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1114_16544_20390 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2112_22673_23252 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21900_16751 M00967_43_000000000-A3JHG_1_1109_19691_19130 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4865_20136 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4865_20136 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4865_20136 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_4865_20136 M00967_43_000000000-A3JHG_1_1113_15332_23431 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227 M00967_43_000000000-A3JHG_1_1112_24992_23873 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725 M00967_43_000000000-A3JHG_1_1112_21256_12063 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725 M00967_43_000000000-A3JHG_1_2113_18673_6259 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2105_10730_3162 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2103_23519_12843 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14282_18817 M00967_43_000000000-A3JHG_1_2101_17475_7154 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10328_10718 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10328_10718 M00967_43_000000000-A3JHG_1_1110_10693_14739 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10328_10718 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10328_10718 M00967_43_000000000-A3JHG_1_2101_7254_13419 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18879_6171 M00967_43_000000000-A3JHG_1_1106_22417_19200 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18879_6171 M00967_43_000000000-A3JHG_1_1108_18996_9436 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_18879_6171 M00967_43_000000000-A3JHG_1_1104_12222_5327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1114_24119_8383 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1114_10011_16636 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1113_27906_8733 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10941_17222 M00967_43_000000000-A3JHG_1_1104_10875_20179 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_10941_17222 M00967_43_000000000-A3JHG_1_1112_9549_5158 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9868_4758 M00967_43_000000000-A3JHG_1_1113_20207_16728 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9868_4758 M00967_43_000000000-A3JHG_1_1102_15058_11924 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9868_4758 M00967_43_000000000-A3JHG_1_1110_16506_20291 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20465_8611 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20465_8611 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20465_8611 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20465_8611 M00967_43_000000000-A3JHG_1_1107_24903_5978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751 M00967_43_000000000-A3JHG_1_2106_14859_14822 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_9875_8400 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_9875_8400 M00967_43_000000000-A3JHG_1_1113_26736_16107 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_9875_8400 M00967_43_000000000-A3JHG_1_1113_24501_12692 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_9875_8400 M00967_43_000000000-A3JHG_1_1104_7070_12571 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_9875_8400 M00967_43_000000000-A3JHG_1_1114_12081_2719 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6519_20940 M00967_43_000000000-A3JHG_1_1113_28366_19519 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5458_8163 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5458_8163 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5458_8163 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_5458_8163 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264 M00967_43_000000000-A3JHG_1_2114_24618_11376 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27702_14915 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27702_14915 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27702_14915 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_2114_18566_5886 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9133_12955 M00967_43_000000000-A3JHG_1_2114_13884_15208 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9133_12955 M00967_43_000000000-A3JHG_1_1104_17535_11802 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9133_12955 M00967_43_000000000-A3JHG_1_1114_18788_21055 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_2111_20158_16824 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_2112_11899_4014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136 M00967_43_000000000-A3JHG_1_2102_16879_22327 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1106_23697_7097 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1113_15332_23431 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1114_24119_8383 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1114_10011_16636 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_5922_6513 M00967_43_000000000-A3JHG_1_1112_14780_8078 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_5922_6513 M00967_43_000000000-A3JHG_1_2103_19149_16619 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_2111_20158_16824 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_2112_11899_4014 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_2102_16879_22327 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638 M00967_43_000000000-A3JHG_1_1103_14890_4136 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15129_15739 M00967_43_000000000-A3JHG_1_1104_7522_5556 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15129_15739 M00967_43_000000000-A3JHG_1_1112_20884_13058 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_1101_13714_14338 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_2101_13130_17466 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848 M00967_43_000000000-A3JHG_1_1114_23025_24395 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910 M00967_43_000000000-A3JHG_1_1114_10055_21108 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1105_19799_18751 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14273_14388 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14273_14388 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14273_14388 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14273_14388 M00967_43_000000000-A3JHG_1_1104_27702_14915 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_1107_2468_14102 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_1110_15463_23056 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_1105_25410_11631 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_2103_10074_5728 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609 M00967_43_000000000-A3JHG_1_2114_16615_9462 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_1114_10055_21108 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702 M00967_43_000000000-A3JHG_1_1103_23750_17910 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2104_15288_13724 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515 M00967_43_000000000-A3JHG_1_2106_17976_10139 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1108_4865_20136 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_2111_19248_7551 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646 M00967_43_000000000-A3JHG_1_1110_15924_11287 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767 M00967_43_000000000-A3JHG_1_1106_9232_9214 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767 M00967_43_000000000-A3JHG_1_1106_8303_10880 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767 M00967_43_000000000-A3JHG_1_2110_16325_18649 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767 M00967_43_000000000-A3JHG_1_2102_22381_10329 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6888_8092 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6888_8092 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6888_8092 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6888_8092 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_1105_4106_18467 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965 M00967_43_000000000-A3JHG_1_1111_18784_23504 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320 M00967_43_000000000-A3JHG_1_1107_12430_16983 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844 M00967_43_000000000-A3JHG_1_1108_19250_12721 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10394_17608 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10394_17608 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10394_17608 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10394_17608 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18874_21200 M00967_43_000000000-A3JHG_1_1112_14780_8078 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18874_21200 M00967_43_000000000-A3JHG_1_2103_19149_16619 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18874_21200 M00967_43_000000000-A3JHG_1_1101_5922_6513 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1106_13721_7517 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1105_11983_14742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_2111_15422_19940 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01195" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18354_8608 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18354_8608 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18354_8608 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18354_8608 M00967_43_000000000-A3JHG_1_1107_24903_5978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18354_8608 M00967_43_000000000-A3JHG_1_1105_20465_8611 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03101" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21901_10432 M00967_43_000000000-A3JHG_1_2114_21674_11680 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16795_27764 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16795_27764 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16795_27764 M00967_43_000000000-A3JHG_1_1113_7150_17375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16795_27764 M00967_43_000000000-A3JHG_1_1103_10830_22855 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_9710_20744 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_9710_20744 M00967_43_000000000-A3JHG_1_1110_10588_13620 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_9710_20744 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_9710_20744 M00967_43_000000000-A3JHG_1_2114_18566_5886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14512_24313 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14512_24313 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14512_24313 M00967_43_000000000-A3JHG_1_1108_19250_12721 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14512_24313 M00967_43_000000000-A3JHG_1_2109_18345_18844 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14252_8710 M00967_43_000000000-A3JHG_1_1109_22380_20532 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14252_8710 M00967_43_000000000-A3JHG_1_1103_13411_22875 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1106_12555_24141 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_2112_11899_4014 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1103_14890_4136 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616 M00967_43_000000000-A3JHG_1_1101_23714_7638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416 M00967_43_000000000-A3JHG_1_1107_24903_5978 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416 M00967_43_000000000-A3JHG_1_1105_20465_8611 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416 M00967_43_000000000-A3JHG_1_2107_18354_8608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12402_11394 M00967_43_000000000-A3JHG_1_1110_17653_8119 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27837_14578 M00967_43_000000000-A3JHG_1_1105_14835_21938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27837_14578 M00967_43_000000000-A3JHG_1_2106_7943_14658 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27837_14578 M00967_43_000000000-A3JHG_1_1110_17805_7443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837 M00967_43_000000000-A3JHG_1_2107_13016_9270 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16110_14718 M00967_43_000000000-A3JHG_1_1106_25798_21572 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16110_14718 M00967_43_000000000-A3JHG_1_1112_27414_10900 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16110_14718 M00967_43_000000000-A3JHG_1_1109_11299_13989 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16110_14718 M00967_43_000000000-A3JHG_1_1109_23081_16781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16110_14718 M00967_43_000000000-A3JHG_1_1114_28031_16211 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_2114_24618_11376 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189 M00967_43_000000000-A3JHG_1_1104_29281_13264 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_2106_16818_12005 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_1102_18759_22050 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718 M00967_43_000000000-A3JHG_1_2102_21816_16599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_2104_21630_21997 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670 M00967_43_000000000-A3JHG_1_2102_15622_9697 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17550_17269 M00967_43_000000000-A3JHG_1_2106_20458_8348 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26069_24105 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26069_24105 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26069_24105 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26069_24105 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24287_19984 M00967_43_000000000-A3JHG_1_1102_27488_14514 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_24287_19984 M00967_43_000000000-A3JHG_1_1103_11725_18144 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17713_26784 M00967_43_000000000-A3JHG_1_1107_3735_12375 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17713_26784 M00967_43_000000000-A3JHG_1_2103_15381_10377 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17713_26784 M00967_43_000000000-A3JHG_1_1113_13777_28036 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9215_20566 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9215_20566 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9215_20566 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9215_20566 M00967_43_000000000-A3JHG_1_1106_17680_15930 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_2102_10683_16772 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.01195" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11020_6991 M00967_43_000000000-A3JHG_1_1101_11666_7824 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11020_6991 M00967_43_000000000-A3JHG_1_1114_5902_6841 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11020_6991 M00967_43_000000000-A3JHG_1_1113_23401_8348 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11020_6991 M00967_43_000000000-A3JHG_1_1109_26230_18106 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_2102_10683_16772 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801 M00967_43_000000000-A3JHG_1_2113_18673_6259 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801 M00967_43_000000000-A3JHG_1_1108_24102_4725 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279 M00967_43_000000000-A3JHG_1_1113_26736_16107 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279 M00967_43_000000000-A3JHG_1_1113_24501_12692 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279 M00967_43_000000000-A3JHG_1_1104_7070_12571 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279 M00967_43_000000000-A3JHG_1_1114_12081_2719 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279 M00967_43_000000000-A3JHG_1_1104_9875_8400 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_1105_25210_11850 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_1107_14488_13179 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_1112_11788_5032 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535 M00967_43_000000000-A3JHG_1_2111_15720_29081 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_2101_9084_14502 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_1111_12899_5675 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_1104_5274_12644 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799 M00967_43_000000000-A3JHG_1_1110_19653_28369 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19777_24988 M00967_43_000000000-A3JHG_1_2114_21674_11680 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19777_24988 M00967_43_000000000-A3JHG_1_2113_11875_9361 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19777_24988 M00967_43_000000000-A3JHG_1_2105_21901_10432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15151_22654 M00967_43_000000000-A3JHG_1_1101_11666_7824 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15151_22654 M00967_43_000000000-A3JHG_1_1114_5902_6841 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15151_22654 M00967_43_000000000-A3JHG_1_1113_23401_8348 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15151_22654 M00967_43_000000000-A3JHG_1_1109_26230_18106 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15151_22654 M00967_43_000000000-A3JHG_1_1109_11020_6991 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_1105_4106_18467 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_1111_18784_23504 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772 M00967_43_000000000-A3JHG_1_2111_24817_7965 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_2113_18673_6259 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_1108_24102_4725 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407 M00967_43_000000000-A3JHG_1_1107_5528_8801 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1108_4865_20136 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_1102_13690_6672 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_2107_6956_16009 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_2101_27710_20218 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_1107_2468_14102 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_1110_15463_23056 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_1105_25410_11631 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_2103_10074_5728 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_2114_16615_9462 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270 M00967_43_000000000-A3JHG_1_2114_10056_13609 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7754_25974 M00967_43_000000000-A3JHG_1_2112_17281_21576 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7754_25974 M00967_43_000000000-A3JHG_1_1112_16159_23387 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7754_25974 M00967_43_000000000-A3JHG_1_2107_22959_20262 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7754_25974 M00967_43_000000000-A3JHG_1_1104_7782_10984 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7754_25974 M00967_43_000000000-A3JHG_1_2114_24274_20575 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041 M00967_43_000000000-A3JHG_1_1109_15598_4401 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23908_15902 M00967_43_000000000-A3JHG_1_2111_14170_11158 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23908_15902 M00967_43_000000000-A3JHG_1_2110_8189_7507 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_20010_17589 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_20010_17589 M00967_43_000000000-A3JHG_1_1110_9258_26812 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_20010_17589 M00967_43_000000000-A3JHG_1_2105_10211_20812 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_20010_17589 M00967_43_000000000-A3JHG_1_2103_21648_16922 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_20010_17589 M00967_43_000000000-A3JHG_1_2102_10158_13191 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2111_15422_19940 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_1114_10055_21108 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_1103_23750_17910 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065 M00967_43_000000000-A3JHG_1_2113_8991_5702 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02326" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880 M00967_43_000000000-A3JHG_1_2107_26555_14955 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880 M00967_43_000000000-A3JHG_1_2106_6909_9337 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_2102_10683_16772 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1108_15844_3660 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2107_23831_12272 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1108_10730_20249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1107_7008_19978 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1107_14088_12072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2104_13048_23330 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2102_8893_14291 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2112_25988_9682 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448 M00967_43_000000000-A3JHG_1_1107_26400_19526 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1109_12129_4985 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2110_8933_4869 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_1101_17961_23432 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2106_4477_9888 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982 M00967_43_000000000-A3JHG_1_2102_10627_9330 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1112_7560_26006 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26969_15672 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26969_15672 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26969_15672 M00967_43_000000000-A3JHG_1_2101_17535_7182 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26969_15672 M00967_43_000000000-A3JHG_1_1108_14909_4503 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26969_15672 M00967_43_000000000-A3JHG_1_1114_14958_19894 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2102_10683_16772 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1114_10011_16636 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1110_18344_10035 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1113_26736_16107 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1113_24501_12692 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1104_7070_12571 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1114_12081_2719 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1104_9875_8400 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772 M00967_43_000000000-A3JHG_1_1107_24891_17279 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11463_19602 M00967_43_000000000-A3JHG_1_1112_10024_21424 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11463_19602 M00967_43_000000000-A3JHG_1_1114_21549_4010 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11463_19602 M00967_43_000000000-A3JHG_1_1102_5327_17123 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_2104_21630_21997 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_2102_15622_9697 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567 M00967_43_000000000-A3JHG_1_1112_11370_15670 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_1107_12430_16983 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467 M00967_43_000000000-A3JHG_1_2109_19574_4320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_1110_15924_11287 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921 M00967_43_000000000-A3JHG_1_2113_25465_9646 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.012" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256 M00967_43_000000000-A3JHG_1_1113_28366_19519 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28168_19224 M00967_43_000000000-A3JHG_1_1106_17565_8490 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28168_19224 M00967_43_000000000-A3JHG_1_1108_25457_17518 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28168_19224 M00967_43_000000000-A3JHG_1_2107_7207_7404 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_2102_10627_9330 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679 M00967_43_000000000-A3JHG_1_2112_9811_9982 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1113_27906_8733 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1105_19799_18751 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12440_13627 M00967_43_000000000-A3JHG_1_1102_10011_20138 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12440_13627 M00967_43_000000000-A3JHG_1_2102_28446_16991 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12440_13627 M00967_43_000000000-A3JHG_1_1109_9752_18363 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_12440_13627 M00967_43_000000000-A3JHG_1_2111_10993_5398 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1106_13721_7517 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1105_11983_14742 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690 M00967_43_000000000-A3JHG_1_2109_20708_17882 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_2112_7837_7502 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586 M00967_43_000000000-A3JHG_1_1104_11494_18542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_1109_15598_4401 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028 M00967_43_000000000-A3JHG_1_1103_24460_16041 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20074_25538 M00967_43_000000000-A3JHG_1_1103_13411_22875 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2111_15422_19940 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_1101_25551_18277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2109_20708_17882 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559 M00967_43_000000000-A3JHG_1_2105_15118_19690 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2114_12741_23025 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18298_21125 M00967_43_000000000-A3JHG_1_1106_24475_12056 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18298_21125 M00967_43_000000000-A3JHG_1_1110_18950_6153 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18298_21125 M00967_43_000000000-A3JHG_1_1105_9515_26311 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16703_5011 M00967_43_000000000-A3JHG_1_1113_10719_18379 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2102_10683_16772 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_21947_18735 M00967_43_000000000-A3JHG_1_1108_11048_4561 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_21947_18735 M00967_43_000000000-A3JHG_1_1102_8524_16122 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404 M00967_43_000000000-A3JHG_1_2101_8442_23370 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_1109_15598_4401 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_1103_24460_16041 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776 M00967_43_000000000-A3JHG_1_2104_23298_12028 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7676_25736 M00967_43_000000000-A3JHG_1_1109_15029_15883 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_7676_25736 M00967_43_000000000-A3JHG_1_1112_19666_21477 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22942_23318 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22942_23318 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22942_23318 M00967_43_000000000-A3JHG_1_2101_7254_13419 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22942_23318 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22942_23318 M00967_43_000000000-A3JHG_1_1108_10328_10718 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20440_13060 M00967_43_000000000-A3JHG_1_2104_12505_3424 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_20440_13060 M00967_43_000000000-A3JHG_1_1103_5754_14689 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_1101_17961_23432 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_2106_4477_9888 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20503_21801 M00967_43_000000000-A3JHG_1_1102_10011_20138 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20503_21801 M00967_43_000000000-A3JHG_1_2102_28446_16991 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20503_21801 M00967_43_000000000-A3JHG_1_1109_9752_18363 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20503_21801 M00967_43_000000000-A3JHG_1_2111_10993_5398 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20503_21801 M00967_43_000000000-A3JHG_1_2106_12440_13627 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24714_23272 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24714_23272 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24714_23272 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24714_23272 M00967_43_000000000-A3JHG_1_1104_27702_14915 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24714_23272 M00967_43_000000000-A3JHG_1_2114_14273_14388 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20972_19668 M00967_43_000000000-A3JHG_1_1107_13192_19802 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673 M00967_43_000000000-A3JHG_1_2106_25956_17550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_14865_2720 M00967_43_000000000-A3JHG_1_2111_22131_23386 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.01556" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01556" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6400_18545 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6400_18545 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6400_18545 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6400_18545 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6250_5987 M00967_43_000000000-A3JHG_1_1110_9258_26812 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6250_5987 M00967_43_000000000-A3JHG_1_2113_11145_23790 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_6250_5987 M00967_43_000000000-A3JHG_1_1103_20010_17589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_2106_14859_14822 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1105_19799_18751 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19891_5082 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19891_5082 M00967_43_000000000-A3JHG_1_2106_27160_14501 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19891_5082 M00967_43_000000000-A3JHG_1_2103_20113_16810 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19891_5082 M00967_43_000000000-A3JHG_1_1105_15636_27192 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19891_5082 M00967_43_000000000-A3JHG_1_2110_16565_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523 M00967_43_000000000-A3JHG_1_1113_13105_21336 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2104_17991_24804 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.04264" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541 M00967_43_000000000-A3JHG_1_2101_17535_7182 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541 M00967_43_000000000-A3JHG_1_1108_14909_4503 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541 M00967_43_000000000-A3JHG_1_1114_14958_19894 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541 M00967_43_000000000-A3JHG_1_2112_26969_15672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18844_27717 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18844_27717 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16853_2840 M00967_43_000000000-A3JHG_1_2111_6887_10490 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16853_2840 M00967_43_000000000-A3JHG_1_1102_17763_13764 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16853_2840 M00967_43_000000000-A3JHG_1_2103_12503_10753 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163 M00967_43_000000000-A3JHG_1_2114_14273_14388 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163 M00967_43_000000000-A3JHG_1_1112_24714_23272 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329 M00967_43_000000000-A3JHG_1_2109_20708_17882 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1113_13105_21336 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850 M00967_43_000000000-A3JHG_1_1111_15521_13523 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24109_8015 M00967_43_000000000-A3JHG_1_1104_7522_5556 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24109_8015 M00967_43_000000000-A3JHG_1_1112_20884_13058 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24109_8015 M00967_43_000000000-A3JHG_1_1111_15129_15739 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2644_15119 M00967_43_000000000-A3JHG_1_1104_7522_5556 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2644_15119 M00967_43_000000000-A3JHG_1_1112_20884_13058 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2644_15119 M00967_43_000000000-A3JHG_1_1111_15129_15739 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2644_15119 M00967_43_000000000-A3JHG_1_1109_24109_8015 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1108_4865_20136 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1104_12675_27249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1101_26020_8626 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1101_17961_23432 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2106_4477_9888 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528 M00967_43_000000000-A3JHG_1_1114_10373_6011 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2106_25337_11987 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1113_13105_21336 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1111_15521_13523 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687 M00967_43_000000000-A3JHG_1_1109_8925_8850 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_1105_4106_18467 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_1111_18784_23504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_2111_24817_7965 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856 M00967_43_000000000-A3JHG_1_1105_9374_22772 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10061_3691 M00967_43_000000000-A3JHG_1_1111_22890_9747 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10061_3691 M00967_43_000000000-A3JHG_1_1109_14359_12484 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2101_8442_23370 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1110_26450_7049 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456 M00967_43_000000000-A3JHG_1_1108_18681_22857 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23381_22618 M00967_43_000000000-A3JHG_1_1112_14780_8078 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23381_22618 M00967_43_000000000-A3JHG_1_2103_19149_16619 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23381_22618 M00967_43_000000000-A3JHG_1_1101_5922_6513 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23381_22618 M00967_43_000000000-A3JHG_1_2108_18874_21200 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_2111_8291_8111 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_2114_24618_11376 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_1104_29281_13264 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999 M00967_43_000000000-A3JHG_1_1113_17274_13189 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_1104_24665_17168 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_1104_10943_19850 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_2113_13473_22310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_1114_13545_27181 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_1111_18224_14666 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_2103_16319_7131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449 M00967_43_000000000-A3JHG_1_1112_16577_22524 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17636_10033 M00967_43_000000000-A3JHG_1_1102_22375_5719 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17636_10033 M00967_43_000000000-A3JHG_1_1110_19584_26279 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7287_9945 M00967_43_000000000-A3JHG_1_1106_21375_27109 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7287_9945 M00967_43_000000000-A3JHG_1_2103_18861_20938 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8585_16521 M00967_43_000000000-A3JHG_1_1101_9331_5806 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8585_16521 M00967_43_000000000-A3JHG_1_1108_11559_26077 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_1107_12430_16983 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_2109_19574_4320 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998 M00967_43_000000000-A3JHG_1_2109_19808_24467 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2110_23478_21034 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2105_12184_22923 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_2106_25956_17550 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235 M00967_43_000000000-A3JHG_1_1106_19617_4733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_1101_25551_18277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2105_15118_19690 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_1106_19617_4733 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.04264" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.04264" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.04264" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1109_26233_13220 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1113_13105_21336 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1111_15521_13523 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1109_8925_8850 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124 M00967_43_000000000-A3JHG_1_1108_11375_17687 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1114_10373_6011 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8877_3935 M00967_43_000000000-A3JHG_1_1104_17583_11048 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8877_3935 M00967_43_000000000-A3JHG_1_1110_7742_10390 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8877_3935 M00967_43_000000000-A3JHG_1_1110_19584_26279 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19690_27437 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19690_27437 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19690_27437 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_19690_27437 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2101_8442_23370 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1110_26450_7049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1108_18681_22857 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198 M00967_43_000000000-A3JHG_1_1107_23853_7456 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5702_10845 M00967_43_000000000-A3JHG_1_1104_9875_8400 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9164_14402 M00967_43_000000000-A3JHG_1_2114_13884_15208 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9164_14402 M00967_43_000000000-A3JHG_1_1104_17535_11802 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9164_14402 M00967_43_000000000-A3JHG_1_1114_18788_21055 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9164_14402 M00967_43_000000000-A3JHG_1_1103_9133_12955 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6545_8894 M00967_43_000000000-A3JHG_1_2111_22131_23386 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6545_8894 M00967_43_000000000-A3JHG_1_1112_14865_2720 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310 M00967_43_000000000-A3JHG_1_1110_23003_13541 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1111_8225_24102 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554 M00967_43_000000000-A3JHG_1_1101_25610_23225 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8777_8861 M00967_43_000000000-A3JHG_1_1104_17583_11048 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8777_8861 M00967_43_000000000-A3JHG_1_1110_7742_10390 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8777_8861 M00967_43_000000000-A3JHG_1_1110_19584_26279 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8777_8861 M00967_43_000000000-A3JHG_1_1101_8877_3935 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1109_15598_4401 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1103_24460_16041 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_2104_23298_12028 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868 M00967_43_000000000-A3JHG_1_1114_7920_25776 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24625_19641 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24625_19641 M00967_43_000000000-A3JHG_1_2112_17219_14317 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24625_19641 M00967_43_000000000-A3JHG_1_1109_12129_4985 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24625_19641 M00967_43_000000000-A3JHG_1_2110_8933_4869 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1111_8225_24102 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_1101_25610_23225 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855 M00967_43_000000000-A3JHG_1_2113_19649_18554 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_8304_3695 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_2109_14530_23810 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_2106_25956_17550 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1106_19617_4733 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_2101_17535_7182 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_1108_14909_4503 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_1114_14958_19894 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_2112_26969_15672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750 M00967_43_000000000-A3JHG_1_1110_23003_13541 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10010_14992 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10010_14992 M00967_43_000000000-A3JHG_1_1110_13879_1994 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10010_14992 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10010_14992 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_10010_14992 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18804_14594 M00967_43_000000000-A3JHG_1_1113_6044_19045 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18804_14594 M00967_43_000000000-A3JHG_1_2108_19309_25091 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18804_14594 M00967_43_000000000-A3JHG_1_1107_27199_15268 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17603_21659 M00967_43_000000000-A3JHG_1_1106_22417_19200 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17603_21659 M00967_43_000000000-A3JHG_1_1104_12222_5327 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17603_21659 M00967_43_000000000-A3JHG_1_1107_18879_6171 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1113_27906_8733 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7624_17316 M00967_43_000000000-A3JHG_1_2105_20695_7710 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7624_17316 M00967_43_000000000-A3JHG_1_2109_21713_13835 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7624_17316 M00967_43_000000000-A3JHG_1_2113_21662_22858 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471 M00967_43_000000000-A3JHG_1_2104_15263_18242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471 M00967_43_000000000-A3JHG_1_1110_9258_26812 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471 M00967_43_000000000-A3JHG_1_2105_10211_20812 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471 M00967_43_000000000-A3JHG_1_2103_21648_16922 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471 M00967_43_000000000-A3JHG_1_1103_20010_17589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_2114_24618_11376 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1104_29281_13264 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1113_17274_13189 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765 M00967_43_000000000-A3JHG_1_1106_7807_21999 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23085_20010 M00967_43_000000000-A3JHG_1_1113_20770_14973 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23085_20010 M00967_43_000000000-A3JHG_1_2102_24666_20800 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23085_20010 M00967_43_000000000-A3JHG_1_1109_5351_11203 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_1109_8088_17239 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_1111_24706_13976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_2101_17535_7182 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_1108_14909_4503 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_1114_14958_19894 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_2112_26969_15672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_1110_23003_13541 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206 M00967_43_000000000-A3JHG_1_2110_23990_16750 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17039_19920 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17039_19920 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17039_19920 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17039_19920 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796 M00967_43_000000000-A3JHG_1_1108_23966_22243 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9943_6220 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9943_6220 M00967_43_000000000-A3JHG_1_1101_3758_10627 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9943_6220 M00967_43_000000000-A3JHG_1_1112_16230_18061 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2101_8442_23370 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1108_18681_22857 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1107_23853_7456 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875 M00967_43_000000000-A3JHG_1_1104_19763_16198 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_2104_15263_18242 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_1110_9258_26812 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_1102_22955_21067 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_2105_10211_20812 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_2103_21648_16922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_1103_20010_17589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003 M00967_43_000000000-A3JHG_1_2108_5933_20471 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_1106_13721_7517 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_1105_11983_14742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_2112_7837_7502 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_1104_11494_18542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062 M00967_43_000000000-A3JHG_1_2104_9988_18586 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_2106_25956_17550 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_1103_24229_16660 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753 M00967_43_000000000-A3JHG_1_2110_24147_19370 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2101_8442_23370 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1110_26450_7049 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1108_18681_22857 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1107_23853_7456 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_1104_19763_16198 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783 M00967_43_000000000-A3JHG_1_2104_21537_12875 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624 M00967_43_000000000-A3JHG_1_1113_19148_12981 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_1105_25210_11850 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_1107_14488_13179 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_1112_11788_5032 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_2111_15720_29081 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349 M00967_43_000000000-A3JHG_1_1107_10545_13535 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091 M00967_43_000000000-A3JHG_1_2111_19706_18823 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1112_21256_12063 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_2113_18673_6259 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1108_24102_4725 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1107_5528_8801 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875 M00967_43_000000000-A3JHG_1_1105_12311_16407 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18175_23385 M00967_43_000000000-A3JHG_1_1105_14835_21938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18175_23385 M00967_43_000000000-A3JHG_1_2106_7943_14658 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18175_23385 M00967_43_000000000-A3JHG_1_1110_17805_7443 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18175_23385 M00967_43_000000000-A3JHG_1_1114_27837_14578 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_2106_16818_12005 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_1102_18759_22050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_2102_21816_16599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992 M00967_43_000000000-A3JHG_1_1112_15601_12718 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24650_7997 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24650_7997 M00967_43_000000000-A3JHG_1_1112_10808_11763 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_24650_7997 M00967_43_000000000-A3JHG_1_2104_9368_14850 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1113_13105_21336 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1111_15521_13523 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1109_8925_8850 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1108_11375_17687 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380 M00967_43_000000000-A3JHG_1_1102_15600_19124 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17376_13100 M00967_43_000000000-A3JHG_1_1102_21382_27245 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17376_13100 M00967_43_000000000-A3JHG_1_1105_27543_11258 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17376_13100 M00967_43_000000000-A3JHG_1_2113_16796_6663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17376_13100 M00967_43_000000000-A3JHG_1_1111_4667_8023 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17376_13100 M00967_43_000000000-A3JHG_1_2103_18926_9248 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_1101_11754_11390 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114 M00967_43_000000000-A3JHG_1_2113_20319_11310 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2104_15288_13724 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2106_17976_10139 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462 M00967_43_000000000-A3JHG_1_2113_7399_10515 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944 M00967_43_000000000-A3JHG_1_2103_23519_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1110_27941_14131 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1109_8925_8850 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1108_11375_17687 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1102_15600_19124 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1111_19212_21380 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1113_19148_12981 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758 M00967_43_000000000-A3JHG_1_1114_5058_10624 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_2106_16818_12005 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_1104_11751_17981 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_2114_22279_21530 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780 M00967_43_000000000-A3JHG_1_1112_15601_12718 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951 M00967_43_000000000-A3JHG_1_2104_4913_14332 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5527_16685 M00967_43_000000000-A3JHG_1_1113_20207_16728 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5527_16685 M00967_43_000000000-A3JHG_1_1102_15058_11924 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5527_16685 M00967_43_000000000-A3JHG_1_1110_16506_20291 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5527_16685 M00967_43_000000000-A3JHG_1_1105_9868_4758 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_20923_14281 M00967_43_000000000-A3JHG_1_1103_12991_25169 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920 M00967_43_000000000-A3JHG_1_1112_10808_11763 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920 M00967_43_000000000-A3JHG_1_2104_9368_14850 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920 M00967_43_000000000-A3JHG_1_1111_24650_7997 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12058 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12058 M00967_43_000000000-A3JHG_1_1112_10194_20788 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12058 M00967_43_000000000-A3JHG_1_1112_9141_21408 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12058 M00967_43_000000000-A3JHG_1_1112_6751_17524 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12058 M00967_43_000000000-A3JHG_1_1110_10298_23586 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_1101_17961_23432 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_1108_23966_22243 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_2112_24625_19641 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778 M00967_43_000000000-A3JHG_1_2105_10589_14796 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815 M00967_43_000000000-A3JHG_1_2105_17039_19920 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2111_19706_18823 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931 M00967_43_000000000-A3JHG_1_1113_27297_13091 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1111_8225_24102 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_1101_25610_23225 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2113_19649_18554 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804 M00967_43_000000000-A3JHG_1_2112_12800_22855 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_2111_19248_7551 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_1110_15924_11287 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_2113_25465_9646 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782 M00967_43_000000000-A3JHG_1_2109_10445_7921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13772_8177 M00967_43_000000000-A3JHG_1_1109_22380_20532 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13772_8177 M00967_43_000000000-A3JHG_1_2102_14252_8710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13772_8177 M00967_43_000000000-A3JHG_1_2104_20074_25538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2105_19668_20827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1110_15924_11287 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_2113_25465_9646 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_2109_10445_7921 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866 M00967_43_000000000-A3JHG_1_1101_15683_17782 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12965_2018 M00967_43_000000000-A3JHG_1_1109_9629_6769 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_12965_2018 M00967_43_000000000-A3JHG_1_1105_23612_18072 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_2104_4913_14332 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324 M00967_43_000000000-A3JHG_1_1107_25872_9951 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557 M00967_43_000000000-A3JHG_1_2106_27160_14501 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557 M00967_43_000000000-A3JHG_1_2103_20113_16810 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557 M00967_43_000000000-A3JHG_1_1105_15636_27192 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557 M00967_43_000000000-A3JHG_1_2110_16565_10623 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557 M00967_43_000000000-A3JHG_1_1111_19891_5082 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_1111_19384_17999 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2102_10627_9330 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2112_9811_9982 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405 M00967_43_000000000-A3JHG_1_2107_16642_25679 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446 M00967_43_000000000-A3JHG_1_1110_10033_25797 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446 M00967_43_000000000-A3JHG_1_1106_17680_15930 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446 M00967_43_000000000-A3JHG_1_1108_10328_10718 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446 M00967_43_000000000-A3JHG_1_1109_9215_20566 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_2107_12720_21761 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_2103_19057_10430 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_2108_22020_21434 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_2105_17326_26211 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_1106_2293_12053 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_2113_13414_6546 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665 M00967_43_000000000-A3JHG_1_2102_19287_23968 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_1114_10055_21108 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_1103_23750_17910 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042 M00967_43_000000000-A3JHG_1_2113_8991_5702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_1101_7042_19790 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_2106_27160_14501 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_2103_20113_16810 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_1105_15636_27192 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_2110_16565_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_1111_19891_5082 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263 M00967_43_000000000-A3JHG_1_1103_8349_14557 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18276_27021 M00967_43_000000000-A3JHG_1_2108_8220_23281 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_18276_27021 M00967_43_000000000-A3JHG_1_1111_11386_7041 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1109_12129_4985 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2110_8933_4869 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2101_13130_17466 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1114_23025_24395 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1108_14370_23848 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_2102_21816_16599 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1104_11751_17981 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_2114_22279_21530 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758 M00967_43_000000000-A3JHG_1_1107_7832_8780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1110_10946_20858 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21038_9589 M00967_43_000000000-A3JHG_1_1109_9629_6769 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21038_9589 M00967_43_000000000-A3JHG_1_2108_5077_12621 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21038_9589 M00967_43_000000000-A3JHG_1_1105_23612_18072 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_21038_9589 M00967_43_000000000-A3JHG_1_1114_12965_2018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_2112_8686_5552 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_1106_17565_8490 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_2102_17019_10818 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_1114_3131_16050 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_1107_21038_8569 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_1102_10827_17300 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348 M00967_43_000000000-A3JHG_1_1107_27189_20696 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1103_2359_11998 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1102_21170_25493 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2114_16268_21350 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2101_8442_23370 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1110_26450_7049 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1108_18681_22857 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1107_23853_7456 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_1104_19763_16198 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2104_21537_12875 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779 M00967_43_000000000-A3JHG_1_2101_16474_12783 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2111_19706_18823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1113_27297_13091 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127 M00967_43_000000000-A3JHG_1_1103_11491_13931 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840 M00967_43_000000000-A3JHG_1_1105_8356_13589 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1110_27941_14131 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.04264" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.03876" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.03876" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.03876" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_1112_24992_23873 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_1113_28366_19519 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058 M00967_43_000000000-A3JHG_1_2107_5824_20256 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20651_22709 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20651_22709 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20651_22709 M00967_43_000000000-A3JHG_1_1114_22804_12324 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20651_22709 M00967_43_000000000-A3JHG_1_2113_11757_15493 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_20651_22709 M00967_43_000000000-A3JHG_1_1101_21305_8478 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18284_2758 M00967_43_000000000-A3JHG_1_1109_15029_15883 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18284_2758 M00967_43_000000000-A3JHG_1_1112_19666_21477 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_18284_2758 M00967_43_000000000-A3JHG_1_1114_7676_25736 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1109_17953_9880 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1114_22942_23318 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_1111_19384_17999 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_2102_10627_9330 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_2112_9811_9982 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425 M00967_43_000000000-A3JHG_1_2107_16642_25679 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_16989_10433 M00967_43_000000000-A3JHG_1_1102_11115_19075 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_16989_10433 M00967_43_000000000-A3JHG_1_2106_20527_14577 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121 M00967_43_000000000-A3JHG_1_1110_5736_9217 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121 M00967_43_000000000-A3JHG_1_2108_16551_3455 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121 M00967_43_000000000-A3JHG_1_1103_8311_8476 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121 M00967_43_000000000-A3JHG_1_2111_13819_4746 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121 M00967_43_000000000-A3JHG_1_2110_19314_26014 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1112_21256_12063 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_2113_18673_6259 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1108_24102_4725 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1107_5528_8801 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1105_12311_16407 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1113_19457_3875 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1108_24102_4725 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_2104_4913_14332 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073 M00967_43_000000000-A3JHG_1_1113_6747_17324 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_1105_13597_20997 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_2113_7399_10515 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086 M00967_43_000000000-A3JHG_1_1110_14855_8462 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557 M00967_43_000000000-A3JHG_1_1109_14735_26068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_2111_24817_7965 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474 M00967_43_000000000-A3JHG_1_1108_10511_13856 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_1112_24992_23873 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497 M00967_43_000000000-A3JHG_1_1108_25479_20227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.01195" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.003984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_1111_19384_17999 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_2112_9811_9982 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_1103_28451_19405 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946 M00967_43_000000000-A3JHG_1_2105_15982_24425 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609 M00967_43_000000000-A3JHG_1_1106_17680_15930 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609 M00967_43_000000000-A3JHG_1_1109_9215_20566 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609 M00967_43_000000000-A3JHG_1_2114_17062_26446 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_2103_23519_12843 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724 M00967_43_000000000-A3JHG_1_1110_14814_10944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138 M00967_43_000000000-A3JHG_1_1104_10691_10850 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18158_25538 M00967_43_000000000-A3JHG_1_1102_27488_14514 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18158_25538 M00967_43_000000000-A3JHG_1_1103_11725_18144 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18158_25538 M00967_43_000000000-A3JHG_1_1110_24287_19984 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2102_17579_3448 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2102_17415_24873 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014 M00967_43_000000000-A3JHG_1_1102_10298_5101 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_2112_7837_7502 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_1104_11494_18542 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_2104_9988_18586 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123 M00967_43_000000000-A3JHG_1_2102_12732_5062 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1113_19148_12981 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1114_5058_10624 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502 M00967_43_000000000-A3JHG_1_1109_10494_10758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_2112_11899_4014 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1113_2128_16290 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_2102_16879_22327 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1114_13158_9182 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1103_14890_4136 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_1101_23714_7638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638 M00967_43_000000000-A3JHG_1_2101_7947_10616 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_1112_25336_20130 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9624_20695 M00967_43_000000000-A3JHG_1_1104_17315_20406 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9624_20695 M00967_43_000000000-A3JHG_1_1110_9902_7407 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295 M00967_43_000000000-A3JHG_1_2113_13714_22583 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_1101_6929_7655 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_1111_11673_24359 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_2106_10758_21336 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_2102_17019_10818 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_1114_3131_16050 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_1107_21038_8569 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_1102_10827_17300 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_1107_27189_20696 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093 M00967_43_000000000-A3JHG_1_2111_13174_19348 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25950_7889 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25950_7889 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25950_7889 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25950_7889 M00967_43_000000000-A3JHG_1_2109_10394_17608 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25950_7889 M00967_43_000000000-A3JHG_1_2108_10705_11765 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763 M00967_43_000000000-A3JHG_1_2104_9326_20908 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_1104_27702_14915 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_2114_14273_14388 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_1112_24714_23272 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739 M00967_43_000000000-A3JHG_1_1110_16185_17163 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247 M00967_43_000000000-A3JHG_1_1110_2564_19006 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1101_11754_11390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_2113_20319_11310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702 M00967_43_000000000-A3JHG_1_1110_15744_23114 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_2113_8332_13243 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_2104_4913_14332 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1107_25872_9951 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_1113_6747_17324 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482 M00967_43_000000000-A3JHG_1_2103_17104_22073 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1112_24992_23873 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1113_28366_19519 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_1104_6519_20940 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_2107_5824_20256 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_2109_14780_27058 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143 M00967_43_000000000-A3JHG_1_2102_12357_13497 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_2113_25465_9646 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_2109_10445_7921 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_1101_15683_17782 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617 M00967_43_000000000-A3JHG_1_1114_22494_17866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_2112_11743_8674 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_2111_23078_24856 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_1112_5118_11366 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_2107_27757_15813 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_1112_21256_12063 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_2113_18673_6259 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_1108_24102_4725 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_1107_5528_8801 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_1105_12311_16407 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_1113_19457_3875 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994 M00967_43_000000000-A3JHG_1_2103_19006_21979 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1114_23055_10307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1114_1863_15569 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1102_2114_15227 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2106_5552_10582 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2104_18899_8185 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2106_12253_21736 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2102_22167_26446 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1108_14021_11937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1102_8887_21995 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2113_20973_10900 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1105_14971_7732 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1101_23926_12310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1113_9185_6542 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1112_18294_27835 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1103_12955_24887 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2107_21426_22343 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1113_9498_21548 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1106_18846_19788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1101_25615_24528 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1112_14508_26097 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1110_12199_12204 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1109_23085_25742 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1107_10632_26179 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2113_4057_19587 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2113_15115_2610 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2112_21019_6342 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2112_19379_3300 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2110_17772_24108 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2109_16110_6542 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2108_17213_14575 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2105_22686_4913 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2105_21571_16049 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2103_11158_26523 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2102_22092_15614 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2101_20145_22042 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1114_8142_5567 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1114_21716_25171 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1112_14080_4232 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1111_5994_19726 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1110_22175_11297 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1110_13298_3746 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1108_8714_18612 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1106_14256_14203 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1106_11283_15681 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1105_3060_12805 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1102_13343_4567 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2113_7165_8175 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2113_26782_20607 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2112_26291_17310 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2108_20601_20789 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2106_15186_22937 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2105_15139_15368 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2105_14589_9498 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2104_14075_7093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1112_22736_6885 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1111_8849_21176 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1109_3671_10080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1109_12151_14526 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1108_21244_15970 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1104_23895_17769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1103_27900_10832 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2114_20216_27116 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2112_24260_7949 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2108_19974_4752 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2101_16339_18404 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1108_18681_22857 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1107_23853_7456 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_1104_19763_16198 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2104_21537_12875 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2101_16474_12783 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023 M00967_43_000000000-A3JHG_1_2110_27183_12779 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2112_22880_7920 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1114_10373_6011 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1102_10298_5101 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294 M00967_43_000000000-A3JHG_1_1113_9810_23014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1112_7075_7100 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1111_14408_20028 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1113_2128_16290 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1108_12507_2610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_2102_16879_22327 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1114_13158_9182 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1103_14890_4136 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1101_23714_7638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_2101_7947_10616 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830 M00967_43_000000000-A3JHG_1_1113_14224_15638 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27163_22727 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27163_22727 M00967_43_000000000-A3JHG_1_2107_26555_14955 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27163_22727 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27163_22727 M00967_43_000000000-A3JHG_1_2106_6909_9337 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27163_22727 M00967_43_000000000-A3JHG_1_1109_17953_9880 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18422_23262 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18422_23262 M00967_43_000000000-A3JHG_1_2109_14117_4744 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2104_15288_13724 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2106_17976_10139 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2113_7399_10515 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_1110_14855_8462 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158 M00967_43_000000000-A3JHG_1_2103_15482_15086 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409 M00967_43_000000000-A3JHG_1_1106_8303_10880 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409 M00967_43_000000000-A3JHG_1_2110_16325_18649 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409 M00967_43_000000000-A3JHG_1_2102_22381_10329 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409 M00967_43_000000000-A3JHG_1_2112_7963_5767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23262_25144 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23262_25144 M00967_43_000000000-A3JHG_1_2109_14117_4744 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23262_25144 M00967_43_000000000-A3JHG_1_2113_9766_20824 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23262_25144 M00967_43_000000000-A3JHG_1_1108_18422_23262 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039 M00967_43_000000000-A3JHG_1_1113_9810_23014 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_2104_9326_20908 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130 M00967_43_000000000-A3JHG_1_1111_23503_22763 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1110_27941_14131 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_2104_9326_20908 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1111_23503_22763 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414 M00967_43_000000000-A3JHG_1_1105_9050_25130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2106_25956_17550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1106_19617_4733 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2110_24147_19370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811 M00967_43_000000000-A3JHG_1_2101_20223_25753 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493 M00967_43_000000000-A3JHG_1_2103_19878_24663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27270_10834 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27270_10834 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27270_10834 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27270_10834 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_27270_10834 M00967_43_000000000-A3JHG_1_1103_28451_19405 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24923_8045 M00967_43_000000000-A3JHG_1_1104_9801_4846 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24923_8045 M00967_43_000000000-A3JHG_1_2101_16955_14005 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15705_5052 M00967_43_000000000-A3JHG_1_1111_22890_9747 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15705_5052 M00967_43_000000000-A3JHG_1_1109_14359_12484 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_15705_5052 M00967_43_000000000-A3JHG_1_1108_10061_3691 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1105_13597_20997 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2106_17976_10139 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_2103_15482_15086 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864 M00967_43_000000000-A3JHG_1_1107_9318_10158 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1104_6949_16896 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999 M00967_43_000000000-A3JHG_1_2103_19878_24663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_2107_13016_9270 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959 M00967_43_000000000-A3JHG_1_1114_19529_17837 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1111_23503_22763 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520 M00967_43_000000000-A3JHG_1_1105_9050_25130 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14928_26203 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14928_26203 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14928_26203 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14928_26203 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_2102_17415_24873 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_13258_9769 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_2103_23519_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_1110_14814_10944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600 M00967_43_000000000-A3JHG_1_1114_3483_21724 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10390_20786 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10390_20786 M00967_43_000000000-A3JHG_1_1106_11240_22282 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10390_20786 M00967_43_000000000-A3JHG_1_1104_10875_20179 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10390_20786 M00967_43_000000000-A3JHG_1_1112_9549_5158 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_10390_20786 M00967_43_000000000-A3JHG_1_1106_10941_17222 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18106_21085 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18106_21085 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_18106_21085 M00967_43_000000000-A3JHG_1_2103_16795_27764 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1106_23697_7097 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1113_15332_23431 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14915_26251 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14915_26251 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_14915_26251 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2107_26555_14955 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2106_6909_9337 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2103_21648_16922 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302 M00967_43_000000000-A3JHG_1_1108_27163_22727 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181 M00967_43_000000000-A3JHG_1_2113_16978_21215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13599_17305 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13599_17305 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13599_17305 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13599_17305 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_13599_17305 M00967_43_000000000-A3JHG_1_1102_14928_26203 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1110_15924_11287 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_2113_25465_9646 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_2109_10445_7921 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1101_15683_17782 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1114_22494_17866 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449 M00967_43_000000000-A3JHG_1_1110_12004_2617 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02713" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.04264" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1110_2564_19006 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833 M00967_43_000000000-A3JHG_1_1110_20095_8247 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.0428" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.01172" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_1104_27702_14915 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_2114_14273_14388 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_1112_24714_23272 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807 M00967_43_000000000-A3JHG_1_1111_17207_6739 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256 M00967_43_000000000-A3JHG_1_2112_11736_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110 M00967_43_000000000-A3JHG_1_1113_4490_10708 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_1105_8356_13589 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924 M00967_43_000000000-A3JHG_1_2109_27058_11840 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464 M00967_43_000000000-A3JHG_1_2109_12163_4647 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2112_11736_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016 M00967_43_000000000-A3JHG_1_2108_12449_21256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844 M00967_43_000000000-A3JHG_1_2109_10394_17608 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844 M00967_43_000000000-A3JHG_1_1111_25950_7889 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_1104_27702_14915 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_2114_14273_14388 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_1112_24714_23272 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_1110_16185_17163 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_1111_17207_6739 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393 M00967_43_000000000-A3JHG_1_2108_14707_9807 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893 M00967_43_000000000-A3JHG_1_2107_26879_8110 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_1113_4490_10708 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991 M00967_43_000000000-A3JHG_1_2107_26879_8110 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2111_15422_19940 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_1101_25551_18277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2105_15118_19690 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245 M00967_43_000000000-A3JHG_1_1111_6400_18545 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02326" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.007812" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.01556" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03876" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2113_16978_21215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571 M00967_43_000000000-A3JHG_1_2112_10029_17181 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1113_19148_12981 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1114_5058_10624 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1109_10494_10758 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053 M00967_43_000000000-A3JHG_1_1113_18700_9502 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16924_2551 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16924_2551 M00967_43_000000000-A3JHG_1_2109_14117_4744 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16924_2551 M00967_43_000000000-A3JHG_1_1108_18422_23262 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_2103_19878_24663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1104_6527_16493 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_1105_4106_18467 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_2111_24817_7965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_1105_9374_22772 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721 M00967_43_000000000-A3JHG_1_2102_16491_2474 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13838_9027 M00967_43_000000000-A3JHG_1_1110_20816_14150 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13838_9027 M00967_43_000000000-A3JHG_1_2110_19140_21811 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2109_12163_4647 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1104_6949_16896 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1105_19799_18751 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16472_4147 M00967_43_000000000-A3JHG_1_2106_14859_14822 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16472_4147 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_16472_4147 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_1107_20352_16896 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_2103_23519_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_1110_14814_10944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_1114_3483_21724 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044 M00967_43_000000000-A3JHG_1_1102_11222_12600 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1104_14218_8400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2110_26129_21059 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2106_14245_10215 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.036" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1112_10641_14749 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1106_12085_17040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1113_25205_6894 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2110_7067_15061 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2110_17584_23187 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2109_17577_12359 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2106_22868_26146 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1109_2798_11023 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_28850_15369 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_28850_15369 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_28850_15369 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_28850_15369 M00967_43_000000000-A3JHG_1_1109_14735_26068 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_28850_15369 M00967_43_000000000-A3JHG_1_2102_25051_20557 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19252_26342 M00967_43_000000000-A3JHG_1_1102_27488_14514 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19252_26342 M00967_43_000000000-A3JHG_1_1103_11725_18144 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19252_26342 M00967_43_000000000-A3JHG_1_1110_24287_19984 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_19252_26342 M00967_43_000000000-A3JHG_1_1114_18158_25538 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_1113_4490_10708 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2107_26879_8110 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433 M00967_43_000000000-A3JHG_1_2104_17795_17991 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1101_3758_10627 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1108_27354_15573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1107_24903_5978 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1112_16230_18061 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_1105_20465_8611 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_2107_18354_8608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_2101_22555_16416 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899 M00967_43_000000000-A3JHG_1_2104_9943_6220 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481 M00967_43_000000000-A3JHG_1_1113_7150_17375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481 M00967_43_000000000-A3JHG_1_1103_10830_22855 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481 M00967_43_000000000-A3JHG_1_2103_16795_27764 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481 M00967_43_000000000-A3JHG_1_1101_18106_21085 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_1104_6949_16896 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1107_12430_16983 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_2109_19574_4320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_2109_19808_24467 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644 M00967_43_000000000-A3JHG_1_1104_22299_24998 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_1102_14193_19079 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_1102_4962_7612 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_1106_9232_9214 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_1106_8303_10880 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_2110_16325_18649 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_2102_22381_10329 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_2112_7963_5767 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507 M00967_43_000000000-A3JHG_1_1107_16665_12409 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_1105_14219_3778 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_1102_14928_26203 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045 M00967_43_000000000-A3JHG_1_2110_13599_17305 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1102_23295_14739 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_2105_17957_13444 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1114_25590_23883 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1105_25210_11850 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1107_14488_13179 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1112_11788_5032 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_2111_15720_29081 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1107_10545_13535 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606 M00967_43_000000000-A3JHG_1_1113_6268_10349 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_2111_24817_7965 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_1108_10511_13856 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_2102_16491_2474 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161 M00967_43_000000000-A3JHG_1_2101_17218_22721 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1107_23638_4659 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093 M00967_43_000000000-A3JHG_1_1103_26302_22051 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_9992_20296 M00967_43_000000000-A3JHG_1_2102_27520_15637 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_2111_15422_19940 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2108_6233_13205 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_2112_8686_5552 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381 M00967_43_000000000-A3JHG_1_2111_21247_15845 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2108_12449_21256 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980 M00967_43_000000000-A3JHG_1_2105_3878_16016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1103_5171_14027 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1112_25336_20130 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803 M00967_43_000000000-A3JHG_1_1113_13032_6958 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.012" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2114_4725_7023 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2106_3727_12917 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2105_18791_12472 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096 M00967_43_000000000-A3JHG_1_1112_12814_18299 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1101_25610_23225 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2113_19649_18554 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_2112_12800_22855 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343 M00967_43_000000000-A3JHG_1_1102_22379_19804 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2103_19006_21979 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1112_12814_18299 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309 M00967_43_000000000-A3JHG_1_1104_10606_3096 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354 M00967_43_000000000-A3JHG_1_1114_22804_12324 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354 M00967_43_000000000-A3JHG_1_2113_11757_15493 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354 M00967_43_000000000-A3JHG_1_1101_21305_8478 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354 M00967_43_000000000-A3JHG_1_2108_20651_22709 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_2114_7946_17808 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1105_10606_2663 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1113_8139_17897 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1109_12129_4985 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2110_8933_4869 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1103_6362_13340 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1108_13402_18823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2113_14931_21754 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2112_24625_19641 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2105_10589_14796 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_2113_13714_22583 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353 M00967_43_000000000-A3JHG_1_1102_24841_18959 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638 M00967_43_000000000-A3JHG_1_2103_21597_21834 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638 M00967_43_000000000-A3JHG_1_2106_14094_18415 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638 M00967_43_000000000-A3JHG_1_1110_15335_17802 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638 M00967_43_000000000-A3JHG_1_1108_7641_20331 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638 M00967_43_000000000-A3JHG_1_2112_20183_27184 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638 M00967_43_000000000-A3JHG_1_2110_10607_6385 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336 M00967_43_000000000-A3JHG_1_2106_7936_8291 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336 M00967_43_000000000-A3JHG_1_1113_7150_17375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336 M00967_43_000000000-A3JHG_1_1101_18106_21085 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2109_12163_4647 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1112_24992_23873 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1108_25479_20227 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_2102_12357_13497 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574 M00967_43_000000000-A3JHG_1_1110_13950_25143 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1110_13879_1994 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1112_25336_20130 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1113_13032_6958 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260 M00967_43_000000000-A3JHG_1_1104_8057_17803 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_1112_11370_15670 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754 M00967_43_000000000-A3JHG_1_2109_25599_13567 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15661_13758 M00967_43_000000000-A3JHG_1_1109_9629_6769 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15661_13758 M00967_43_000000000-A3JHG_1_1105_23612_18072 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15661_13758 M00967_43_000000000-A3JHG_1_1114_12965_2018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15661_13758 M00967_43_000000000-A3JHG_1_2112_21038_9589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1114_10055_21108 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1103_23750_17910 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_2113_8991_5702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_1101_21197_21065 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873 M00967_43_000000000-A3JHG_1_2113_21308_17042 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9580_7709 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9580_7709 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9580_7709 M00967_43_000000000-A3JHG_1_1108_19250_12721 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9580_7709 M00967_43_000000000-A3JHG_1_2109_18345_18844 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_9580_7709 M00967_43_000000000-A3JHG_1_2102_14512_24313 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_24670_14227 M00967_43_000000000-A3JHG_1_1110_6952_6984 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_1108_19250_12721 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_2109_18345_18844 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_2102_14512_24313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113 M00967_43_000000000-A3JHG_1_2105_9580_7709 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1106_18001_27099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_1110_13999_26329 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_1109_14735_26068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269 M00967_43_000000000-A3JHG_1_2102_25051_20557 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2114_28218_15620 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2104_21630_21997 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2102_15622_9697 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_1112_11370_15670 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2109_25599_13567 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495 M00967_43_000000000-A3JHG_1_2108_25850_8754 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1104_10691_10850 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841 M00967_43_000000000-A3JHG_1_1114_26942_22032 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2113_16978_21215 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2112_10029_17181 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165 M00967_43_000000000-A3JHG_1_2102_23398_14571 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_2112_22672_12392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_1109_14735_26068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_2102_25051_20557 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_1111_28850_15369 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682 M00967_43_000000000-A3JHG_1_2104_22154_4269 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_2104_17227_9135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2103_19878_24663 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_1104_6527_16493 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727 M00967_43_000000000-A3JHG_1_2101_9500_21244 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1112_25336_20130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_2110_10010_14992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1113_13032_6958 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_1104_8057_17803 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719 M00967_43_000000000-A3JHG_1_2108_26075_20260 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_1110_13879_1994 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_2110_10010_14992 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807 M00967_43_000000000-A3JHG_1_2101_6909_12719 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.003922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_2101_9084_14502 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_1111_12899_5675 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_1104_5274_12644 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_1110_19653_28369 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630 M00967_43_000000000-A3JHG_1_1106_22826_4799 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1104_10606_3096 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894 M00967_43_000000000-A3JHG_1_1102_18640_14309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1112_7560_26006 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_1101_4346_14940 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208 M00967_43_000000000-A3JHG_1_2108_23299_14833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1112_12814_18299 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1104_10606_3096 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1102_18640_14309 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948 M00967_43_000000000-A3JHG_1_1114_3366_17894 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_1101_17656_7294 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2104_20685_7431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2102_12930_14637 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2103_18492_19103 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2101_13953_7091 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2102_10627_9330 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2112_9811_9982 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2107_16642_25679 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_1103_28451_19405 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2105_15982_24425 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_2101_22573_22946 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157 M00967_43_000000000-A3JHG_1_1104_27270_10834 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_2106_12578_10682 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_1101_18693_22396 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520 M00967_43_000000000-A3JHG_1_2104_18525_11294 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1101_18922_4934 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1113_4318_16941 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1101_13714_14338 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_1104_11335_21750 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_2114_24635_10372 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_1107_24903_5978 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_1105_20465_8611 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_2107_18354_8608 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_2101_22555_16416 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824 M00967_43_000000000-A3JHG_1_1110_18205_23899 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_2113_11924_15441 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_1109_18126_7545 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_2106_16818_12005 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_1102_18759_22050 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_2102_21816_16599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_1112_15601_12718 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139 M00967_43_000000000-A3JHG_1_1113_13321_19992 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1105_8356_13589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_2109_27058_11840 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_1102_13258_9769 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443 M00967_43_000000000-A3JHG_1_2106_23285_7924 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2113_16978_21215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2112_10029_17181 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2102_23398_14571 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561 M00967_43_000000000-A3JHG_1_2104_11929_10165 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881 M00967_43_000000000-A3JHG_1_2114_26633_11062 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2102_17579_3448 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2102_17415_24873 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1102_13319_18244 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1114_23944_10783 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2114_9526_8096 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1113_27906_8733 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2111_8304_3695 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2111_19706_18823 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1113_27297_13091 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_1103_11491_13931 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2109_8160_14127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052 M00967_43_000000000-A3JHG_1_2104_19264_18585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1102_22713_17777 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1102_10298_5101 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_2102_17415_24873 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1113_9810_23014 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1109_24246_18294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397 M00967_43_000000000-A3JHG_1_1106_21706_6039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_2107_13016_9270 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_1114_19529_17837 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879 M00967_43_000000000-A3JHG_1_1102_24841_18959 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2112_11736_17751 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2108_12449_21256 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_2105_3878_16016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683 M00967_43_000000000-A3JHG_1_1105_21018_4980 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_26683_22024 M00967_43_000000000-A3JHG_1_1104_7522_5556 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_26683_22024 M00967_43_000000000-A3JHG_1_1112_20884_13058 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_26683_22024 M00967_43_000000000-A3JHG_1_1111_15129_15739 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_26683_22024 M00967_43_000000000-A3JHG_1_1109_24109_8015 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_26683_22024 M00967_43_000000000-A3JHG_1_1108_2644_15119 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10463_16723 M00967_43_000000000-A3JHG_1_1104_13731_17325 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10463_16723 M00967_43_000000000-A3JHG_1_2110_23478_21034 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10463_16723 M00967_43_000000000-A3JHG_1_1109_9197_9746 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10463_16723 M00967_43_000000000-A3JHG_1_1103_10830_22855 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_10463_16723 M00967_43_000000000-A3JHG_1_2105_12184_22923 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_2104_9326_20908 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1111_23503_22763 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1105_9050_25130 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1105_12786_23414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759 M00967_43_000000000-A3JHG_1_1102_16656_25520 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_1106_18001_27099 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1104_6949_16896 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_1114_26942_22032 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664 M00967_43_000000000-A3JHG_1_2104_19605_13841 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1101_25551_18277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2105_15118_19690 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1104_10691_10850 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_1114_28534_17138 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778 M00967_43_000000000-A3JHG_1_2104_19605_13841 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_13011_10787 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_12163_4647 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1102_10298_5101 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1109_24246_18294 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519 M00967_43_000000000-A3JHG_1_1106_6775_23397 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2113_19649_18554 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_2112_12800_22855 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1102_22379_19804 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185 M00967_43_000000000-A3JHG_1_1103_27632_18343 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16960_19773 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16960_19773 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16960_19773 M00967_43_000000000-A3JHG_1_1113_7150_17375 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16960_19773 M00967_43_000000000-A3JHG_1_1101_18106_21085 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_16960_19773 M00967_43_000000000-A3JHG_1_2113_13419_4336 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26629_16356 M00967_43_000000000-A3JHG_1_1101_9331_5806 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26629_16356 M00967_43_000000000-A3JHG_1_1108_11559_26077 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_26629_16356 M00967_43_000000000-A3JHG_1_1104_8585_16521 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2104_19264_18585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_2114_26633_11062 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_2113_20319_11310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1110_15744_23114 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109 M00967_43_000000000-A3JHG_1_1110_18711_9702 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2105_17039_19920 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421 M00967_43_000000000-A3JHG_1_2111_17711_14171 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2103_19878_24663 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2101_9500_21244 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605 M00967_43_000000000-A3JHG_1_2102_25569_21727 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2104_17227_9135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049 M00967_43_000000000-A3JHG_1_2103_24176_5781 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1103_7913_12679 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_2113_24996_25271 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1108_13402_18823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_2114_7946_17808 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_1106_13721_7517 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_1105_11983_14742 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269 M00967_43_000000000-A3JHG_1_2102_17714_13657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_2109_19574_4320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_2109_19808_24467 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1104_22299_24998 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602 M00967_43_000000000-A3JHG_1_1109_18240_15644 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_1103_24229_16660 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_2110_24147_19370 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701 M00967_43_000000000-A3JHG_1_2101_20223_25753 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18108_6799 M00967_43_000000000-A3JHG_1_1108_11048_4561 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18108_6799 M00967_43_000000000-A3JHG_1_1102_8524_16122 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_18108_6799 M00967_43_000000000-A3JHG_1_2101_21947_18735 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231 M00967_43_000000000-A3JHG_1_2112_18144_24421 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1101_14069_1827 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_6782_8678 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_29227_17392 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_12338_23302 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2106_19750_8249 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2105_16910_23126 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1109_7971_24259 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1105_24533_12281 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1102_12767_6517 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1101_19852_6086 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2113_11516_17364 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1110_6233_13348 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2104_6831_13690 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1112_14201_3200 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2107_5816_12823 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2113_28447_11534 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1114_9494_6921 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1102_26826_15050 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2107_22192_18692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1106_16049_12791 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2112_8250_17966 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2110_22767_24294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2108_18545_16099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2104_28710_13771 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1113_18462_22624 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1106_5620_18651 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2113_12698_4375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_18844_14068 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2109_15158_9571 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1114_27471_20890 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1105_13025_22602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1104_19589_20033 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2114_4609_11919 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2113_5393_13669 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_23532_22350 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2104_14582_18801 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2101_26706_7767 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2101_25853_16524 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1114_7614_20887 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1114_26027_6706 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1112_17779_24453 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1111_17856_12278 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1109_18939_28276 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1108_19974_13521 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1108_16907_24932 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1104_23236_5434 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1103_19447_15932 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1102_15632_13472 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1112_18186_10299 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_4814_18702 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_28308_17388 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2111_15523_6202 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2110_8652_13094 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2109_18721_12153 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2108_17961_17141 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2107_23464_17422 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2104_4792_21056 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2102_8178_6298 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1113_25861_7699 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1113_15543_24961 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1110_8577_13143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1105_17602_8360 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1104_13037_4245 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2108_23551_13901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_2104_11018_16527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1113_13105_21336 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1111_15521_13523 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1109_8925_8850 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1108_11375_17687 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1102_15600_19124 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612 M00967_43_000000000-A3JHG_1_1111_19212_21380 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037 M00967_43_000000000-A3JHG_1_2111_17860_28348 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2105_19668_20827 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_1113_4490_10708 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2107_26879_8110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2104_6381_12893 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_2104_17795_17991 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294 M00967_43_000000000-A3JHG_1_1110_21962_17433 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_2102_21816_16599 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1104_11751_17981 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_2114_22279_21530 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_1107_7832_8780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847 M00967_43_000000000-A3JHG_1_2112_8454_7758 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043 M00967_43_000000000-A3JHG_1_2113_4494_9088 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_1102_14077_24664 0.007968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2114_27537_17929 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2112_10826_19542 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_1102_17816_13048 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2111_5139_21514 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2108_5052_15488 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2107_7268_7252 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2106_23846_21173 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2105_19668_20827 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_1105_19259_21539 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2114_21881_13315 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_1110_25975_18317 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2101_8034_16374 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_1113_4490_10708 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2107_26879_8110 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2104_6381_12893 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2104_17795_17991 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_1110_21962_17433 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380 M00967_43_000000000-A3JHG_1_2107_10624_8294 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15684_7042 M00967_43_000000000-A3JHG_1_1102_14677_6391 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15684_7042 M00967_43_000000000-A3JHG_1_2114_12364_4557 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15684_7042 M00967_43_000000000-A3JHG_1_1114_19821_2207 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15684_7042 M00967_43_000000000-A3JHG_1_1101_12732_20169 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_15684_7042 M00967_43_000000000-A3JHG_1_1111_12249_8143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7735_17880 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7735_17880 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1112_6577_24020 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2110_14498_5844 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1104_23248_7732 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2106_17671_25995 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_24119_8383 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_10011_16636 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1113_27906_8733 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1102_22229_4678 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2109_12163_4647 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729 M00967_43_000000000-A3JHG_1_2106_8679_15164 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_1102_14198_21368 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_1113_7150_17375 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_2103_16795_27764 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_1110_15560_14481 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502 M00967_43_000000000-A3JHG_1_1108_16960_19773 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992 M00967_43_000000000-A3JHG_1_1109_8914_20463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_2113_4494_9088 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766 M00967_43_000000000-A3JHG_1_2105_7943_8043 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22000_19843 M00967_43_000000000-A3JHG_1_1103_8515_3790 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_22000_19843 M00967_43_000000000-A3JHG_1_2104_13757_8649 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2106_17976_10139 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_2113_7399_10515 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_1110_14855_8462 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_1107_9318_10158 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958 M00967_43_000000000-A3JHG_1_1103_21608_2864 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2113_16978_21215 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2112_10029_17181 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2102_23398_14571 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_2104_11929_10165 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163 M00967_43_000000000-A3JHG_1_1110_16691_26561 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1103_26302_22051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326 M00967_43_000000000-A3JHG_1_1106_21550_17093 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1104_6949_16896 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1114_26942_22032 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1109_8693_13422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678 M00967_43_000000000-A3JHG_1_1104_12197_8664 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1105_7380_21363 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_2111_20158_16824 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1110_17220_22884 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1106_12555_24141 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1112_22274_20191 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1112_11506_10347 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_2101_17405_14654 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_2107_19676_23496 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1103_14890_4136 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1101_23714_7638 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_2101_7947_10616 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1113_14224_15638 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197 M00967_43_000000000-A3JHG_1_1109_14610_28830 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476 M00967_43_000000000-A3JHG_1_1102_14677_6391 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476 M00967_43_000000000-A3JHG_1_2114_12364_4557 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476 M00967_43_000000000-A3JHG_1_1114_19821_2207 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476 M00967_43_000000000-A3JHG_1_1101_12732_20169 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476 M00967_43_000000000-A3JHG_1_1111_12249_8143 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476 M00967_43_000000000-A3JHG_1_2104_15684_7042 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_1104_11751_17981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_1107_7832_8780 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_2112_8454_7758 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721 M00967_43_000000000-A3JHG_1_2106_19379_4847 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2110_23478_21034 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2109_14530_23810 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1106_19617_4733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1103_24229_16660 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2110_24147_19370 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2101_20223_25753 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_1105_12578_19811 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361 M00967_43_000000000-A3JHG_1_2109_7813_4701 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_1109_8914_20463 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280 M00967_43_000000000-A3JHG_1_2102_10248_4992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1111_3610_17990 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1109_6486_11603 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1102_17795_18609 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1110_17890_15072 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1103_14435_10999 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1102_12515_6375 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1112_12814_18299 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1104_10606_3096 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1102_18640_14309 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1114_3366_17894 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759 M00967_43_000000000-A3JHG_1_1112_5981_8948 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23441_23046 M00967_43_000000000-A3JHG_1_1101_11856_9674 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23441_23046 M00967_43_000000000-A3JHG_1_1108_20070_20418 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23441_23046 M00967_43_000000000-A3JHG_1_2101_13466_27462 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23441_23046 M00967_43_000000000-A3JHG_1_1101_12695_21368 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_23441_23046 M00967_43_000000000-A3JHG_1_2111_6981_7811 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21353_8415 M00967_43_000000000-A3JHG_1_1101_13158_12636 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_21353_8415 M00967_43_000000000-A3JHG_1_1105_18988_15558 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1101_13333_16849 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_2113_18844_7914 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1112_26033_17779 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1110_10006_21831 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1114_13258_26764 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1110_27306_15292 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1110_15924_11287 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_2113_25465_9646 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_2109_10445_7921 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1101_15683_17782 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_1114_22494_17866 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285 M00967_43_000000000-A3JHG_1_2109_14413_2449 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7963_16361 M00967_43_000000000-A3JHG_1_2106_20458_8348 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_7963_16361 M00967_43_000000000-A3JHG_1_1111_17550_17269 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1114_12127_4319 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_1109_8693_13422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351 M00967_43_000000000-A3JHG_1_2112_15112_7605 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25166_7929 M00967_43_000000000-A3JHG_1_1110_12738_11413 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25166_7929 M00967_43_000000000-A3JHG_1_1105_15675_15142 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25166_7929 M00967_43_000000000-A3JHG_1_2104_27637_18457 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077 M00967_43_000000000-A3JHG_1_2109_25240_9047 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1102_13319_18244 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8324_8698 M00967_43_000000000-A3JHG_1_1104_17583_11048 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8324_8698 M00967_43_000000000-A3JHG_1_1110_7742_10390 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8324_8698 M00967_43_000000000-A3JHG_1_1110_19584_26279 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8324_8698 M00967_43_000000000-A3JHG_1_1101_8877_3935 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_8324_8698 M00967_43_000000000-A3JHG_1_2112_8777_8861 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2104_15263_18242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1106_25828_15956 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2112_11736_17751 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2108_12449_21256 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_2105_3878_16016 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1105_21018_4980 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589 M00967_43_000000000-A3JHG_1_1106_23502_7683 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2111_19706_18823 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1113_27297_13091 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1103_11491_13931 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_2109_8160_14127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782 M00967_43_000000000-A3JHG_1_1108_18644_12052 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1101_6977_5486 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_2103_6640_14822 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1108_24697_25438 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1114_12898_16926 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1107_7195_18281 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_1109_9649_19293 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799 M00967_43_000000000-A3JHG_1_2112_28271_11256 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_1102_14928_26203 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_2110_13599_17305 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948 M00967_43_000000000-A3JHG_1_1108_8770_12045 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21458_15751 M00967_43_000000000-A3JHG_1_1114_26968_12650 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21458_15751 M00967_43_000000000-A3JHG_1_2106_15738_17268 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_2109_10394_17608 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_1111_25950_7889 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901 M00967_43_000000000-A3JHG_1_2105_24795_12844 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2108_21459_14649 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1101_11035_15765 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2112_13923_24399 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1105_21138_11762 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2102_17579_3448 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_2102_17415_24873 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1102_13319_18244 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_1101_28516_16324 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_2108_17025_20887 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_2107_11302_8835 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_2106_20224_25526 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_2105_18687_19607 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_2103_23303_18988 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_2103_23519_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_1110_14814_10944 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_1114_3483_21724 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_1102_11222_12600 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455 M00967_43_000000000-A3JHG_1_1112_15982_3044 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1114_16655_8470 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2113_13714_22583 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435 M00967_43_000000000-A3JHG_1_1108_10305_19494 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081 M00967_43_000000000-A3JHG_1_2101_6223_8694 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_2103_22029_15973 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_2106_7936_8291 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_1113_7150_17375 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_1101_18106_21085 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_2113_13419_4336 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787 M00967_43_000000000-A3JHG_1_1108_16960_19773 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_1110_5315_13833 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_1113_12104_26590 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_2113_23687_14507 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_2106_14743_23547 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_1113_7562_15957 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_2114_24618_11376 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_1104_29281_13264 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_1113_17274_13189 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_1106_7807_21999 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181 M00967_43_000000000-A3JHG_1_2108_10705_11765 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15601_18021 M00967_43_000000000-A3JHG_1_1113_6044_19045 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15601_18021 M00967_43_000000000-A3JHG_1_2108_19309_25091 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15601_18021 M00967_43_000000000-A3JHG_1_1107_27199_15268 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15601_18021 M00967_43_000000000-A3JHG_1_2109_18804_14594 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1102_5880_14023 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_2105_7983_21245 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1112_24992_23873 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_1108_25479_20227 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_2102_12357_13497 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395 M00967_43_000000000-A3JHG_1_2109_25923_16574 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_2102_5706_8003 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_2111_17711_14171 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_2112_18144_24421 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643 M00967_43_000000000-A3JHG_1_2108_7561_16231 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2106_8679_15164 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714 M00967_43_000000000-A3JHG_1_2103_22490_8729 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_2113_4494_9088 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_2105_7943_8043 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466 M00967_43_000000000-A3JHG_1_2101_24924_5766 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128 M00967_43_000000000-A3JHG_1_1109_12799_6921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808 M00967_43_000000000-A3JHG_1_1105_20257_16697 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1103_26302_22051 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1106_21550_17093 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619 M00967_43_000000000-A3JHG_1_1114_6229_5326 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_2101_6223_8694 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039 M00967_43_000000000-A3JHG_1_1106_6004_8081 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2104_17227_9135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2103_24176_5781 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360 M00967_43_000000000-A3JHG_1_2112_10335_11049 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1109_12799_6921 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_1101_25286_19670 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2109_9943_10858 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2101_7382_19629 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_1106_20644_25868 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_1102_3871_8294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2112_8019_22340 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2106_10832_13902 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_1111_18784_23504 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2111_24817_7965 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_1108_10511_13856 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2102_16491_2474 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_2101_17218_22721 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987 M00967_43_000000000-A3JHG_1_1106_5363_6161 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1110_27941_14131 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2101_6223_8694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_1106_6004_8081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892 M00967_43_000000000-A3JHG_1_2112_24286_16039 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531 M00967_43_000000000-A3JHG_1_2112_13415_4791 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2111_17860_28348 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2108_12975_25037 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_2114_18566_5886 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1113_19148_12981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1114_5058_10624 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1109_10494_10758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_1113_18700_9502 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717 M00967_43_000000000-A3JHG_1_2102_20055_23053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1107_12430_16983 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2109_19574_4320 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2109_19808_24467 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1104_22299_24998 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_1109_18240_15644 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991 M00967_43_000000000-A3JHG_1_2110_22930_12602 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1101_25610_23225 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2113_19649_18554 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_2112_12800_22855 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1102_22379_19804 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1103_27632_18343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366 M00967_43_000000000-A3JHG_1_1101_11538_27185 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1109_15598_4401 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1103_24460_16041 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2104_23298_12028 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_1114_7920_25776 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980 M00967_43_000000000-A3JHG_1_2112_2702_12868 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_1105_20257_16697 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969 M00967_43_000000000-A3JHG_1_2114_7386_6808 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1103_7329_17775 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1102_13449_7439 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_2110_6116_13104 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1101_3758_10627 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1108_27354_15573 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_2110_26547_22871 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1107_24903_5978 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1112_16230_18061 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1105_20465_8611 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_2107_18354_8608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_2101_22555_16416 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_2104_9943_6220 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1110_18205_23899 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155 M00967_43_000000000-A3JHG_1_1112_18755_1824 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_1105_15701_17582 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_1112_17916_7193 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_1103_7913_12679 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_2113_24996_25271 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_1108_13402_18823 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_2114_7946_17808 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343 M00967_43_000000000-A3JHG_1_2111_7971_11807 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8408_13436 M00967_43_000000000-A3JHG_1_2111_5664_6671 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8408_13436 M00967_43_000000000-A3JHG_1_1108_10911_24264 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8408_13436 M00967_43_000000000-A3JHG_1_2107_20718_23357 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_8408_13436 M00967_43_000000000-A3JHG_1_2101_10084_22191 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7871_10547 M00967_43_000000000-A3JHG_1_1108_10911_24264 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_7871_10547 M00967_43_000000000-A3JHG_1_2102_8408_13436 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8550_6381 M00967_43_000000000-A3JHG_1_1105_15878_26239 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_8550_6381 M00967_43_000000000-A3JHG_1_1107_15635_4811 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_2109_25240_9047 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732 M00967_43_000000000-A3JHG_1_1111_20322_27077 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_1102_14928_26203 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_2110_13599_17305 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_1108_8770_12045 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018 M00967_43_000000000-A3JHG_1_1109_4756_16948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2112_9712_10612 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2107_22341_10832 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2107_12814_4583 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2105_9386_12253 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2114_11892_8192 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1109_10864_8146 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804 M00967_43_000000000-A3JHG_1_1108_18644_12052 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1101_24025_14108 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2107_17498_8085 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1114_22108_9150 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1110_26933_11956 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1103_25170_16709 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2114_15255_5535 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1111_23754_16069 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1110_7693_5256 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2104_21630_21997 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2102_15622_9697 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_1112_11370_15670 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2109_25599_13567 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2108_25850_8754 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681 M00967_43_000000000-A3JHG_1_2104_20227_11495 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1101_28553_10567 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2112_26066_19401 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2104_18261_8446 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2103_19578_6645 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1104_23257_9242 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1111_16200_26616 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2112_14726_9028 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2113_21899_25122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1114_17203_1583 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1109_15598_4401 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1103_24460_16041 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2104_23298_12028 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_1114_7920_25776 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2112_2702_12868 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887 M00967_43_000000000-A3JHG_1_2107_11226_20980 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1110_8805_12834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2102_21177_5197 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2103_24176_5781 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2112_10335_11049 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2111_17074_12360 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1101_11754_11390 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_2113_20319_11310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1110_15744_23114 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_1110_18711_9702 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007 M00967_43_000000000-A3JHG_1_2113_21647_20109 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2107_14648_14535 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_16544_20390 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_2109_25240_9047 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1111_20322_27077 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170 M00967_43_000000000-A3JHG_1_1114_25131_8732 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1104_14019_23841 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430 M00967_43_000000000-A3JHG_1_1114_11189_15616 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10106_14274 M00967_43_000000000-A3JHG_1_1110_17653_8119 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10106_14274 M00967_43_000000000-A3JHG_1_2101_12402_11394 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740 M00967_43_000000000-A3JHG_1_1104_19441_6353 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740 M00967_43_000000000-A3JHG_1_2104_15263_18242 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740 M00967_43_000000000-A3JHG_1_1102_22955_21067 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740 M00967_43_000000000-A3JHG_1_2102_10158_13191 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740 M00967_43_000000000-A3JHG_1_1103_20010_17589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740 M00967_43_000000000-A3JHG_1_2102_5706_8003 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.01953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_26372_17971 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2107_17002_3531 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2106_8679_15164 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_2103_22490_8729 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824 M00967_43_000000000-A3JHG_1_1103_23742_6714 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21619_15325 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21619_15325 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21619_15325 M00967_43_000000000-A3JHG_1_1101_4346_14940 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21619_15325 M00967_43_000000000-A3JHG_1_1114_27071_8397 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_2112_7837_7502 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1104_11494_18542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_2104_9988_18586 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_2102_12732_5062 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196 M00967_43_000000000-A3JHG_1_1113_6843_21123 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21768_26571 M00967_43_000000000-A3JHG_1_1110_12738_11413 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21768_26571 M00967_43_000000000-A3JHG_1_1105_15675_15142 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21768_26571 M00967_43_000000000-A3JHG_1_2104_27637_18457 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_21768_26571 M00967_43_000000000-A3JHG_1_1111_25166_7929 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1101_18044_1900 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2105_19677_24897 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2105_23104_8731 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1103_27444_10708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2103_23355_23413 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1107_7612_7844 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2114_8017_5788 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2114_18566_5886 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1110_3587_9347 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1104_24362_5528 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1113_19148_12981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1114_5058_10624 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1109_10494_10758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_1113_18700_9502 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2102_20055_23053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040 M00967_43_000000000-A3JHG_1_2108_12620_23717 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18791_11806 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18791_11806 M00967_43_000000000-A3JHG_1_1103_5171_14027 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18791_11806 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18791_11806 M00967_43_000000000-A3JHG_1_2101_13551_2912 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_18791_11806 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2110_10107_20150 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2110_13110_1977 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2113_13714_22583 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821 M00967_43_000000000-A3JHG_1_1107_21227_17435 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1110_10033_25797 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_2101_7254_13419 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1106_17680_15930 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1109_9215_20566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1114_22942_23318 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_2114_17062_26446 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273 M00967_43_000000000-A3JHG_1_1114_9915_4609 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_2104_9326_20908 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1111_23503_22763 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1105_9050_25130 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1105_12786_23414 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626 M00967_43_000000000-A3JHG_1_1104_23849_8759 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_1105_20257_16697 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758 M00967_43_000000000-A3JHG_1_2114_7386_6808 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_2105_3878_16016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1105_21018_4980 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1106_23502_7683 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134 M00967_43_000000000-A3JHG_1_1110_26616_17589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2112_11736_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2108_12449_21256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_2105_3878_16016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1105_21018_4980 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1106_23502_7683 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1110_26616_17589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790 M00967_43_000000000-A3JHG_1_1103_23234_7134 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325 M00967_43_000000000-A3JHG_1_2114_10324_8129 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1104_10606_3096 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1102_18640_14309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1114_3366_17894 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1112_5981_8948 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343 M00967_43_000000000-A3JHG_1_1113_17095_9759 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1104_11494_18542 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_2104_9988_18586 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_2102_12732_5062 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1113_6843_21123 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299 M00967_43_000000000-A3JHG_1_1109_22795_14196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386 M00967_43_000000000-A3JHG_1_1107_15635_4811 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386 M00967_43_000000000-A3JHG_1_2112_17281_21576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386 M00967_43_000000000-A3JHG_1_1104_7782_10984 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386 M00967_43_000000000-A3JHG_1_2114_24274_20575 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386 M00967_43_000000000-A3JHG_1_1103_7754_25974 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386 M00967_43_000000000-A3JHG_1_1114_8550_6381 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2111_17711_14171 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2112_18144_24421 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_2108_7561_16231 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041 M00967_43_000000000-A3JHG_1_1104_14415_16643 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1102_20832_5250 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1103_12896_14027 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1105_10076_22271 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2102_6445_10536 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2106_19460_3735 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2113_18085_11993 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1112_21500_6901 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1108_18335_28660 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2112_21705_13651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1110_20540_12162 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2114_4913_11992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2111_28638_18003 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2103_23083_4972 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1113_13627_24399 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1111_8225_24102 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1106_19006_23053 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1102_21121_24835 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2103_15669_23137 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1112_17991_13659 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1101_25610_23225 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2113_19649_18554 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2112_12800_22855 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1102_22379_19804 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1103_27632_18343 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_1101_11538_27185 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226 M00967_43_000000000-A3JHG_1_2107_20457_11366 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1111_18825_20030 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_25342_9197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.044" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2104_17227_9135 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2103_24176_5781 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2112_10335_11049 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_2111_17074_12360 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_2101_13130_17466 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1114_23025_24395 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1108_14370_23848 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526 M00967_43_000000000-A3JHG_1_1114_27071_8397 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_14544_23278 M00967_43_000000000-A3JHG_1_1104_11668_16421 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03516" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_2109_25240_9047 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1111_20322_27077 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1114_25131_8732 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958 M00967_43_000000000-A3JHG_1_1112_13372_27170 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1110_2564_19006 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1110_20095_8247 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_2108_23299_14833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1113_14260_20208 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790 M00967_43_000000000-A3JHG_1_1110_14688_5568 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_15345_25846 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2113_13714_22583 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1108_10305_19494 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1107_21227_17435 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007 M00967_43_000000000-A3JHG_1_1106_5336_9821 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17004_17030 M00967_43_000000000-A3JHG_1_1101_13158_12636 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17004_17030 M00967_43_000000000-A3JHG_1_1105_18988_15558 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_17004_17030 M00967_43_000000000-A3JHG_1_1112_21353_8415 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15065_8580 M00967_43_000000000-A3JHG_1_1110_20816_14150 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15065_8580 M00967_43_000000000-A3JHG_1_2110_19140_21811 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2109_15065_8580 M00967_43_000000000-A3JHG_1_2101_13838_9027 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2109_25240_9047 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1111_20322_27077 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1114_25131_8732 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_1112_13372_27170 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922 M00967_43_000000000-A3JHG_1_2110_25915_23958 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355 M00967_43_000000000-A3JHG_1_1104_7522_5556 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355 M00967_43_000000000-A3JHG_1_1112_20884_13058 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355 M00967_43_000000000-A3JHG_1_1111_15129_15739 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355 M00967_43_000000000-A3JHG_1_1109_24109_8015 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355 M00967_43_000000000-A3JHG_1_1108_2644_15119 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355 M00967_43_000000000-A3JHG_1_1105_26683_22024 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1102_22085_12796 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1107_21793_13370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1114_14259_12215 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_2114_20575_21363 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1111_9466_21791 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1112_25336_20130 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_2110_10010_14992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1113_13032_6958 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_1104_8057_17803 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_2108_26075_20260 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_2101_6909_12719 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673 M00967_43_000000000-A3JHG_1_2101_17437_26807 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1101_18693_22396 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_2104_18525_11294 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357 M00967_43_000000000-A3JHG_1_1112_21719_22520 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1101_14168_10823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1102_16041_22886 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1104_17560_17233 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1101_18143_13375 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2108_24812_22312 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2111_12166_13729 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1111_17009_11320 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2105_12511_24138 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2102_17332_24887 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1114_23713_17193 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1104_24989_12614 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1112_24425_19170 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1111_7592_13444 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2113_27299_8066 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1108_11757_26753 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2113_27238_12024 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2110_24102_23570 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1107_15708_17981 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1105_27980_8986 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2112_11736_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2108_12449_21256 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_2105_3878_16016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1105_21018_4980 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1106_23502_7683 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1110_26616_17589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1103_23234_7134 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532 M00967_43_000000000-A3JHG_1_1103_18369_27790 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1101_13595_8524 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2103_14570_24653 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1106_7357_12797 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2105_6844_12008 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1107_22525_22794 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2104_10372_21530 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1112_15029_25626 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2106_17741_19018 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1114_12357_13871 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2110_15016_5261 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2108_12744_24921 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1101_25551_18277 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2105_15118_19690 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1104_10691_10850 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1114_28534_17138 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_2104_19605_13841 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535 M00967_43_000000000-A3JHG_1_1103_6514_8778 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1103_26302_22051 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1106_21550_17093 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_1114_6229_5326 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722 M00967_43_000000000-A3JHG_1_2114_5448_14619 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2106_20168_3238 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2111_17711_14171 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2111_7971_11807 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1107_21227_17435 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_1106_5336_9821 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101 M00967_43_000000000-A3JHG_1_2109_6753_10007 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682 M00967_43_000000000-A3JHG_1_2105_7116_11773 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_1103_9123_12021 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_1110_5736_9217 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_2108_16551_3455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_1103_8311_8476 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_2111_13819_4746 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_2110_19314_26014 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131 M00967_43_000000000-A3JHG_1_2103_26747_11121 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042 M00967_43_000000000-A3JHG_1_2110_24314_8984 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1110_2564_19006 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_1110_20095_8247 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2108_23299_14833 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683 M00967_43_000000000-A3JHG_1_2110_21476_15790 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101 M00967_43_000000000-A3JHG_1_2109_14117_4744 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101 M00967_43_000000000-A3JHG_1_2113_9766_20824 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101 M00967_43_000000000-A3JHG_1_1108_18422_23262 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101 M00967_43_000000000-A3JHG_1_1106_23262_25144 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101 M00967_43_000000000-A3JHG_1_2102_16924_2551 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481 M00967_43_000000000-A3JHG_1_1113_11760_14772 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.028" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2105_7116_11773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319 M00967_43_000000000-A3JHG_1_2104_24218_17682 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_18278_3345 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1107_15327_17780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1110_7426_18774 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2113_12032_3782 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1107_22659_17429 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1105_10652_23248 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1102_25912_14594 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1114_10520_9402 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1107_21227_17435 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_1106_5336_9821 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2109_6753_10007 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2104_8663_16101 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991 M00967_43_000000000-A3JHG_1_2103_12547_24481 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1103_26302_22051 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1106_21550_17093 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_1114_6229_5326 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2114_5448_14619 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550 M00967_43_000000000-A3JHG_1_2105_13267_15722 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16841_5683 M00967_43_000000000-A3JHG_1_1102_27488_14514 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16841_5683 M00967_43_000000000-A3JHG_1_1103_11725_18144 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16841_5683 M00967_43_000000000-A3JHG_1_1110_24287_19984 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_16841_5683 M00967_43_000000000-A3JHG_1_1111_19252_26342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_2107_13016_9270 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1114_19529_17837 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1102_24841_18959 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_2114_16137_10353 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430 M00967_43_000000000-A3JHG_1_1106_24555_20879 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127 M00967_43_000000000-A3JHG_1_2110_22245_24576 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_1108_19250_12721 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_2109_18345_18844 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_2102_14512_24313 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_2105_9580_7709 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702 M00967_43_000000000-A3JHG_1_2105_23153_7113 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1101_5883_17133 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1114_12471_18325 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1113_18958_10702 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1110_4449_10146 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1110_3713_15410 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1102_12322_7970 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1114_18793_4661 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_2114_20314_10028 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_2111_17688_14654 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1114_18112_24383 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1112_10190_23278 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1103_16766_20121 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1114_13983_6574 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_2104_4913_14332 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1107_25872_9951 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1113_6747_17324 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_2103_17104_22073 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116 M00967_43_000000000-A3JHG_1_1110_15277_6482 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1109_8914_20463 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_2102_10248_4992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141 M00967_43_000000000-A3JHG_1_1113_28094_17280 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1101_23238_24359 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1110_20095_8247 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2108_23299_14833 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_1110_14688_5568 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2110_21476_15790 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916 M00967_43_000000000-A3JHG_1_2103_7853_22683 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2112_13415_4791 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040 M00967_43_000000000-A3JHG_1_2110_11592_18531 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1105_8356_13589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_2109_27058_11840 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_2106_23285_7924 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391 M00967_43_000000000-A3JHG_1_1110_7275_13443 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1114_8829_4071 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2109_12163_4647 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_2103_22490_8729 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326 M00967_43_000000000-A3JHG_1_1103_23742_6714 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_2113_24913_16194 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_2112_7837_7502 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_2102_12732_5062 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1113_6843_21123 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115 M00967_43_000000000-A3JHG_1_1101_24745_24299 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16971_16405 M00967_43_000000000-A3JHG_1_2111_6887_10490 0.003937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16971_16405 M00967_43_000000000-A3JHG_1_1102_17763_13764 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16971_16405 M00967_43_000000000-A3JHG_1_2103_12503_10753 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16971_16405 M00967_43_000000000-A3JHG_1_1105_11497_20872 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_16971_16405 M00967_43_000000000-A3JHG_1_1110_16853_2840 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2112_13415_4791 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_2110_11592_18531 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910 M00967_43_000000000-A3JHG_1_1113_11975_6040 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1104_18371_18182 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1108_18770_23200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1106_8718_15420 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1108_21122_24729 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_2108_16914_2576 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_2101_13130_17466 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1114_23025_24395 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1108_14370_23848 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1106_15986_21920 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_2112_8528_6078 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_1114_27071_8397 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046 M00967_43_000000000-A3JHG_1_2111_19511_20526 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2113_16554_11081 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_18244_26058 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_17088_10563 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_27414_13728 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552 M00967_43_000000000-A3JHG_1_1113_8826_23586 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21850_8958 M00967_43_000000000-A3JHG_1_2102_7041_13746 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21850_8958 M00967_43_000000000-A3JHG_1_1110_17368_18443 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1110_27941_14131 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2101_6223_8694 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_1106_6004_8081 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2112_24286_16039 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898 M00967_43_000000000-A3JHG_1_2110_19846_20892 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1109_17953_9880 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_2110_22245_24576 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827 M00967_43_000000000-A3JHG_1_1114_9494_11127 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_2112_6888_8092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1103_24229_16660 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_2110_24147_19370 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_2101_20223_25753 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_2109_7813_4701 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608 M00967_43_000000000-A3JHG_1_1114_11237_4361 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1109_12799_6921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2110_24314_8984 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276 M00967_43_000000000-A3JHG_1_2104_17794_25042 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2101_6223_8694 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1106_6004_8081 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2112_24286_16039 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_2110_19846_20892 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978 M00967_43_000000000-A3JHG_1_1110_21794_19898 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2110_24314_8984 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_2104_17794_25042 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110 M00967_43_000000000-A3JHG_1_1109_16375_7276 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1104_14019_23841 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1114_11189_15616 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415 M00967_43_000000000-A3JHG_1_1112_11092_24430 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_1114_22804_12324 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_2113_11757_15493 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_1101_21305_8478 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_2108_20651_22709 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324 M00967_43_000000000-A3JHG_1_1101_4190_14354 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1103_20010_17589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1110_14688_5568 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_2110_22245_24576 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1114_9494_11127 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228 M00967_43_000000000-A3JHG_1_1110_12977_5827 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7344_20185 M00967_43_000000000-A3JHG_1_2104_12505_3424 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7344_20185 M00967_43_000000000-A3JHG_1_1103_5754_14689 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7344_20185 M00967_43_000000000-A3JHG_1_1114_20440_13060 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5459_18973 M00967_43_000000000-A3JHG_1_1112_10024_21424 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5459_18973 M00967_43_000000000-A3JHG_1_1114_21549_4010 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5459_18973 M00967_43_000000000-A3JHG_1_1102_5327_17123 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_5459_18973 M00967_43_000000000-A3JHG_1_2110_11463_19602 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26352_22378 M00967_43_000000000-A3JHG_1_1114_26968_12650 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26352_22378 M00967_43_000000000-A3JHG_1_2106_15738_17268 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_26352_22378 M00967_43_000000000-A3JHG_1_1109_21458_15751 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_1101_9620_19745 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_1114_18190_18406 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_1113_28366_19519 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_2107_5824_20256 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_2109_14780_27058 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524 M00967_43_000000000-A3JHG_1_1110_13950_25143 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554 M00967_43_000000000-A3JHG_1_2102_22604_4240 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554 M00967_43_000000000-A3JHG_1_1107_14329_23174 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554 M00967_43_000000000-A3JHG_1_2109_10394_17608 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554 M00967_43_000000000-A3JHG_1_1111_25950_7889 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554 M00967_43_000000000-A3JHG_1_2105_24795_12844 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554 M00967_43_000000000-A3JHG_1_1109_11295_11901 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1112_27485_13416 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2107_26555_14955 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1109_8014_5010 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2106_6909_9337 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1109_17953_9880 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1108_27163_22727 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_2110_22245_24576 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1114_9494_11127 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621 M00967_43_000000000-A3JHG_1_1110_12977_5827 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_2107_13016_9270 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1114_19529_17837 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1102_24841_18959 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_2114_16137_10353 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_1106_24555_20879 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900 M00967_43_000000000-A3JHG_1_2101_10104_19430 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1101_9813_20001 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1110_10033_25797 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1103_26542_17589 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1106_17680_15930 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1109_9215_20566 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_2114_17062_26446 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1114_9915_4609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083 M00967_43_000000000-A3JHG_1_1105_23153_7273 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_2114_10324_8129 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801 M00967_43_000000000-A3JHG_1_1102_8406_20325 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1101_17674_2779 0.007968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2103_7701_13077 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2104_4696_10935 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2111_21066_7506 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1107_26583_8053 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1113_13883_23943 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1111_12107_16431 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1105_27583_14497 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1102_14615_8833 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2113_9583_13925 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2113_17952_9366 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2110_5588_24334 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2107_12844_13432 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1114_21829_7068 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1113_15168_11856 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1111_21833_21040 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2106_17117_12389 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1105_10343_20641 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1101_9360_18060 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2112_24725_10550 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2110_27146_22098 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1110_25219_18794 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1101_3782_8109 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2105_25408_18555 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1111_23416_24696 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1104_20592_3815 0.04382" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1106_15436_27017 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2109_19279_7836 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2103_10962_17123 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1114_6688_5293 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1110_27797_16769 0.01594" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1105_25694_18609 0.03984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1104_24270_11650 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1103_16540_20675 0.03187" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1113_20973_15490 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1109_16625_10134 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1109_16714_23075 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2113_19864_14092 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2108_11818_4055 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2107_20507_18628 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2104_26030_6786 0.02789" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2101_6223_8694 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1106_6004_8081 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2112_24286_16039 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_2110_19846_20892 0.03586" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1110_21794_19898 0.0239" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955 M00967_43_000000000-A3JHG_1_1108_19892_23978 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1101_18415_21676 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_2104_9975_21883 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1110_9936_12702 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1106_26087_11342 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1113_11017_18921 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_2110_11085_20271 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1114_10055_21108 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1103_23750_17910 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_2113_8991_5702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_1101_21197_21065 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_2113_21308_17042 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747 M00967_43_000000000-A3JHG_1_2106_17640_11873 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13340_15811 M00967_43_000000000-A3JHG_1_1111_22890_9747 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13340_15811 M00967_43_000000000-A3JHG_1_1109_14359_12484 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13340_15811 M00967_43_000000000-A3JHG_1_1108_10061_3691 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_13340_15811 M00967_43_000000000-A3JHG_1_1104_15705_5052 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1105_13657_20414 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1114_25962_10907 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2114_14336_9476 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2109_21549_9949 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2103_14234_20506 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1112_8821_13863 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2101_28391_18939 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1107_17052_3882 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2107_13720_13200 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1102_27837_16697 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2107_27604_15299 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1105_13597_20997 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2112_8686_5552 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1106_10456_18286 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2114_14906_9268 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2111_20415_6804 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2109_7315_17427 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2109_27247_19486 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2112_12631_18383 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2113_7399_10515 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1110_14855_8462 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2111_21247_15845 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_2103_15482_15086 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1107_9318_10158 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1103_21608_2864 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863 M00967_43_000000000-A3JHG_1_1105_22535_20381 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1101_17289_26831 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1112_15323_28130 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2105_21230_21464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2111_27603_17495 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2103_24284_21559 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2113_20319_11310 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1110_15744_23114 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1110_18711_9702 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_2113_21647_20109 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324 M00967_43_000000000-A3JHG_1_1113_19563_8007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11983_2593 M00967_43_000000000-A3JHG_1_1109_22380_20532 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11983_2593 M00967_43_000000000-A3JHG_1_2102_14252_8710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11983_2593 M00967_43_000000000-A3JHG_1_2104_20074_25538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_11983_2593 M00967_43_000000000-A3JHG_1_1111_13772_8177 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2106_8106_16988 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1109_8914_20463 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_2102_10248_4992 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1113_28094_17280 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546 M00967_43_000000000-A3JHG_1_1113_9372_3141 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2103_17221_27256 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2111_10149_9721 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1114_19514_5000 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2112_25145_5016 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1109_8914_20463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_2102_10248_4992 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1113_28094_17280 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1113_9372_3141 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713 M00967_43_000000000-A3JHG_1_1109_24372_12546 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2110_18978_25958 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2102_23347_17242 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2111_17711_14171 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2112_18144_24421 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1104_14415_16643 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_1106_5336_9821 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2114_19251_6041 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911 M00967_43_000000000-A3JHG_1_2104_8663_16101 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2101_4572_15105 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.024" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2105_7116_11773 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2104_24218_17682 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094 M00967_43_000000000-A3JHG_1_2102_5906_19319 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2105_11308_6301 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1110_15855_19234 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1101_16007_27282 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1105_17600_8589 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2113_26675_15471 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1111_12028_24705 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1106_5336_9821 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1112_2989_20122 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123 M00967_43_000000000-A3JHG_1_1110_12096_22703 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1110_14688_5568 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_2110_22245_24576 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1114_9494_11127 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1110_12977_5827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1106_8681_8228 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284 M00967_43_000000000-A3JHG_1_1106_18033_2621 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2114_22468_4333 M00967_43_000000000-A3JHG_1_2109_18443_5251 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2108_19062_16604 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03502" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_14790_1829 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2110_11195_7814 0.003922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1107_21132_10931 0.01562" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2101_26784_17549 0.01172" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2113_24485_10653 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2105_7169_20746 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.04297" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.03891" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2112_11494_8595 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_2106_8679_15164 0.03906" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1103_23742_6714 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671 M00967_43_000000000-A3JHG_1_1110_22982_25824 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1102_8823_6749 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1101_15591_4696 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_2103_22514_6355 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1112_24306_21531 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_2111_8186_13301 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1113_20075_16901 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1103_3710_21770 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_2106_12578_10682 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1111_15312_18978 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_2102_27720_18606 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1105_11157_16891 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1111_24385_9596 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1101_18693_22396 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_2104_18525_11294 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_1112_21719_22520 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524 M00967_43_000000000-A3JHG_1_2107_11282_7357 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1113_8826_23586 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608 M00967_43_000000000-A3JHG_1_1110_4126_16552 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2112_13415_4791 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_2110_11592_18531 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1113_11975_6040 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664 M00967_43_000000000-A3JHG_1_1111_12881_9910 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2101_12255_19099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_1113_11760_14772 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2103_12547_24481 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1112_11343_21537 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1111_17002_7999 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_2108_17613_16563 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_2113_12446_11570 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_2103_7856_11382 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_2101_9084_14502 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1111_12899_5675 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1104_5274_12644 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1110_19653_28369 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1106_22826_4799 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111 M00967_43_000000000-A3JHG_1_1114_4902_9630 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1113_8826_23586 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_1110_4126_16552 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243 M00967_43_000000000-A3JHG_1_2112_25043_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2110_24314_8984 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_2104_17794_25042 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1109_16375_7276 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963 M00967_43_000000000-A3JHG_1_1108_14383_15110 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1101_17802_26911 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1103_25888_23585 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_2110_13298_25616 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1114_13389_13295 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1109_12715_19004 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1108_12975_13764 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1104_14213_12414 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_2109_14974_11618 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1101_11754_11390 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_2111_13555_13333 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_2113_20319_11310 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1110_15744_23114 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1110_18711_9702 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_2113_21647_20109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1113_19563_8007 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961 M00967_43_000000000-A3JHG_1_1102_11664_24324 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_2106_12968_23126 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_1101_8537_7072 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_2113_22451_22884 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_2101_25094_24210 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_1102_14928_26203 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_2110_13599_17305 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_1108_8770_12045 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_1109_4756_16948 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521 M00967_43_000000000-A3JHG_1_1114_22706_5018 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_1101_7265_9662 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_1101_20653_21248 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_2112_18460_8495 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_1113_12062_8139 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_2105_24712_15313 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_1113_28366_19519 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_2107_5824_20256 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_2109_14780_27058 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_1110_13950_25143 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228 M00967_43_000000000-A3JHG_1_1106_25959_18524 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23657_12369 M00967_43_000000000-A3JHG_1_1110_18313_3175 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23657_12369 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23657_12369 M00967_43_000000000-A3JHG_1_1101_11480_8830 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23657_12369 M00967_43_000000000-A3JHG_1_1107_7195_18281 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23657_12369 M00967_43_000000000-A3JHG_1_1108_13402_18823 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1101_18089_2781 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1109_21243_9500 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2102_12696_27241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1113_19532_4899 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1105_14547_12843 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2109_9416_24199 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2109_15226_21168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2109_11269_19733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2106_17102_4744 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2106_16184_19905 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1103_22674_6370 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1101_20815_21052 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1107_25867_16609 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2114_10618_18178 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2102_7679_12922 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2102_19327_26344 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1101_22681_5598 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1106_5748_9323 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2106_18656_26472 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1106_3005_18956 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1113_16033_3594 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1101_8786_10844 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1107_15496_5008 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2112_26672_16660 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2109_6453_7009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_2109_21370_14363 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1111_23503_22763 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1105_9050_25130 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1105_12786_23414 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1102_16656_25520 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1104_23849_8759 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458 M00967_43_000000000-A3JHG_1_1103_7927_15626 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2113_21352_15666 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_1102_6990_17751 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2111_17860_28348 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2108_12975_25037 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2112_18311_25065 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1114_7103_21358 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1112_18213_15560 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1113_14703_18340 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1106_23455_4853 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2109_23522_9652 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2105_22379_16929 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2110_24314_8984 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2104_17794_25042 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1109_16375_7276 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_1108_14383_15110 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271 M00967_43_000000000-A3JHG_1_2110_24113_10963 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2101_14628_22758 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_1113_11232_14725 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_1108_14299_17220 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2113_23848_13201 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2109_20493_16450 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_1114_17730_26068 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_1106_13721_7517 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_1105_11983_14742 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2102_17714_13657 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_1109_24246_18294 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743 M00967_43_000000000-A3JHG_1_2110_5187_18269 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515 M00967_43_000000000-A3JHG_1_1102_6552_16619 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515 M00967_43_000000000-A3JHG_1_1110_13879_1994 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515 M00967_43_000000000-A3JHG_1_2108_22087_14118 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515 M00967_43_000000000-A3JHG_1_2110_10010_14992 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515 M00967_43_000000000-A3JHG_1_2108_26075_20260 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515 M00967_43_000000000-A3JHG_1_2101_17437_26807 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1111_13451_28512 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1109_20401_24219 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2108_27497_14930 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2106_12583_13406 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1113_21762_27306 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2112_17214_15927 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2111_23260_18439 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2111_10081_26254 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1114_24430_23737 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2111_5639_10651 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2108_17448_14122 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1105_16167_11796 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1101_18682_27059 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1110_29313_18090 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1108_12332_11456 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2114_17959_5140 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1113_14734_11233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1113_15529_5161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1101_18517_10721 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1112_18981_20025 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2104_17227_9135 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2103_24176_5781 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2112_10335_11049 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2111_17074_12360 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2112_13608_21597 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_13686_15465 M00967_43_000000000-A3JHG_1_1114_22681_21130 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2106_13686_15465 M00967_43_000000000-A3JHG_1_2107_23657_12369 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1101_5370_21051 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1107_13053_11087 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2102_12263_16584 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1105_12242_23443 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2110_16087_14724 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2113_7083_22842 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1113_21718_5711 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1114_9103_25313 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1110_29167_14391 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2106_25956_17550 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1112_19464_17673 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1106_19617_4733 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1104_15789_5235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2110_24147_19370 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2101_20223_25753 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1105_12578_19811 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_2109_7813_4701 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1114_11237_4361 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171 M00967_43_000000000-A3JHG_1_1109_7699_18608 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1103_28341_20456 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1102_10298_5101 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1113_9810_23014 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1109_24246_18294 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1106_6775_23397 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092 M00967_43_000000000-A3JHG_1_1101_11952_13519 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1101_15031_12738 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1111_20235_16372 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_2108_25695_18492 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1114_22804_12324 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_2113_11757_15493 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1110_10693_14739 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1101_21305_8478 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_2108_20651_22709 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1101_4190_14354 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920 M00967_43_000000000-A3JHG_1_1107_22214_8324 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26300_11226 M00967_43_000000000-A3JHG_1_1113_18548_4292 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26300_11226 M00967_43_000000000-A3JHG_1_1104_14252_27883 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_26300_11226 M00967_43_000000000-A3JHG_1_1112_11192_11762 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2106_18053_21392 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2102_25569_21727 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1112_2989_20122 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086 M00967_43_000000000-A3JHG_1_1106_22705_6123 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1107_6465_14196 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1112_8674_21438 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2110_17263_11534 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1110_18810_18264 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1112_22806_22503 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2103_19878_24663 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1104_6527_16493 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2101_9500_21244 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_1114_26942_22032 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2102_25569_21727 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170 M00967_43_000000000-A3JHG_1_2112_15112_7605 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11391_8155 M00967_43_000000000-A3JHG_1_1105_15878_26239 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11391_8155 M00967_43_000000000-A3JHG_1_1107_15635_4811 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11391_8155 M00967_43_000000000-A3JHG_1_2103_7887_6353 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2103_11391_8155 M00967_43_000000000-A3JHG_1_1114_8550_6381 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_2114_10324_8129 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1102_8406_20325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452 M00967_43_000000000-A3JHG_1_1105_15500_1801 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2114_10324_8129 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1102_8406_20325 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_1105_15500_1801 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917 M00967_43_000000000-A3JHG_1_2102_25992_9452 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1101_12395_11489 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1114_19875_9916 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1110_10609_18161 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_2107_23336_11976 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1111_14398_12314 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1104_13776_17053 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1103_8412_8356 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1108_13878_20553 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1110_5826_22164 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_2113_13714_22583 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1112_9419_4295 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1107_13057_19114 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1101_16182_25089 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_2111_17711_14171 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_2112_18144_24421 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_2108_7561_16231 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1104_14415_16643 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_2114_19251_6041 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913 M00967_43_000000000-A3JHG_1_1112_18506_25911 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1107_27593_11400 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2103_9090_14716 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1108_19250_12721 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2114_26633_11062 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2105_9580_7709 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2105_23153_7113 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2113_4494_9088 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2105_7943_8043 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_2101_24924_5766 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713 M00967_43_000000000-A3JHG_1_1101_11823_22466 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1105_20257_16697 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2114_7386_6808 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_2104_10592_12969 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254 M00967_43_000000000-A3JHG_1_1103_24150_14758 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1101_13234_1983 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1107_20139_17113 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1101_7408_16842 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1114_13397_2211 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1112_12844_3769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1109_7996_16802 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1101_19009_23185 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2114_4499_18914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2110_7822_24159 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2101_13320_3436 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1114_27304_8277 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2106_5509_18056 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1110_16292_27414 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2101_25710_23156 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1101_12302_23776 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1108_5894_23389 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2110_15820_14074 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2106_8030_23327 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1112_18411_17052 0.02" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1109_24493_23253 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2113_29656_14155 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2109_28564_10017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2101_4159_12177 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1114_8165_8910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1109_19506_3309 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1104_6617_17876 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1105_16224_23592 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2114_24873_12639 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2108_7751_22068 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2106_28000_12820 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2105_18677_14481 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1113_28928_16770 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1111_4823_22009 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1106_12236_14361 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1114_22144_24942 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2113_15093_8390 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1113_15967_2074 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1103_8361_9953 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2106_18324_28644 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2105_15636_26203 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1107_13116_15821 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2114_12634_10967 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2109_24964_16600 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2108_16667_9805 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2103_4125_17157 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2101_7036_14239 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_2101_4780_15092 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1114_15498_19369 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1111_20933_6700 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1108_22866_20909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1102_13900_15927 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1101_16641_11299 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1104_20588_5990 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1104_10606_3096 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1102_18640_14309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1114_3366_17894 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1112_5981_8948 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1113_17095_9759 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067 M00967_43_000000000-A3JHG_1_1102_6774_6343 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_1113_11760_14772 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2103_12547_24481 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434 M00967_43_000000000-A3JHG_1_2111_12365_24314 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1104_26838_20823 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1101_9553_14094 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_2109_7804_9298 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1112_8060_13320 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1106_26888_10548 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1113_19332_23681 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1111_16173_17418 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_2111_22855_17407 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1109_24561_21388 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1110_26670_15463 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1110_18108_15259 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1109_8693_13422 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024 M00967_43_000000000-A3JHG_1_1111_26033_16351 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_24119_7738 M00967_43_000000000-A3JHG_1_1101_21761_13031 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1101_16061_3516 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1102_28036_9037 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2102_15542_26928 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1105_9179_20196 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2114_15143_1651 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2105_26015_12877 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2102_12782_8845 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1104_8579_3599 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2112_22596_18948 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1108_16488_9936 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2109_17316_4550 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2110_18983_13738 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1113_23768_16589 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1112_15603_7816 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2111_14657_1748 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2103_13384_15039 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1103_4372_14342 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1110_13330_10638 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2109_19976_22044 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2112_10585_20149 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2111_11460_16881 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1109_11326_21178 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2108_26358_22652 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1114_19698_19740 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1111_12315_7486 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1110_20810_3468 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1109_26818_15061 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2110_19623_12472 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_1113_11760_14772 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2103_12547_24481 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2111_12365_24314 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619 M00967_43_000000000-A3JHG_1_2101_26265_15434 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1104_11900_6285 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1112_25108_12490 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1106_23133_8169 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2111_18507_22576 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1110_2564_19006 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_1110_20095_8247 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2108_23299_14833 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2110_21476_15790 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481 M00967_43_000000000-A3JHG_1_2103_7853_22683 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2112_22954_15571 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2108_11790_16027 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2107_14096_15016 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2107_10803_20222 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1103_18945_9651 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1102_14900_7576 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1111_22362_24215 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2108_16172_11937 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2102_26214_23057 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2106_10776_6955 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1102_10298_5101 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1109_24246_18294 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_1106_6775_23397 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413 M00967_43_000000000-A3JHG_1_2105_17259_2092 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2111_17860_28348 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2108_12975_25037 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2109_23117_11899 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181 M00967_43_000000000-A3JHG_1_2107_14468_14971 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2114_10324_8129 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1102_8406_20325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_1105_15500_1801 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2102_25992_9452 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476 M00967_43_000000000-A3JHG_1_2102_19730_4917 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296 M00967_43_000000000-A3JHG_1_1106_25798_21572 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296 M00967_43_000000000-A3JHG_1_1112_27414_10900 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296 M00967_43_000000000-A3JHG_1_1109_11299_13989 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296 M00967_43_000000000-A3JHG_1_1109_23081_16781 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296 M00967_43_000000000-A3JHG_1_1114_28031_16211 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296 M00967_43_000000000-A3JHG_1_1114_16110_14718 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2102_11614_20083 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1107_27878_20060 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2104_8408_16485 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2101_23114_22755 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1112_12887_9342 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1102_12551_11293 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2104_19435_2222 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1107_26777_15267 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2112_13415_4791 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2110_11592_18531 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1113_11975_6040 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_1111_12881_9910 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281 M00967_43_000000000-A3JHG_1_2111_4991_9664 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1105_20257_16697 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2114_7386_6808 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2104_10592_12969 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_1103_24150_14758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725 M00967_43_000000000-A3JHG_1_2102_14775_20254 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2114_26633_11062 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2113_4494_9088 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2105_7943_8043 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2101_24924_5766 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_1101_11823_22466 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422 M00967_43_000000000-A3JHG_1_2102_15674_19713 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1106_18001_27099 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1112_11092_24430 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961 M00967_43_000000000-A3JHG_1_1107_23336_26415 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1101_20384_5679 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_10447_18633 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2101_22171_7080 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1104_20673_8839 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2114_20868_11538 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2113_4525_9290 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_6243_11429 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_22961_17195 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2112_10480_18043 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1110_24315_7382 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2112_6265_10463 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.016" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_18172_10876 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_18888_26858 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1108_23570_6782 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2112_13608_21597 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2105_7116_11773 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2104_24218_17682 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_2102_5906_19319 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790 M00967_43_000000000-A3JHG_1_1106_7651_17094 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1106_21254_7027 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1112_15717_14694 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1106_22453_4343 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2114_22474_15335 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1113_8826_23586 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1111_5281_15326 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_1110_4126_16552 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2112_25043_23608 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288 M00967_43_000000000-A3JHG_1_2110_25043_10243 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10723_2426 M00967_43_000000000-A3JHG_1_1105_14835_21938 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1112_10723_2426 M00967_43_000000000-A3JHG_1_1110_13033_21990 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1101_11381_14019 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1104_21303_18168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1112_27130_19681 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1105_15090_18340 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1105_20176_26018 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1103_16001_2026 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_2110_22006_25345 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_2107_19127_1886 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1113_21801_14983 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1105_8356_13589 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_2109_27058_11840 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_2106_23285_7924 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1110_7275_13443 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068 M00967_43_000000000-A3JHG_1_1112_20414_23391 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1101_12259_21251 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2104_25418_11257 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1101_10331_23332 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2107_14104_6500 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1110_9431_18167 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2106_6909_9337 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1113_19958_8814 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1106_17680_15930 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1109_9215_20566 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1111_16441_6974 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2114_18928_10051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2108_13526_22864 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2106_25719_12422 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1108_27163_22727 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2114_7006_17302 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1110_14688_5568 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2111_10343_15424 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2103_16979_24828 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2110_22245_24576 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1114_9494_11127 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_1106_8681_8228 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709 M00967_43_000000000-A3JHG_1_2114_6705_14284 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1101_21702_5444 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1111_21792_3253 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2111_13579_10781 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2101_17292_21391 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2108_14290_10968 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1114_13584_18783 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2105_15973_8357 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2108_14452_7789 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1112_11161_25428 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2112_11352_13241 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2108_6390_9974 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2112_28152_16274 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1108_24171_25394 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1114_9210_8620 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1105_13842_19281 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2112_12749_8826 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1108_23186_26439 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2112_27658_13780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2102_16184_16848 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1113_10248_22431 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1105_20257_16697 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2114_7386_6808 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2104_10592_12969 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1103_24150_14758 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_2102_14775_20254 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508 M00967_43_000000000-A3JHG_1_1113_22863_10725 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1101_21616_8560 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1112_13682_16758 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2114_10492_14599 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2112_15899_17037 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1114_14280_26421 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1111_22050_3053 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1109_23011_5924 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2111_16058_17833 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2110_23584_15948 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2106_15211_24821 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1101_11348_22601 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2111_16922_20621 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1104_15748_9889 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2110_7457_8637 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2114_16977_22333 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1112_25018_15007 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2110_15516_28587 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1107_16297_12364 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1107_13819_9393 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2114_18715_18416 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1111_26760_12620 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2107_4339_17322 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1103_22654_26891 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1112_28222_11771 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1109_8914_20463 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_2102_10248_4992 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1113_28094_17280 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1113_9372_3141 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1109_24372_12546 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155 M00967_43_000000000-A3JHG_1_1108_2714_17713 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1113_12649_18573 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2114_11245_12909 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2101_15912_14604 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1109_8911_22008 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2101_13832_24138 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1103_12866_5545 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_14993_16910 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1114_10193_19783 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1102_26403_8487 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1113_8826_23586 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1110_4126_16552 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2112_25043_23608 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_2110_25043_10243 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924 M00967_43_000000000-A3JHG_1_1112_29269_13288 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20113_10044 M00967_43_000000000-A3JHG_1_2102_28446_16991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20113_10044 M00967_43_000000000-A3JHG_1_1109_9752_18363 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_20113_10044 M00967_43_000000000-A3JHG_1_2106_12440_13627 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17950_22790 M00967_43_000000000-A3JHG_1_1104_11668_16421 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1109_17950_22790 M00967_43_000000000-A3JHG_1_2111_14544_23278 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_1105_22508_16792 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_1108_13421_28822 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_2104_25789_8271 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_1105_19059_19452 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_1110_26069_24105 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_1109_14735_26068 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_2102_25051_20557 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_2104_22154_4269 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669 M00967_43_000000000-A3JHG_1_2103_25987_20682 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1101_17656_27121 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1111_10121_22720 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2109_9657_11455 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1114_20989_19235 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2103_10169_13566 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1107_20748_16898 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1113_7625_23288 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1103_19800_26276 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1101_23281_23823 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1112_10568_25353 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2108_10109_11627 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2107_18729_12719 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2102_11553_9071 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2108_27773_22176 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1108_28526_15940 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1103_12608_3163 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1110_17818_21151 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1107_4622_21981 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2108_14981_25247 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2101_10680_16017 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1114_24795_10657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1114_16123_8166 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2114_10324_8129 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1102_8406_20325 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1105_15500_1801 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2102_25992_9452 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_2102_19730_4917 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112 M00967_43_000000000-A3JHG_1_1114_14939_20476 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1101_26181_9437 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2108_10435_17390 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1101_8995_24296 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1112_18757_14747 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2108_19943_26491 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1105_15005_21488 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1104_20104_23610 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2105_23092_9359 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2104_14573_20188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1111_14622_25632 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2109_19574_4320 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2109_19808_24467 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1104_22299_24998 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_1109_18240_15644 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2110_22930_12602 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774 M00967_43_000000000-A3JHG_1_2107_7538_13991 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_14203_2907 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_8084_12645 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1111_24599_6936 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_12761_22353 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_6259_8307 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_16038_11947 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2108_19840_26409 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2106_14305_11884 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2105_17320_11249 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2103_25452_6018 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_11964_4686 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1103_3864_17599 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_10573_19345 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_21712_25652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_23929_25607 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_7971_11346 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_17362_10105 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2102_27795_15713 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_15218_23608 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_14152_27072 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1109_24712_5827 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1109_21060_16840 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1108_14399_23420 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2104_9776_14422 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2104_25610_6877 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1112_20902_11558 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_19534_17052 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2110_10506_11014 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_25546_10365 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2102_26655_15452 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_13281_5144 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_4007_20870 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_18550_10076 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_11692_22443 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_14485_10905 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_27262_22663 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_12737_15531 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1107_5337_17921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1103_10837_19652 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1102_19038_11166 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1102_20639_13713 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_12423_13578 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_20644_19641 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2103_26632_14477 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_5425_15020 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_24111_15349 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_15548_11732 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_12882_9143 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1112_25301_17173 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_4925_16493 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_25043_9903 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_13931_12506 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1107_8474_16938 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1104_17110_15247 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1108_20614_22010 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_24616_16104 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2110_20937_6188 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_25409_11127 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1108_27653_13933 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1103_17414_11243 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_18085_25907 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_17216_4292 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_22227_7717 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_18393_28178 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_16997_10935 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_25056_22893 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2104_7819_20401 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2104_18217_19581 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2102_5852_19432 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2101_20164_2274 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_24088_5500 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_21924_11050 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_19795_28223 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_19071_5566 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_14308_17024 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1111_8296_18650 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_7766_4240 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1106_14254_12461 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_3088_15065 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1103_23925_21591 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1102_16964_23052 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_2004_15863 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_3905_12418 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_16258_26585 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_17324_3447 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2108_7645_19778 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2103_24918_13396 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2103_20753_25954 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2102_18868_8410 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2101_22980_4780 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2101_17322_21438 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_19795_24825 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_13342_26294 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1111_8141_21233 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1111_5848_24084 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1108_13873_2429 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1107_21848_23651 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_13609_10025 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2108_10602_19971 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1107_4609_18391 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_23230_12044 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2114_14201_8661 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2108_27871_9754 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2103_9222_10640 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_6495_13602 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_6141_22497 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_16174_11502 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1112_9636_6179 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1111_21999_17538 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1107_3548_11064 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1107_19357_22811 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1106_20020_16264 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1104_4327_10073 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_26634_17611 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_16914_26242 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2113_14482_15634 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2105_10122_15632 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1102_17902_26485 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_25878_10062 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2113_13275_27560 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_12022_9082 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_12205_7359 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2110_17618_20533 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2107_24787_25663 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2105_16818_5464 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2101_9650_25321 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_21383_22868 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1106_28698_12913 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_23231_4481 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1105_18275_26860 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1104_7862_4687 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1103_12270_15236 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1102_4170_17056 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_7373_14922 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1106_4308_15219 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1104_18476_6578 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_22417_22191 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_29414_16449 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2106_17127_12464 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2101_12238_5807 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_21607_13167 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1111_13586_2804 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_19644_17655 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1106_28442_12734 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2111_7333_20307 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2109_9362_22992 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2107_13773_19305 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2107_13406_20132 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2106_23373_7365 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2107_22934_10355 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2106_8679_15164 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1108_21905_17218 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1103_23742_6714 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1101_15011_15369 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1114_13266_17804 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_22982_25824 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1113_8826_23586 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1110_4126_16552 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2112_25043_23608 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_2110_25043_10243 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1112_29269_13288 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228 M00967_43_000000000-A3JHG_1_1109_6553_9924 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1112_11092_24430 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1107_23336_26415 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420 M00967_43_000000000-A3JHG_1_1113_18370_24961 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_1109_15887_5905 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_2112_20678_18094 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_1104_27578_17650 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_1104_27702_14915 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_2114_14273_14388 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_1112_24714_23272 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_1111_17207_6739 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_2108_14707_9807 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453 M00967_43_000000000-A3JHG_1_2105_18816_26393 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1101_22481_5236 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1101_21195_10623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1102_17084_15866 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1102_24688_14454 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1114_21222_13060 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1104_10834_8009 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2105_29107_15046 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1108_18821_5778 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1113_26886_12231 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2107_12316_24495 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2101_13474_15202 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1114_14691_17449 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1104_6949_16896 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2104_20064_5598 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1104_6527_16493 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1102_7847_17999 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1114_26942_22032 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2102_25569_21727 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1104_12197_8664 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_1114_23027_15678 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604 M00967_43_000000000-A3JHG_1_2101_24905_9024 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_1104_13822_10866 0.003953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2101_20023_3542 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2103_11954_7925 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2106_10304_14471 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2104_11759_16153 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2109_19731_20280 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_1104_5458_8163 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2111_25352_17924 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_1108_9239_22168 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2114_10524_26952 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2113_16978_21215 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2112_10029_17181 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2102_23398_14571 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_2104_11929_10165 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_1110_16691_26561 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680 M00967_43_000000000-A3JHG_1_1114_9024_20163 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2103_13201_17677 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1104_14019_23841 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1114_11189_15616 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1112_11092_24430 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1107_23336_26415 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1113_18370_24961 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563 M00967_43_000000000-A3JHG_1_1107_21734_25420 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1101_19252_7867 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1103_5231_15971 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1102_21369_2858 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2112_26773_14017 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2107_26817_14749 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1112_17696_15281 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2109_6882_14781 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1102_11941_12226 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2112_10013_23090 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2111_6592_20714 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1110_10946_20858 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2112_23298_24981 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2108_6233_13205 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2106_9462_24856 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1101_12446_20708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1103_12195_22317 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1114_8377_12333 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1114_20531_17680 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1110_4801_12425 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2110_9717_11591 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1111_18978_23806 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2112_26347_9344 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1112_9306_24571 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2101_25412_7972 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1105_23025_12099 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2110_23067_17619 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1109_12799_6921 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1101_10382_22128 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2110_24314_8984 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2104_17794_25042 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_1109_16375_7276 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397 M00967_43_000000000-A3JHG_1_2110_24113_10963 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16075_4721 M00967_43_000000000-A3JHG_1_2103_15381_10377 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16075_4721 M00967_43_000000000-A3JHG_1_1113_13777_28036 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1106_16075_4721 M00967_43_000000000-A3JHG_1_1110_17713_26784 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1112_2989_20122 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_1106_22705_6123 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522 M00967_43_000000000-A3JHG_1_2104_11063_11086 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1101_24218_7727 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_7659_11830 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2114_4298_19710 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2111_16376_7553 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2105_8155_16465 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2104_13228_22585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_11514_14845 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1107_6417_21653 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2104_25226_10311 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2101_3680_12611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_19506_16402 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2111_25169_20207 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_20218_10272 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1102_20961_15950 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2104_7131_19760 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_25362_24755 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2110_22037_7673 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2103_5527_9613 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1107_19181_20279 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2101_25211_5709 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_20220_13470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1108_26047_23516 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2112_22323_22944 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2111_21529_21161 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2106_12132_25703 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2103_3688_9745 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2102_10617_12843 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_23518_14384 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1110_27186_10400 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1105_8707_15634 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2113_10370_2988 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_18367_8585 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_14338_24655 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1112_27973_10692 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1107_20108_5368 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1103_11367_13197 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2114_17435_3730 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2112_16562_5570 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2108_4691_12067 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2108_12490_16505 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2106_7250_19443 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2104_26311_14104 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2104_15640_8806 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2103_9278_23216 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_9308_15644 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1112_14947_19152 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1111_15247_9017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1107_22825_25509 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1105_16333_26309 0.01984" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2111_12821_2190 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2110_5497_7982 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2107_21893_17093 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2103_11591_8763 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2101_23950_13306 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_4659_12375 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_11281_2740 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1112_23531_7580 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1104_17143_19056 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1101_14159_27572 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2114_24035_9289 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2109_10681_7135 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2108_21817_19761 0.032" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2103_11687_23017 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1114_18510_8776 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_16063_11536 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1112_7981_6310 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1112_13563_20676 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1105_8310_14174 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1103_16201_28240 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1103_13035_5452 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1101_20589_8650 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2114_18278_17470 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2107_16534_14769 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2105_6882_12684 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1110_17014_10530 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1109_2986_15638 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1106_8066_16367 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2110_17635_9975 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2108_7255_14963 0.007937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1111_15637_23152 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1109_18196_9880 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1104_10756_24942 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1107_22580_21773 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1104_17151_24507 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1102_19172_4834 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2109_10631_18447 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1112_13632_26764 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1103_7350_12102 0.01587" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2107_28574_13792 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_20185_10282 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1110_22113_16232 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2110_18060_5720 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2105_7116_11773 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2104_24218_17682 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_2102_5906_19319 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1106_7651_17094 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532 M00967_43_000000000-A3JHG_1_1113_13987_16790 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1102_3707_18726 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1106_12231_13452 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_2108_12567_15271 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1104_19278_14733 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_2108_13445_6827 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1110_18361_6083 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_2107_13016_9270 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1114_19529_17837 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1102_24841_18959 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1106_24555_20879 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_2101_10104_19430 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712 M00967_43_000000000-A3JHG_1_1105_5882_22900 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17706_11167 M00967_43_000000000-A3JHG_1_2103_14393_8923 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17706_11167 M00967_43_000000000-A3JHG_1_1111_4607_20709 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17706_11167 M00967_43_000000000-A3JHG_1_1103_7671_14378 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17706_11167 M00967_43_000000000-A3JHG_1_1102_5575_6810 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1101_25599_11475 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1102_22051_11604 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2111_22278_7396 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2110_20054_3531 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2107_8240_20277 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2106_16743_9576 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2114_28476_17852 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1114_9368_6991 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1114_23881_16039 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2107_15433_11976 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2103_29235_16481 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1114_12644_14109 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2108_22756_12283 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1108_11391_5528 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2104_17870_18881 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_1106_6775_23397 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335 M00967_43_000000000-A3JHG_1_2105_17259_2092 0.04314" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1102_7311_6821 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1102_20031_8045 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2108_7674_17990 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1114_8603_11968 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1113_21002_11725 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1103_15776_7109 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2112_21004_8857 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2109_16433_12706 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1102_19764_24086 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1112_21753_6188 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2112_11313_13873 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2111_16323_3522 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2106_15051_17101 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2103_28700_17194 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_1110_2564_19006 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2103_7853_22683 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326 M00967_43_000000000-A3JHG_1_2101_13911_2481 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1101_16910_7379 0.0119" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2113_12406_27672 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1113_23502_11596 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1106_22881_7086 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1112_26682_16357 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1106_17692_2367 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2106_21633_11578 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2103_10530_5261 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2113_7430_24285 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1114_13400_5574 0.01992" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1113_20274_18664 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1106_23059_17918 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2105_10730_3162 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1113_10619_10110 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1111_24333_20558 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1105_24828_8853 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2109_8100_6611 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1113_9062_6419 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2106_9918_20510 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1114_20892_15372 0.03968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1114_10011_16636 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1112_20297_3328 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1112_13976_5900 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1108_19698_21996 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2113_7343_26333 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2108_5881_15408 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2106_21459_16802 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2105_6171_20541 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1111_6317_24602 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1110_25186_13251 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2112_23722_9668 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2106_5691_11023 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2106_19274_24278 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2103_16606_4023 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2102_25497_17323 0.03175" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1109_8621_15527 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2111_19706_18823 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1113_27297_13091 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1103_11491_13931 0.02381" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_2109_8160_14127 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1108_18644_12052 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739 M00967_43_000000000-A3JHG_1_1110_20712_23782 0.02778" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1101_8868_4602 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1106_20287_11044 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2108_10122_16816 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1111_21774_11978 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1110_11611_23800 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1105_16624_8157 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1102_27180_15478 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2114_19661_25608 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2112_12287_24914 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1109_24850_14336 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1104_18175_14208 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2114_8962_26729 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2112_24855_7054 0.01569" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2107_12427_15408 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2102_5173_21862 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2114_15082_26647 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2111_20930_15299 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2109_27908_9851 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1110_7982_9474 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2103_14978_21851 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2101_24821_18536 0.03137" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1114_9365_4621 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1112_22172_7939 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2104_26923_12364 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2113_23168_9750 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2111_26393_21929 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2108_26742_16490 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2106_17332_22169 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2105_23960_21462 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2103_12055_12246 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2103_11783_23160 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2101_16354_15467 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1111_11535_25208 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1111_11403_5737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1110_14278_11965 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1104_5923_11043 0.03113" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2112_4873_16721 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2112_10008_8529 0.007874" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2109_18300_12673 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2106_22746_5360 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2106_20577_15921 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2105_22749_25576 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2103_21922_24386 0.01176" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1113_26510_16371 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1106_25979_21288 0.03529" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1101_16143_23069 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2113_8012_12345 0.007843" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2111_9619_23737 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1114_8492_23708 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1114_18739_8131 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1112_10073_3247 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1104_7613_8427 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1102_26098_19093 0.02734" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2114_18555_17761 0.03125" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2111_28641_13458 0.01181" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2107_4030_17110 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1110_14609_6975 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1107_6365_24079 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2112_20105_17530 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2109_19519_2320 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2109_16258_1585 0.01961" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2105_18070_5688 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2103_27740_10743 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1114_8059_18290 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2112_17191_16655 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2110_12889_27699 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2108_27865_15182 0.02724" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2108_15244_28739 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2105_24305_5134 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2102_26372_21137 0.02335" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1114_25025_23939 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1113_23986_14034 0.02344" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2109_25240_9047 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1111_20322_27077 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1114_25131_8732 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_1112_13372_27170 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2110_25915_23958 0.02745" << endl; 
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278 M00967_43_000000000-A3JHG_1_2108_15232_28922 0.02353" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1107_19059_3463 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1101_16780_2259 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1103_15994_22947 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1112_9943_15231 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1107_13967_11089 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1108_15027_8794 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_2113_24913_16194 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1106_22610_14458 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_2112_7837_7502 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1104_11494_18542 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_2104_9988_18586 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_2102_12732_5062 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1113_6843_21123 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1109_22795_14196 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145 M00967_43_000000000-A3JHG_1_1101_24745_24299 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1101_10133_8460 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1101_14364_8401 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2110_11932_12993 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2109_20252_6302 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2107_8153_5482 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2107_27195_14074 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1111_10387_25149 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1109_13648_13972 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1107_17032_8072 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1113_12711_3318 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1101_15463_5782 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2109_11412_26175 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1110_28129_18678 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1109_13208_3584 0.01969" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1103_26302_22051 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1106_21550_17093 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_1114_6229_5326 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2114_5448_14619 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2105_13267_15722 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072 M00967_43_000000000-A3JHG_1_2101_17219_12550 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1101_9376_4054 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1113_19266_19695 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1110_14717_26485 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1108_4844_10914 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1102_4146_15034 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2102_26498_14617 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1113_7278_14313 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2105_12113_5426 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2108_14798_3606 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1107_7966_6427 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2112_14835_3016 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2112_11545_12010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1110_22492_13115 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2114_22532_10009 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2113_5333_12504 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2112_26207_7695 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2114_8388_7715 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1110_15637_12992 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1108_27208_20174 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1102_12417_5953 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2110_26207_14420 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2103_15244_25817 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2113_9492_17806 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2105_19416_13506 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2114_20979_14273 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2106_6366_20022 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1113_18521_9535 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_2110_11592_18531 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1113_11975_6040 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1111_12881_9910 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692 M00967_43_000000000-A3JHG_1_1113_26517_18281 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_1104_26066_8122 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_1110_11254_14780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2108_18841_25194 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_1101_16390_9529 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2112_9093_4236 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2112_16774_9147 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_1106_15955_6621 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2111_17860_28348 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2108_12975_25037 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_1113_13971_21290 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_2107_14468_14971 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388 M00967_43_000000000-A3JHG_1_1114_21630_21181 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1101_20183_7496 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2113_14457_3580 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1102_3890_17221 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1110_18818_19657 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1103_18198_2609 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1112_9301_13162 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1112_17691_12056 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2105_16439_13484 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1104_23422_20961 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2109_13351_28335 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2114_26633_11062 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1109_17670_1881 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2113_4494_9088 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2105_7943_8043 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2101_24924_5766 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1101_11823_22466 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_2102_15674_19713 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840 M00967_43_000000000-A3JHG_1_1113_19158_8422 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825 M00967_43_000000000-A3JHG_1_1111_20569_3100 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825 M00967_43_000000000-A3JHG_1_2109_14117_4744 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825 M00967_43_000000000-A3JHG_1_2113_9766_20824 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825 M00967_43_000000000-A3JHG_1_1108_18422_23262 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825 M00967_43_000000000-A3JHG_1_1106_23262_25144 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825 M00967_43_000000000-A3JHG_1_2103_27141_19101 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1101_6836_23417 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1113_8082_6010 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2113_18397_7516 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2106_22891_19654 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2108_18358_2270 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1102_11324_6735 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2102_21816_16599 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1108_23963_16017 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1104_11751_17981 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1106_11208_20468 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2114_22279_21530 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1107_7832_8780 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2112_8454_7758 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_2106_19379_4847 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937 M00967_43_000000000-A3JHG_1_1114_15409_10721 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_2110_16815_14755 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1101_15533_5293 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_2107_23675_17555 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1113_22462_8142 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1105_6877_11063 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1112_2989_20122 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1106_22705_6123 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_2104_11063_11086 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100 M00967_43_000000000-A3JHG_1_1105_7593_10522 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.01186" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.007905" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.01575" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_2102_19632_15186 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.02372" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1104_14019_23841 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1114_11189_15616 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1112_11092_24430 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1107_23336_26415 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1113_18370_24961 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1107_21734_25420 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076 M00967_43_000000000-A3JHG_1_1106_24084_10563 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1104_24493_5303 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1104_25962_6708 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1102_16785_23687 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_2112_23259_13531 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1110_17627_6261 0.03571" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1108_20841_7623 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_2103_16331_20380 0.04348" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1105_22998_9973 0.01976" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_2111_12498_13580 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1114_18296_7709 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1113_24017_25976 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1102_7684_6622 0.03162" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1110_22680_22915 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_2102_18381_13991 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1112_2989_20122 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1106_22705_6123 0.02767" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_2104_11063_11086 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1106_25330_8604 0.04365" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1105_7593_10522 0.03953" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284 M00967_43_000000000-A3JHG_1_1103_19321_18100 0.03557" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1101_14341_8428 0.02756" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1102_15350_5623 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1102_7484_9022 0.02362" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1107_14437_24723 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1107_18906_15466 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_2108_23615_21129 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_2106_22580_3507 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1113_18114_26077 0.03922" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1102_21808_10504 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_2102_13864_6831 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1112_10200_9979 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1105_11090_20196 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_2102_14650_8724 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_2105_21156_7257 0.03543" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1112_11092_24430 0.04331" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1107_23336_26415 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1113_18370_24961 0.03937" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1107_21734_25420 0.01581" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1106_24084_10563 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229 M00967_43_000000000-A3JHG_1_1103_12856_12076 0.0315" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20702_8361 M00967_43_000000000-A3JHG_1_1110_12295_19699 0.003968" << endl; 
out << "M00967_43_000000000-A3JHG_1_1101_20702_8361 M00967_43_000000000-A3JHG_1_1114_5207_13479 0.02372" << endl; 


    out.close();
    
    return "column.temp";
}

/***********************************************************************/
string DistCDataSet::writeCountFile() {
    
    ofstream out;
    m->openOutputFile("count.temp", out);
    

out << "Representative_Sequence	total	F3D0	F3D1	F3D141	F3D142	F3D143	F3D144	F3D145	F3D146	F3D147	F3D148	F3D149	F3D150	F3D2	F3D3	F3D5	F3D6	F3D7	F3D8	F3D9" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14069_1827	7305	628	0	422	251	306	465	661	394	1602	1216	870	279	146	23	20	11	9	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18044_1900	33	1	0	1	0	0	1	0	0	12	10	3	0	0	0	2	0	3	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13234_1983	12196	495	340	386	242	188	340	562	269	1307	744	769	263	3056	830	272	895	575	230	433" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16780_2259	198	16	17	1	7	3	5	3	20	22	6	14	6	37	7	5	7	1	7	14" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_17674_2779	8829	306	302	333	256	152	242	443	204	1101	662	699	206	1428	541	234	605	447	302	366" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18089_2781	592	98	245	0	0	0	6	0	36	22	0	0	0	0	16	0	72	0	0	97" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14203_2907	7698	390	185	295	142	172	256	453	218	811	518	644	341	1050	389	234	500	382	294	424" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18278_3345	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16061_3516	520	71	61	21	6	10	15	12	18	8	8	37	25	70	6	35	43	8	28	38" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_9376_4054	5207	162	174	327	78	91	36	124	63	86	522	527	103	1193	386	190	244	219	272	410" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11296_4480	35	19	0	1	0	0	0	0	0	0	8	4	2	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_8868_4602	2854	25	53	111	50	66	92	127	77	511	273	282	155	299	214	111	126	114	72	96" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_15591_4696	93	9	0	0	0	0	0	0	9	0	8	8	4	16	0	1	17	0	0	21" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18922_4934	425	26	28	22	8	14	4	5	27	36	45	75	26	37	5	9	29	11	5	13" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_22481_5236	930	276	58	46	8	8	13	9	20	52	24	40	12	147	30	38	50	13	49	37" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_21702_5444	1835	52	113	8	96	37	12	18	4	131	17	74	54	306	83	45	388	104	129	164" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_6977_5486	60	14	5	0	0	1	0	0	8	0	0	0	4	9	1	7	0	1	3	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_20384_5679	3565	135	83	117	72	60	134	213	78	235	184	253	130	569	268	189	217	248	180	200" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_15463_5782	5	0	0	0	0	0	1	0	2	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_9331_5806	6	3	0	1	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_19852_6086	78	0	73	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_17656_7294	586	26	44	6	4	11	14	11	33	23	12	38	25	133	16	37	43	11	40	59" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16910_7379	7436	401	63	481	151	206	316	525	365	1044	826	860	433	445	186	147	378	293	128	188" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_20183_7496	707	67	93	0	0	0	3	5	35	32	8	6	5	182	10	41	100	39	35	46" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_6929_7655	75	60	2	0	0	0	0	0	0	0	0	0	0	12	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_24218_7727	6484	348	114	276	189	181	279	463	249	1026	704	639	266	403	209	133	350	231	171	253" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11666_7824	77	8	0	3	5	4	0	6	15	11	10	8	4	3	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_19252_7867	243	46	38	0	0	0	0	0	0	0	0	0	0	63	8	27	47	14	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14364_8401	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14341_8428	757	58	26	11	16	0	16	0	0	41	86	32	29	82	17	115	66	30	55	77" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_10133_8460	618	32	127	0	0	0	0	0	0	0	0	1	0	222	5	13	20	17	95	86" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13595_8524	819	32	14	76	28	54	11	10	66	69	77	235	117	15	1	0	5	1	5	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_21616_8560	227	9	38	0	0	2	0	0	2	0	0	4	1	90	0	17	28	0	15	21" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_26181_9437	961	37	52	45	4	9	37	32	17	137	102	131	19	29	113	7	26	34	108	22" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_7265_9662	315	47	24	4	7	8	14	8	14	27	20	32	13	35	2	15	20	4	5	16" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11856_9674	282	74	40	0	1	0	0	3	0	20	0	1	0	75	12	1	6	13	9	27" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_28553_10567	559	23	26	54	10	20	8	17	20	43	41	131	75	39	9	0	22	0	7	14" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_21195_10623	246	76	10	21	0	5	8	0	3	0	10	10	2	35	5	10	14	7	15	15" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14168_10823	169	45	33	0	0	0	0	0	0	7	0	0	0	58	4	15	5	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_25599_11475	1297	84	278	28	8	7	10	14	22	67	42	35	18	336	47	77	54	35	42	93" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12395_11489	696	60	56	6	4	3	3	5	18	5	9	8	6	299	31	44	51	10	48	30" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_8084_12645	1958	100	34	98	81	41	80	176	71	435	230	150	69	135	61	60	57	41	17	22" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_15031_12738	89	31	9	0	0	0	0	0	0	0	3	2	3	34	1	2	2	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_21761_13031	655	24	6	47	11	0	9	14	13	87	143	88	42	27	18	20	23	37	19	27" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18143_13375	116	5	4	3	0	0	5	3	1	9	8	14	9	8	1	8	12	5	6	15" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11381_14019	633	36	135	30	0	4	9	10	28	11	20	34	15	97	7	43	20	6	47	81" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_9553_14094	5	2	0	2	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_24025_14108	1271	78	0	95	45	35	98	104	34	273	246	162	29	15	22	29	0	0	6	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11035_15765	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_28516_16324	1064	61	27	37	21	15	39	80	29	116	106	99	39	170	89	40	34	15	14	33" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_7408_16842	12	3	0	0	1	0	3	0	0	0	5	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13333_16849	379	16	20	10	4	10	0	7	19	32	22	35	37	72	6	17	19	7	19	27" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_19534_17052	19	0	0	0	1	0	3	0	0	5	5	1	0	4	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_5883_17133	368	60	48	0	0	0	0	0	0	0	0	0	0	149	9	8	26	8	37	23" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_25286_19670	98	6	6	3	2	0	1	0	0	6	6	7	1	33	8	1	1	0	7	10" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_9620_19745	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_9813_20001	185	24	10	2	1	4	2	0	11	8	0	0	0	38	1	10	31	4	17	22" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_5370_21051	334	42	58	15	2	3	2	0	15	8	11	35	22	77	24	6	4	1	8	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_20653_21248	331	31	41	2	2	0	0	1	4	9	5	6	2	134	3	9	40	11	10	21" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12259_21251	335	24	59	0	1	0	0	0	0	0	1	0	0	74	12	31	42	18	28	45" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12695_21368	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18415_21676	199	114	0	4	3	2	5	10	0	17	26	11	6	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_19009_23185	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_10331_23332	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_6836_23417	46	6	3	0	0	0	6	4	5	1	4	9	7	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_23238_24359	210	5	12	1	1	1	1	3	7	6	2	8	4	19	19	11	24	11	31	44" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_17289_26831	364	10	21	7	1	0	4	5	6	8	9	19	6	71	15	31	36	28	35	52" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_17802_26911	350	25	26	36	2	25	0	10	23	13	42	29	21	18	3	14	16	5	19	23" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_17656_27121	6285	168	127	205	201	114	130	361	192	544	496	453	179	354	448	142	501	516	547	607" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_21369_2858	24	5	0	0	1	0	1	6	3	2	3	0	1	0	0	0	0	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14793_3762	17	10	0	0	0	0	0	0	0	0	0	0	0	7	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_20832_5250	3050	19	111	175	43	76	255	318	182	496	419	496	56	59	175	67	48	12	15	28" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_8473_5446	251	20	17	14	2	4	2	4	14	9	17	45	23	44	5	6	8	2	10	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22375_5719	71	3	5	0	1	0	0	0	6	8	2	3	6	9	0	6	9	0	8	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_12767_6517	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_13690_6672	130	4	3	11	3	7	0	6	8	0	14	26	24	6	0	7	3	1	2	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_8823_6749	90	12	9	3	0	2	0	6	4	7	4	13	6	3	0	0	0	2	12	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_7311_6821	369	49	35	1	2	3	0	1	3	0	5	6	3	147	3	44	9	4	27	27" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_13449_7439	97	8	1	8	5	4	4	6	9	4	12	18	6	3	0	4	0	0	0	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_12322_7970	131	0	0	9	5	3	2	5	12	13	14	52	16	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_20031_8045	79	7	5	5	1	0	0	0	0	0	7	1	0	34	0	7	1	1	4	6" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_7484_9022	55	0	0	0	0	16	0	17	22	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_28036_9037	336	42	34	0	4	0	0	0	0	21	10	0	11	65	7	0	50	5	39	48" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22051_11604	445	0	10	21	1	8	10	9	10	41	17	55	39	86	13	33	30	9	24	29" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_15058_11924	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22085_12796	31	5	5	1	1	0	1	1	1	3	1	1	3	4	0	0	0	0	0	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_17816_13048	3	1	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_17763_13764	14	0	3	1	0	0	1	0	2	0	3	1	0	0	0	1	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_5880_14023	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_24688_14454	202	12	0	3	0	1	4	6	7	8	18	16	4	26	4	20	17	9	14	33" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_29181_14456	6	2	0	0	1	0	0	0	1	0	0	0	0	0	0	1	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_27488_14514	60	15	9	0	0	0	0	0	1	0	0	0	0	16	5	1	6	1	2	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_23295_14739	222	16	3	5	6	3	2	12	1	24	21	9	5	54	16	10	4	10	8	13" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_2114_15227	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_17084_15866	493	68	45	19	5	6	10	8	17	21	14	49	19	70	14	21	37	4	36	30" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_6552_16619	221	5	29	3	0	3	1	7	9	0	6	11	7	68	5	8	12	2	25	20" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_10827_17300	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_27189_17647	8	0	4	0	0	0	0	0	3	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_6990_17751	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_3707_18726	37	15	0	0	0	0	0	0	0	0	0	0	0	13	0	0	0	1	8	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_11115_19075	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14193_19079	20	2	0	0	0	0	1	3	0	4	1	5	2	1	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_10011_20138	27	5	0	3	0	2	0	0	2	2	3	5	1	1	0	0	1	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14198_21368	212	36	33	5	0	0	0	1	4	7	5	17	3	86	10	0	0	1	2	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_16041_22886	58	11	7	0	0	2	0	0	0	6	9	0	0	11	2	3	0	1	6	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_16785_23687	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14077_24664	1129	57	90	4	5	0	5	5	3	31	44	3	4	378	328	13	65	37	18	39" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_16001_2026	4	0	0	0	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_18198_2609	3	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_8515_3790	16	2	2	1	0	0	0	0	1	0	1	2	0	2	1	1	0	0	1	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_22674_6370	814	0	0	13	6	2	0	7	0	0	46	38	19	529	0	80	0	24	50	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_15776_7109	16	0	0	0	0	0	2	0	0	14	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_9123_12021	222	4	9	15	0	12	4	0	12	0	14	60	16	28	6	7	12	6	6	11" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_5171_14027	6	5	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_7671_14378	46	0	0	3	0	0	1	4	0	5	6	4	0	5	0	0	0	1	10	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_5754_14689	10	1	0	1	2	0	0	0	1	1	0	0	1	1	0	0	1	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_5231_15971	17	4	6	0	0	0	0	0	0	0	0	0	0	4	0	0	0	3	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_14518_16099	2	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_26542_17589	8	0	0	0	0	0	0	1	0	0	4	2	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_3864_17599	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_7329_17775	40	10	1	3	0	1	0	3	0	0	0	3	0	2	2	0	2	3	3	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_11725_18144	46	8	5	0	0	0	0	0	0	0	0	0	0	9	4	0	6	4	5	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_15994_22947	137	17	18	1	3	0	3	1	2	8	5	0	0	29	5	8	13	6	9	9" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_25888_23585	107	2	1	4	2	6	0	2	4	6	6	25	11	17	1	4	5	1	3	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_25466_23747	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12991_25169	73	3	6	3	1	0	1	0	3	1	3	8	0	7	4	8	2	2	12	9" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_8579_3599	61	0	0	10	0	1	0	2	16	0	0	17	0	0	0	15	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_9801_4846	17	10	1	0	0	0	1	0	0	1	1	0	1	1	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24493_5303	503	35	44	3	4	1	1	0	5	20	0	11	5	135	11	41	46	0	48	93" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_7522_5556	90	8	5	3	0	1	2	2	2	11	0	7	8	12	2	5	8	0	7	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_19441_6353	156	5	9	8	6	7	4	9	1	29	16	13	0	22	4	3	8	1	5	6" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_25962_6708	2	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_10834_8009	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_26066_8122	284	8	18	2	2	0	1	1	3	16	1	0	2	36	2	17	22	15	47	91" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_14218_8400	1089	33	5	57	23	25	34	77	44	338	171	147	54	31	13	12	6	7	4	8" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_13822_10866	88	3	4	0	0	1	0	0	1	0	0	6	2	33	15	13	8	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17583_11048	5	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	3	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17110_15247	18	0	0	0	0	0	0	0	9	0	0	0	9	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_11668_16421	15	7	0	0	0	0	0	0	0	0	0	2	1	0	0	0	3	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24665_17168	235	21	0	15	22	8	20	5	3	23	60	23	16	4	15	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17560_17233	25	2	1	2	0	1	0	1	2	0	4	3	1	1	0	1	4	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_13731_17325	37	0	18	0	0	0	0	2	0	0	0	5	0	0	0	4	6	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_21303_18168	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_18371_18182	303	16	31	27	2	4	3	2	13	20	19	22	11	42	9	15	20	6	20	21" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_10943_19850	17	2	0	0	0	1	1	0	0	5	8	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17315_20406	32	7	0	0	0	0	0	0	1	0	1	1	0	0	0	0	6	3	3	10" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_26838_20823	18	2	0	1	0	0	0	1	0	6	0	1	0	4	3	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_14252_27883	6	1	0	0	0	0	0	0	1	2	0	0	1	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_10606_2663	4	2	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_11964_4686	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_22998_9973	18	0	0	0	0	0	0	0	0	0	5	0	0	0	0	0	0	13	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23025_12058	2	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_24533_12281	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_14547_12843	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_24616_16104	5	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_22508_16792	326	5	65	7	0	1	1	0	9	9	13	30	14	52	9	42	23	5	16	25" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15701_17582	195	11	9	3	1	1	2	4	3	52	13	20	3	13	1	7	35	0	6	11" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23612_18072	19	0	8	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	6	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_9179_20196	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_13657_20414	102	16	0	1	0	0	0	2	8	2	2	5	4	14	0	2	17	3	9	17" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_7380_21363	46	13	7	0	0	0	4	1	4	0	0	1	1	3	0	0	5	0	5	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_14835_21938	297	3	9	5	7	7	9	17	4	27	12	8	7	51	36	6	21	25	21	22" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_26683_22024	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_18475_22342	14	1	3	0	0	0	0	2	0	0	2	0	1	2	0	1	1	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_12761_22353	16	2	1	1	0	0	0	0	0	0	0	0	0	9	3	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_10652_23248	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22881_7086	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_26888_10548	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_20287_11044	2	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_26087_11342	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_24475_12056	209	23	3	15	0	4	1	6	4	12	16	13	3	33	17	11	7	16	8	17" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_12231_13452	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_4308_15219	3	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_7651_17094	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22417_19200	43	5	0	1	3	0	3	4	0	2	3	6	5	5	0	0	2	0	3	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_25798_21572	31	3	2	0	0	0	0	0	1	4	0	1	1	6	1	4	3	2	3	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_11240_22282	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_12555_24141	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_21375_27109	5	2	0	0	0	0	1	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_19059_3463	205	16	12	1	4	9	5	5	16	19	8	7	5	37	0	11	8	1	15	26" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_21038_8569	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_13053_11087	11	2	0	1	0	0	0	0	0	2	1	0	2	1	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_27593_11400	56	9	6	0	0	0	1	1	8	3	0	3	1	7	3	0	6	0	4	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_3735_12375	48	13	5	1	0	0	2	0	0	3	2	3	2	6	0	1	1	2	2	5" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_21793_13370	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_6465_14196	4	0	1	0	0	0	2	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_20139_17113	3	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_7195_18281	5	0	0	0	0	0	0	1	0	0	0	3	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_13192_19802	9	4	1	0	0	0	0	0	0	1	1	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_6417_21653	9	0	0	0	0	0	0	0	0	0	4	0	5	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_11048_4561	99	2	8	1	0	0	0	0	0	0	0	0	0	24	5	7	29	7	9	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18821_5778	3	0	2	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_19065_7362	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18996_9436	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_4844_10914	68	1	0	3	0	0	0	2	0	0	11	12	3	12	3	2	2	8	3	6" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_20070_20418	19	2	0	0	0	0	0	0	0	3	0	0	0	7	2	0	0	3	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18770_23200	16	2	3	0	0	0	0	0	1	4	0	1	0	0	0	0	0	0	2	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18362_23908	4	0	0	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_24697_25438	12	0	0	2	0	0	0	0	0	5	5	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_13421_28822	55	5	0	4	1	2	2	3	4	2	7	15	4	3	0	0	1	1	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8014_5010	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_15887_5905	324	8	20	18	2	12	3	5	12	35	28	56	18	62	1	6	7	7	10	14" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_9629_6769	7	4	0	0	1	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_21243_9500	6	3	0	2	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_14359_12484	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_15029_15883	21	1	3	2	0	1	2	1	0	2	0	1	1	3	0	2	0	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_7996_16802	38	0	7	1	0	0	1	0	0	0	0	0	1	12	0	5	5	0	6	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8088_17239	219	3	2	41	4	21	2	0	10	11	16	70	33	3	0	1	1	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_22380_20532	25	5	6	0	0	0	0	1	0	0	0	0	0	3	4	0	2	1	2	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_20401_24219	9	0	0	0	0	0	0	4	0	0	0	0	0	3	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_7971_24259	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18313_3175	46	4	0	1	1	0	1	1	1	6	5	3	2	7	0	1	2	1	3	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18361_6083	77	0	0	0	1	0	2	0	11	9	9	11	7	0	0	4	14	0	0	9" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_6952_6984	22	3	0	0	0	0	0	0	1	0	0	5	1	3	0	1	7	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_9902_7407	2	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_9936_12702	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_5315_13833	156	5	14	8	0	7	0	2	6	30	4	15	12	10	3	7	8	4	11	10" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10609_18161	3	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12295_19699	14	1	0	1	2	0	0	0	0	4	2	0	0	0	1	0	0	1	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13033_21990	11	1	0	1	3	0	0	0	0	3	0	0	0	2	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_5826_22164	10	0	0	0	0	0	0	0	0	0	0	0	0	4	3	0	0	0	0	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17220_22884	26	1	1	0	1	0	0	1	3	0	3	1	1	4	1	2	0	0	4	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14717_26485	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14443_28339	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_20569_3100	184	0	15	5	4	2	0	8	16	17	4	10	9	33	0	9	17	5	12	18" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_24599_6936	33	4	0	0	0	0	0	0	0	24	0	0	0	0	0	4	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_11386_7041	8	0	1	1	1	0	0	1	1	1	1	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_17002_7999	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_22890_9747	53	7	6	0	0	1	1	2	0	0	4	0	1	11	1	2	5	2	4	6" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_20235_16372	43	3	3	0	0	0	1	0	2	6	0	7	6	5	1	2	2	1	0	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_19212_21380	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_11673_24359	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_13451_28512	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_12844_3769	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_7075_7100	125	4	38	1	1	0	1	0	0	6	3	2	3	45	7	2	0	1	4	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14780_8078	7	1	0	1	0	0	0	0	0	1	0	0	3	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11192_11762	7	0	0	0	1	0	0	1	0	0	0	0	0	2	0	2	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_20884_13058	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_8060_13320	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_27485_13416	7	0	0	2	0	0	0	0	1	2	1	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_8821_13863	387	1	3	54	9	32	32	38	87	36	9	3	0	2	3	2	13	2	24	37" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18757_14747	35	0	0	1	0	1	2	0	0	13	3	4	0	0	10	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_17696_15281	5	1	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_13682_16758	28	4	11	0	0	0	0	0	0	0	0	5	5	0	0	0	0	0	3	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10194_20788	3	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10024_21424	6	1	1	0	0	0	1	0	0	0	0	0	0	2	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11343_21537	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18548_4292	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19532_4899	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_8082_6010	87	5	19	6	0	1	0	0	3	0	1	16	5	16	2	1	3	2	6	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18958_10702	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_23502_11596	6	3	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_7659_11830	8	4	0	0	0	0	0	0	0	2	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20770_14973	8	1	0	0	0	0	0	0	0	2	0	3	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20207_16728	7	3	1	0	0	0	0	1	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_8139_17897	206	1	3	26	5	7	3	4	8	23	18	60	45	1	0	0	1	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_6044_19045	34	4	0	1	2	0	1	0	0	3	7	3	1	2	5	1	0	1	2	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19266_19695	23	0	4	0	0	0	0	0	0	0	4	0	0	7	0	0	0	1	3	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13777_28036	10	3	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	4	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13397_2211	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12127_4319	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_5902_6841	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19875_9916	11	4	3	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23055_10307	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_25962_10907	135	3	3	15	2	5	4	0	12	27	12	7	6	20	2	2	5	2	5	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8603_11968	5	1	0	0	0	0	0	0	0	0	0	0	0	0	0	4	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22804_12324	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_26968_12650	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21222_13060	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_11514_14845	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_1863_15569	3	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_3131_16050	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12471_18325	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22681_21130	6	1	0	0	0	1	0	0	0	0	0	2	0	1	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_25590_23883	3	1	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_25362_24755	3	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14280_26421	72	0	20	2	0	0	3	3	0	4	0	0	0	16	1	10	0	2	11	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_16955_14005	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13466_27462	6	1	1	0	0	0	0	0	0	1	0	0	0	1	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_17019_10818	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12930_14637	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_6705_15205	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_27520_15637	22	2	2	0	0	0	0	0	1	0	0	1	0	2	0	1	4	0	3	6" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12263_16584	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_15542_26928	3	1	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12696_27241	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_25452_6018	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_22514_6355	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11954_7925	9	0	0	2	0	0	0	0	0	4	0	0	0	0	0	0	0	3	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_14393_8923	11	5	0	0	0	0	0	0	0	0	0	0	2	0	0	4	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_15381_10377	3	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_7701_13077	10	1	1	0	0	0	0	1	0	0	1	1	0	1	0	1	0	0	2	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_22029_15973	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_13201_17677	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_12830_18492	55	2	9	0	1	0	0	2	0	4	3	6	2	9	1	3	3	1	4	5" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_14234_20506	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_18861_20938	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_21597_21834	67	1	0	5	1	0	0	2	2	0	9	14	2	2	0	0	3	4	8	14" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_17221_27256	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_12505_3424	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_20685_7431	33	1	0	0	1	0	1	1	3	4	3	0	3	4	1	1	7	2	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_25418_11257	11	4	1	0	0	0	0	0	0	0	0	0	0	2	1	0	1	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_15263_18242	9	1	0	0	1	1	0	1	0	2	1	0	0	0	0	1	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_9975_21883	7	1	0	0	0	0	0	0	0	5	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_13228_22585	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_17794_25042	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_9234_4159	20	3	0	0	0	0	0	4	5	0	0	1	0	5	1	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_20695_7710	44	1	0	3	4	0	0	3	0	6	13	9	0	0	0	0	4	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15157_10455	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_17320_11249	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_17957_13444	5	1	0	0	1	0	1	0	0	0	0	0	0	0	0	1	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_16439_13484	7	0	0	0	0	0	0	0	0	1	0	0	0	0	0	2	0	3	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_8155_16465	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_22135_19871	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_16910_23126	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_19750_8249	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_16743_9576	6	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_14305_11884	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12340_14438	45	4	3	0	0	0	0	0	2	0	2	0	0	20	0	1	3	2	1	7" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_22891_19654	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_10758_21336	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12968_23126	62	1	0	2	0	3	1	1	2	3	8	13	4	16	1	2	1	0	4	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_15211_24821	5	0	0	0	1	0	0	0	0	0	2	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_28574_13792	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_26817_14749	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_26555_14955	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_27757_15813	48	0	0	0	0	1	0	6	0	0	0	20	19	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_6956_16009	89	2	0	12	1	7	3	1	3	6	11	23	12	3	1	1	2	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_8240_20277	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_5077_12621	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_21459_14649	2	0	0	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_10435_17390	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_7674_17990	2	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_25695_18492	59	0	0	4	0	5	0	1	2	1	13	22	5	0	1	0	0	0	3	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_24812_22312	3	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_8220_23281	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_19309_25091	3	1	0	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_19840_26409	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_7804_9298	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_21549_9949	2	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_9943_10858	6	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_22245_24849	25	0	0	0	0	0	0	0	0	5	0	0	0	0	0	2	7	0	2	9" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_20054_3531	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_6116_13104	6	1	0	0	0	0	0	0	0	2	0	3	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_16815_14755	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_23584_15948	36	0	0	0	3	1	0	0	12	0	0	8	0	0	0	0	12	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_10107_20150	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_23478_21034	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_18978_25958	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_13819_4746	8	0	0	0	0	0	0	3	0	5	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_5664_6671	15	1	0	2	0	0	0	2	0	0	0	1	1	3	0	4	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_22278_7396	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_16376_7553	2	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_6782_8678	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_6887_10490	3	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_20158_16824	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_29227_17392	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_16058_17833	49	0	0	0	1	2	1	2	2	2	4	0	0	5	0	0	14	0	0	16" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_12338_23302	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_8444_24070	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11743_8674	162	9	15	0	4	0	2	0	16	9	14	0	0	45	0	9	12	3	10	14" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26773_14017	60	3	5	0	0	0	0	2	0	1	1	2	1	19	0	5	6	2	8	5" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_15899_17037	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10480_18043	5	0	0	0	0	0	0	0	0	0	5	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26066_19401	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10826_19542	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_14457_3580	2	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_12698_4375	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_18397_7516	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_8332_13243	2	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21352_15666	18	1	0	0	0	1	2	0	0	3	0	8	1	0	0	0	1	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_12406_27672	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_6259_8307	2	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_14336_9476	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_16038_11947	4	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_10492_14599	6	3	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_27537_17929	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_4298_19710	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_15533_5293	5	0	0	2	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13431_10294	277	0	0	29	21	21	14	14	5	48	80	32	13	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_23926_12310	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13158_12636	42	0	0	3	0	0	1	3	4	4	3	6	2	1	0	3	0	0	5	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_7042_19790	130	0	4	9	0	3	3	5	12	10	9	22	11	14	4	5	5	1	7	6" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12302_23776	6	0	0	0	0	0	0	0	0	5	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14677_6391	144	0	1	11	1	5	5	10	12	4	17	34	14	9	0	4	5	3	3	6" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_11324_6735	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_4962_7612	34	0	3	1	0	1	0	0	1	2	6	6	3	5	0	2	0	2	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14615_8833	2	0	0	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_11941_12226	222	0	0	7	9	8	6	11	12	38	10	28	22	0	0	0	0	0	24	47" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_4146_15034	3	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_27837_16697	2	0	0	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22713_17777	289	0	0	15	8	0	17	11	32	74	56	39	37	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_8887_21995	2	0	0	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_21121_24835	10	0	0	0	0	0	0	0	0	0	10	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_22959_4225	4	0	0	1	1	0	0	0	0	0	0	1	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_17414_11243	12	0	0	0	0	0	0	0	0	5	3	0	0	1	0	1	0	1	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_3710_21770	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_5274_12644	8	0	0	0	0	0	8	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_27578_17650	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_10875_20179	20	0	4	2	0	0	0	1	1	0	1	0	1	3	2	1	1	0	1	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_27583_14497	2	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_27806_20120	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_10076_22271	100	0	0	5	2	6	11	17	7	32	0	9	2	0	7	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_12242_23443	84	0	0	9	0	2	0	1	10	7	9	28	16	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_20176_26018	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15878_26239	18	0	0	4	1	0	1	2	1	2	1	4	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_17692_2367	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_14252_8640	9	0	0	1	0	2	1	5	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_7357_12797	2	0	0	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_15774_18346	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_9147_22015	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_15635_4811	11	0	0	2	1	1	0	0	1	0	0	1	0	0	1	0	1	0	0	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_14329_23174	33	0	0	2	0	1	0	1	2	6	0	3	2	6	0	1	0	2	3	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_8149_24158	7	0	0	1	0	0	1	0	0	4	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_15027_8794	8	0	0	1	0	0	0	0	0	3	0	0	1	0	0	0	2	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_16488_9936	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14021_11937	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_20614_22010	5	0	0	0	0	0	0	0	5	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14399_23420	2	0	0	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_21122_24729	107	0	0	9	0	7	0	3	6	6	5	24	11	11	10	0	10	5	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_11757_26753	22	0	0	0	0	0	0	2	3	0	0	9	3	0	0	0	5	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24712_5827	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_11299_13989	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_21060_16840	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_11007_22305	43	0	1	2	0	0	1	3	2	4	7	8	6	4	0	1	1	0	2	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18950_6153	5	0	0	1	0	0	0	1	0	0	0	0	0	1	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_27249_9297	3	0	0	1	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_7742_10390	29	0	3	1	1	0	0	0	1	1	1	0	0	5	0	0	0	2	0	14" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_26933_11956	46	0	0	0	2	0	5	4	0	13	14	5	0	1	1	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_6233_13348	7	0	0	2	0	0	1	1	1	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10588_13620	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10693_14739	2	0	0	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18818_19657	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_11611_23800	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10033_25797	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_9258_26812	27	0	0	6	0	1	1	3	0	2	2	8	0	1	0	0	1	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_16292_27414	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_21792_3253	51	0	3	1	8	0	1	6	0	10	0	0	2	8	3	1	4	0	4	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_21774_11978	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_24706_13976	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_12107_16431	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14201_3200	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_16536_4799	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9549_5158	6	0	0	1	0	0	1	2	1	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_27414_10900	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_26682_16357	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10447_18633	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_19666_21477	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_6577_24020	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_15323_28130	12	0	0	1	0	0	1	0	0	1	0	7	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_26886_12231	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_11232_14725	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20075_16901	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_4318_16941	7	0	0	0	0	0	0	0	0	0	0	5	0	0	1	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_22735_22577	22	0	0	0	3	3	1	2	0	5	7	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13883_23943	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_12104_26590	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_21762_27306	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18793_4661	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19154_8327	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22108_9150	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13389_13295	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19506_16402	3	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23713_17193	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18190_18406	9	0	0	1	0	0	0	0	1	2	0	2	0	1	0	0	0	1	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_15218_23608	3	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14152_27072	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_20023_3542	12	0	0	2	0	1	0	1	1	0	1	5	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17535_7182	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_7382_19629	130	0	0	7	0	6	11	8	9	5	19	37	6	8	0	3	7	0	0	4" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_27710_20218	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_14628_22758	87	0	0	4	0	0	0	4	0	0	0	0	12	2	0	39	26	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_25094_24210	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_6445_10536	39	0	0	1	0	0	0	6	6	11	7	4	0	0	4	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_7041_13746	21	0	1	1	0	1	0	0	2	3	1	2	1	2	1	0	2	1	0	3" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_26498_14617	2	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_27795_15713	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_23347_17242	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_17332_24887	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_22167_26446	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_19057_10430	2	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_18492_19103	24	0	5	1	0	1	0	0	5	2	1	1	1	3	1	2	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_14570_24653	27	0	0	0	0	3	2	0	0	0	9	6	7	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_18261_8446	11	0	0	1	0	2	0	0	0	3	0	3	1	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_6580_11453	82	0	0	3	6	0	8	14	10	7	8	4	6	0	0	0	0	2	10	4" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_20227_11495	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_6831_13690	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_7131_19760	8	0	0	0	0	8	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_17991_24804	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_11723_5649	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_29107_15046	10	0	1	2	0	0	1	0	0	0	1	2	3	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_12511_24138	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_19677_24897	3	0	0	1	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_7936_8291	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_6909_9337	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_25337_11987	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_16818_12005	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12583_13406	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_27160_14501	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_5509_18056	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12253_21736	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_17498_8085	12	0	0	2	0	0	0	3	1	3	1	0	1	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_13720_13200	3	0	0	1	0	1	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_6821_13209	6	0	0	1	0	0	0	2	0	2	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_12720_21761	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_13445_6827	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_27497_14930	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_12567_15271	9	0	7	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_17613_16563	147	0	26	5	1	3	0	3	3	14	11	6	0	2	11	15	14	14	11	8" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_19062_16604	3	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_18841_25194	14	0	0	1	0	0	0	0	0	2	3	1	0	0	0	1	0	0	2	4" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_9657_11455	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_10641_20956	2	0	0	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_15602_23047	2	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_13298_25616	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_17362_10105	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_14170_11158	38	0	0	2	0	3	0	2	4	10	4	5	2	1	0	0	0	0	3	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_8186_13301	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_6114_18144	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11545_12010	4	0	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_20678_18094	9	0	0	0	0	0	0	0	0	9	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22596_18948	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_17281_21576	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_13473_22310	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_13884_15208	21	0	0	1	1	3	2	4	0	3	3	3	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_12741_23025	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13714_14338	2	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_15350_5623	5	0	0	0	4	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_3871_8294	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_21808_10504	9	0	0	0	0	0	0	0	0	5	0	0	0	0	0	0	4	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14563_13874	16	0	0	0	4	1	1	0	0	3	5	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_27180_15478	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_20961_15950	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_18759_22050	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_21382_27245	48	0	0	0	3	8	1	14	1	9	9	3	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_25170_16709	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17535_11802	2	0	0	0	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24989_12614	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_16624_8157	4	0	0	0	1	0	0	0	0	1	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15318_9427	3	0	0	0	1	0	0	0	0	1	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_13597_20997	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15636_27192	2	0	0	0	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_8718_15420	15	0	3	0	0	0	0	0	0	0	0	0	0	0	0	5	0	0	0	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_20644_25868	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_2468_14102	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_15327_17780	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_15844_3660	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_2439_18628	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10911_24264	3	0	0	0	1	0	0	0	0	0	0	0	1	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_12129_4985	10	0	0	0	1	0	0	0	0	2	0	0	0	0	0	0	4	0	1	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_27552_22078	2	0	0	0	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17653_8119	23	0	3	0	1	0	0	0	3	1	0	0	0	3	1	1	3	0	3	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_5736_9217	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18344_10035	8	0	0	0	2	0	0	0	0	1	4	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_20816_14150	33	0	4	0	1	0	1	0	1	2	0	1	0	10	1	0	4	1	3	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_11254_14780	3	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13863_21427	8	0	0	0	1	0	0	1	0	1	2	0	0	0	2	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10006_21831	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_19584_26279	43	0	0	0	2	1	0	2	9	12	4	6	5	1	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_11925_5550	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_12899_5675	2	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_18224_14666	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_16173_17418	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10124_4953	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_17916_7193	6	0	0	0	1	0	0	1	1	2	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_5118_11366	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_20902_11558	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_26033_17779	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_16159_23387	35	0	0	0	1	0	1	1	0	8	2	5	2	0	9	3	2	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_12711_3318	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_23401_8348	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19958_8814	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_27215_10406	2	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_7278_14313	3	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_20218_10272	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14259_12215	8	0	0	0	1	0	1	0	0	0	1	0	0	4	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13584_18783	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_20989_19235	6	0	0	0	2	0	0	0	0	0	2	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13545_27181	4	0	0	0	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_9084_14502	19	0	0	0	2	0	0	0	0	2	4	0	2	1	3	1	1	3	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17405_14654	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_25710_23156	3	0	0	0	1	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_15778_12420	2	0	0	0	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_10530_5261	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_20113_16810	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_25610_6877	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_9776_14422	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_14359_14505	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_11759_16153	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_6844_12008	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_17326_26211	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_20168_3238	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_21633_11578	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_10304_14471	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_15433_11976	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_5816_12823	5	0	0	0	2	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_27604_15299	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_16914_2576	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_14290_10968	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_27613_12769	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_22020_21434	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_17316_4550	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_20085_20830	6	0	0	0	1	0	2	3	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_24829_22616	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_8189_7507	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_25169_20207	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_6592_20714	2	0	0	0	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_17219_14317	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10013_23090	4	0	0	0	1	0	0	0	1	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22673_23252	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_18311_25065	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_15802_4098	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_18844_7914	3	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_20973_10900	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_9583_13925	8	0	0	0	2	0	0	0	0	3	3	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_23687_14507	2	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_24913_16194	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_4725_7023	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_21674_11680	2	0	0	0	1	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_26020_8626	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16007_27282	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_5575_6810	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_26826_15050	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_6362_13340	4	0	0	0	0	1	0	1	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_12222_5327	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_20673_8839	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23257_9242	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_14213_12414	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_18175_14208	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_12675_27249	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15500_1801	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_14971_7732	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_27543_11258	8	0	0	0	0	1	0	1	0	2	2	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_25410_11631	2	0	0	0	0	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15675_15142	4	0	0	0	0	1	0	1	0	0	0	0	0	0	0	0	0	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_18988_15558	2	0	0	0	0	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_19259_21539	93	0	0	0	0	0	0	0	0	8	6	0	0	26	50	0	3	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_9232_9214	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22610_14458	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_10456_18286	2	0	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_17173_6497	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_20748_16898	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_19181_20279	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22525_22794	2	0	0	0	0	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_14437_24723	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14909_4503	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_12975_13764	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10730_20249	2	0	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_7641_20331	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_5894_23389	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24850_14336	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_12715_19004	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12738_11413	9	0	0	0	0	1	1	1	0	2	1	0	0	1	0	0	1	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14183_13016	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15335_17802	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15855_19234	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10946_20858	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15463_23056	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_7592_13444	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_23754_16069	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_15312_18978	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_4607_20709	7	0	1	0	0	2	0	0	2	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_9466_21791	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14868_10194	10	0	0	0	0	2	1	3	0	0	3	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_21256_12063	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_3066_12463	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9301_13162	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_24425_19170	3	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_24501_12692	10	0	0	0	0	0	1	0	0	7	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_29188_13629	2	0	0	0	0	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_26736_16107	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9494_6921	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_22171_7080	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_4572_15105	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12937_4366	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_26655_15452	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_7887_6353	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_19578_6645	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_16319_7131	2	0	0	0	0	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_5527_9613	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_10169_13566	5	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	4" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_16331_20380	75	0	0	0	0	1	2	0	4	7	20	33	7	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_13757_8649	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_12113_5426	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_11308_6301	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15973_8357	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_23104_8731	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_24712_15313	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_21230_21464	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_19460_3735	9	0	0	0	0	1	3	1	0	0	0	4	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12578_10682	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_15738_17268	73	0	0	0	0	1	0	3	0	2	1	2	2	5	4	2	14	12	13	12" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_14094_18415	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_23831_12272	3	0	0	0	0	1	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_12316_24495	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_16551_3455	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_29278_13914	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_25546_10365	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_21713_13835	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_22495_21331	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_8933_4869	4	0	0	0	0	1	0	0	0	0	0	0	0	3	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_22037_7673	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_10506_11014	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_18983_13738	4	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_16087_14724	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_26547_22871	6	0	0	0	0	1	2	0	0	1	0	1	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_8686_5552	9	0	0	0	0	1	1	0	0	2	0	0	0	4	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_12287_24914	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_16554_11081	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_28447_11534	4	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_15255_5535	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_20314_10028	1	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_11245_12909	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_20575_21363	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_19661_25608	2	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11480_8830	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_26372_17971	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12732_20169	3	0	0	0	0	0	1	0	0	0	0	1	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12446_20708	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_20815_21052	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11348_22601	2	0	0	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_17961_23432	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_23281_23823	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22477_7326	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_19038_11166	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22955_21067	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_8311_8476	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_27444_10708	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_2359_11998	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_10837_19652	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12955_24887	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_19800_26276	4	0	0	0	0	0	1	1	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_20104_23610	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_25210_11850	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_11157_16891	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15005_21488	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_21254_7027	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_17565_8490	8	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	2	0	3	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_8303_10880	13	0	0	0	0	0	2	0	0	3	1	1	1	2	1	1	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_2293_12053	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_16049_12791	2	0	0	0	0	0	1	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_25828_15956	47	0	5	0	0	0	2	1	4	0	4	2	1	8	0	6	5	2	3	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_23059_17918	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_23638_4659	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_14088_12072	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_16297_12364	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_18906_15466	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_20352_16896	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_24907_17228	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_5337_17921	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_7008_19978	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22428_23154	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_26047_23516	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18335_28660	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_18126_7545	7	0	1	0	0	0	1	0	0	0	1	2	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_9197_9746	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_6486_11603	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_23081_16781	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_19691_19130	6	0	0	0	0	0	1	1	1	0	2	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24561_21388	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_11260_22540	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24493_23253	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13281_5144	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_7693_5256	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_8805_12834	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12737_15531	2	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_29313_18090	2	0	0	0	0	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17368_18443	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_7426_18774	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_26976_19838	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_27262_22663	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_4667_8023	5	0	0	0	0	0	0	2	0	2	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_4974_8377	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_3610_17990	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_18825_20030	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_16200_26616	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_21500_6901	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10808_11763	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_25108_12490	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10641_14749	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18411_17052	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_6751_17524	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_25336_20130	2	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_8674_21438	2	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18294_27835	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_9185_6542	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_14485_10905	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_2128_16290	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_23768_16589	3	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20274_18664	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_11017_18921	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_7625_23288	3	0	0	0	0	0	2	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19821_2207	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21549_4010	5	0	1	0	0	0	1	0	1	0	0	0	0	0	0	0	1	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13400_5574	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_20220_13470	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12898_16926	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18788_21055	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_24430_23737	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13258_26764	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_25211_5709	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_3680_12611	10	0	1	0	0	0	1	1	0	0	0	0	0	0	0	3	4	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_7254_13419	71	0	0	0	0	0	1	1	7	11	7	26	18	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_28446_16991	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_27720_18606	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_5173_21862	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_23083_4972	6	0	0	0	0	0	0	0	1	1	4	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_19149_16619	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_23303_18988	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_9558_19765	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_25687_16182	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_14245_10215	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_8030_23327	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_9462_24856	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_12427_15408	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_22192_18692	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_22959_20262	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_14452_7789	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_6233_13205	2	0	0	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_17025_20887	3	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_19943_26491	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_14117_4744	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_18443_5251	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_14974_11618	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_19731_20280	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_11692_22443	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_11446_24358	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_15820_14074	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_26129_21059	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_18550_10076	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_5740_11365	10	0	0	0	0	0	1	0	0	0	2	1	4	0	0	0	0	1	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_17688_14654	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_22855_17407	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_23260_18439	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_4007_20870	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_10081_26254	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11899_4014	4	0	0	0	0	0	1	0	0	0	1	1	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_24855_7054	8	0	0	0	0	0	1	0	0	1	2	3	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_9712_10612	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22672_12392	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_17214_15927	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_23298_24981	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_20183_27184	3	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_16796_6663	2	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_27299_8066	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_17952_9366	4	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_18085_11993	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21823_22689	4	0	0	0	0	0	1	3	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_7430_24285	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_12502_2462	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_12364_4557	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_20868_11538	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_8962_26729	1	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_21305_8478	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_24952_9272	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16390_9529	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_25615_24528	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_20639_13713	7	0	0	0	0	0	0	3	0	3	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_17795_18609	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_7913_12679	6	0	0	0	0	0	0	0	0	6	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_16766_20121	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_13411_22875	2	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23248_7732	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_15748_9889	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_7782_10984	5	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_7070_12571	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_6617_17876	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_11751_17981	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_17600_8589	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_24828_8853	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_8707_15634	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_4106_18467	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_11497_20872	3	0	0	0	0	0	0	1	1	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_23697_7097	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_12236_14361	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_12085_17040	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_5620_18651	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_18846_19788	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_24903_5978	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_7966_6427	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_7612_7844	9	0	0	0	0	0	0	1	0	0	7	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_21132_10931	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_14488_13179	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_25867_16609	8	0	0	0	0	0	0	0	0	0	0	0	0	8	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_8474_16938	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22280_22887	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_12507_2610	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_23963_16017	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14299_17220	173	0	0	0	0	0	0	0	22	19	12	9	0	0	0	0	0	6	42	63" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_11559_26077	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_19506_3309	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_9752_18363	2	0	0	0	0	0	0	1	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8911_22008	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10420_3770	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17805_7443	5	0	0	0	0	0	0	1	0	2	0	0	0	0	1	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_7982_9474	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_25043_9903	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_27186_10400	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_20540_12162	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13931_12506	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_26670_15463	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_4925_16493	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21538_17471	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_16506_20291	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_12249_8143	4	0	0	0	0	0	0	1	0	0	1	1	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_24385_9596	6	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	5	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_19384_17999	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_24333_20558	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_15603_7816	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_15717_14694	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_25301_17173	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10190_23278	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10568_25353	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11161_25428	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_15029_25626	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_7560_26006	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_15529_5161	4	0	0	0	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_12882_9143	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_10619_10110	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_15548_11732	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_5425_15020	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_24111_15349	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_9498_21548	3	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18462_22624	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21829_7068	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8165_8910	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23944_10783	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21845_13793	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23518_14384	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_28031_16211	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14958_19894	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_7103_21358	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18112_24383	2	0	0	0	0	0	0	1	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_4159_12177	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_15912_14604	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13130_17466	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_24708_11365	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_10617_12843	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_8893_14291	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_21816_16599	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_19287_23968	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_3688_9745	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_12503_10753	4	0	0	0	0	0	0	2	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_26632_14477	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11460_19125	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_23355_23413	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_25789_8271	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_28710_13771	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_10372_21530	8	0	0	0	0	0	0	1	0	0	0	2	4	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_13048_23330	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_10730_3162	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_18687_19607	3	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_7943_14658	6	0	0	0	0	0	0	1	0	3	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_14859_14822	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_14031_23991	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_20224_25526	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12132_25703	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_11302_8835	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_12844_13432	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_21426_22343	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_14798_3606	7	0	0	0	0	0	0	1	0	0	0	5	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_22087_14118	38	0	0	0	0	0	0	1	0	4	1	3	1	12	2	2	3	0	4	5" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_18545_16099	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_23615_21129	5	0	0	0	0	0	0	1	0	0	2	0	0	0	0	1	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_27908_9851	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_28564_10017	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_20096_14197	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_7315_17427	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_14498_5844	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_11195_7814	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_13356_9691	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_11085_20271	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_22767_24294	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_5588_24334	3	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_19314_26014	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_20415_6804	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_20930_15299	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_16922_20621	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_21529_21161	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_14790_1829	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_14726_9028	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_21705_13651	2	0	0	0	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_8250_17966	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_20644_19641	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_8019_22340	2	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22323_22944	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_12032_3782	3	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_13414_6546	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_11875_9361	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_29656_14155	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_14906_9268	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_12423_13578	3	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_24274_20575	9	0	0	0	0	0	0	0	0	1	2	5	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_15082_26647	3	0	0	0	0	0	0	1	1	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_22681_5598	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11754_11390	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_4346_14940	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18693_22396	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_8524_16122	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_5327_17123	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_28102_18945	3	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12896_14027	3	0	1	0	0	0	0	0	1	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12195_22317	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_10830_22855	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_23279_25937	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_19589_20033	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_11335_21750	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_11983_14742	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_19059_19452	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_13025_22602	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_16224_23592	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_9515_26311	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_13721_7517	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_23133_8169	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_17680_15930	4	0	0	0	0	0	0	0	2	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_11208_20468	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_18001_27099	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_20108_5368	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_27199_15268	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22659_17429	3	0	0	0	0	0	0	0	1	1	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_26400_19526	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_19250_12721	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_27653_13933	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_25457_17518	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_13402_18823	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_15598_4401	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_26230_18106	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_9649_19293	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_23085_25742	2	0	0	0	0	0	0	0	1	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_19468_25966	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12199_12204	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_27306_15292	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10298_23586	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_14398_12314	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_26760_12620	7	0	0	0	0	0	0	0	0	0	0	0	0	1	0	1	0	0	3	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_21833_21040	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_10121_22720	9	0	4	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	3	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_18784_23504	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11788_5032	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_22172_7939	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_27973_10692	8	0	0	0	0	0	0	0	2	6	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_17691_12056	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_24992_23873	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14508_26097	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_21718_5711	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_9062_6419	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_25205_6894	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_15168_11856	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_21801_14983	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_7150_17375	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_6303_20947	2	0	0	0	0	0	0	0	1	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_14338_24655	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18114_26077	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_17203_1583	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9365_4621	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13983_6574	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18367_8585	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_5207_13479	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12357_13871	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_27471_20890	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10055_21108	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23025_24395	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_17730_26068	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13953_7091	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_26784_17549	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_24821_18536	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_7679_12922	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_16879_22327	4	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_19327_26344	9	0	0	0	0	0	0	0	1	0	0	0	0	6	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_10074_5728	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_9090_14716	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_22380_17554	4	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	1	0	1	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_14978_21851	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_15288_13724	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_23153_7113	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_23092_9359	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_7983_21245	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_22580_3507	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_4477_9888	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_20871_13331	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_10832_13902	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17741_19018	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_14743_23547	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_19127_1886	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_23675_17555	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_19676_23496	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_10109_11627	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_5052_15488	3	0	2	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_8100_6611	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_15158_9571	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_25409_11127	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_20493_16450	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_27247_19486	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_20937_6188	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_10607_6385	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_7457_8637	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_16325_18649	3	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_19140_21811	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_22006_25345	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_16365_1730	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_18844_14068	2	0	0	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_28638_18003	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_14835_3016	3	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_7837_7502	4	0	0	0	0	0	0	0	1	2	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22880_7920	5	0	0	0	0	0	0	0	1	4	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_25988_9682	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_10370_2988	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_27238_12024	4	0	0	0	0	0	0	0	1	0	2	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_23848_13201	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_26675_15471	2	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_9766_20824	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_7083_22842	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21662_22858	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21899_25122	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_24996_25271	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_8017_5788	4	0	0	0	0	0	0	0	1	0	0	1	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_9526_8096	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_4913_11992	1	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_10618_18178	2	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_5922_6513	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_8537_7072	11	0	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_23714_7638	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_2004_15863	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_9360_18060	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18682_27059	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_13343_4567	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22229_4678	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_12515_6375	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_15632_13472	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_16964_23052	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_19764_24086	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_21170_25493	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_18244_26058	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_14890_4136	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_14435_10999	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_9133_12955	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_11367_13197	5	0	0	0	0	0	0	0	0	4	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_19447_15932	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_23925_21591	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23236_5434	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24362_5528	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_5458_8163	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_9875_8400	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_5923_11043	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_29281_13264	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_19278_14733	2	0	1	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_27702_14915	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_11494_18542	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_6519_20940	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_9868_4758	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_20465_8611	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_27980_8986	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_16167_11796	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_3060_12805	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_8356_13589	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_3088_15065	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_19799_18751	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_12578_19811	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_10343_20641	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_16333_26309	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22453_4343	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_5748_9323	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_14254_12461	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_14256_14203	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_11283_15681	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_10941_17222	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_19006_23053	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_18879_6171	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_12430_16983	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_15708_17981	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22825_25509	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_10632_26179	4	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_24102_4725	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10328_10718	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_19974_13521	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_8714_18612	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14282_18817	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_4865_20136	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_25479_20227	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_19698_21996	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_16907_24932	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_17953_9880	4	0	0	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_26233_13220	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_21900_16751	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_18939_28276	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13298_3746	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_7766_4240	4	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_3587_9347	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15924_11287	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_22175_11297	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14278_11965	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_4801_12425	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_25249_14781	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17890_15072	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_19653_28369	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_11403_5737	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_15247_9017	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_17856_12278	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_8296_18650	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_5994_19726	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_4823_22009	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_8225_24102	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_22362_24215	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_11535_25208	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_14622_25632	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_20297_3328	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14080_4232	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_13976_5900	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_12988_6590	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10200_9979	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_25018_15007	2	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18213_15560	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_22961_17195	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_16230_18061	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14947_19152	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_16577_22524	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_17779_24453	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_16686_25858	3	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_27906_8733	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_7562_15957	4	0	0	0	0	0	0	0	0	3	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_28928_16770	5	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_28366_19519	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_15332_23431	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13627_24399	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12081_2719	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8829_4071	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19514_5000	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_24088_5500	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19071_5566	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8142_5567	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10373_6011	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_26027_6706	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_24119_8383	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_16655_8470	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13158_9182	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21924_11050	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8377_12333	5	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0	0	0	2	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12644_14109	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_20892_15372	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9308_15644	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10011_16636	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14308_17024	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14691_17449	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_20531_17680	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_16544_20390	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_7614_20887	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21716_25171	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9103_25313	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19795_28223	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_20164_2274	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17475_7154	5	0	1	0	0	0	0	0	0	1	0	0	0	1	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_26706_7767	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_28033_9425	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_6613_14290	2	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13474_15202	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_16354_15467	3	0	0	0	0	0	0	0	0	1	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_25853_16524	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_20145_22042	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_10084_22191	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13832_24138	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_21177_5197	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_13864_6831	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_11553_9071	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_10627_9330	6	0	0	0	0	0	0	0	0	1	0	0	0	0	0	2	2	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_15622_9697	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_22381_10329	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_7671_11355	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_10158_13191	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_19632_15186	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_22092_15614	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_5852_19432	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_11614_20083	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_18926_9248	2	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_6243_11429	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_12055_12246	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_23519_12843	4	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_29235_16481	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_21648_16922	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_12193_17343	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11783_23160	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_9278_23216	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_15345_25846	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11158_26523	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_15640_8806	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_18525_11294	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_26923_12364	4	0	0	0	0	0	0	0	0	3	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_26311_14104	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_9368_14850	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_27637_18457	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_14582_18801	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_18217_19581	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_14573_20188	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_7819_20401	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_21630_21997	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_22686_4913	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_9386_12253	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_18791_12472	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_18677_14481	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_21571_16049	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_19809_18745	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_7169_20746	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_10211_20812	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_23960_21462	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_12184_22923	5	0	0	0	0	0	0	0	0	5	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_20458_8348	4	0	0	0	0	0	0	0	0	1	0	2	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17976_10139	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17117_12389	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_28000_12820	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_3727_12917	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_7250_19443	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_16059_20423	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_9918_20510	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17332_22169	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17671_25995	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_12814_4583	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_7207_7404	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_22341_10832	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_18729_12719	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_20718_23357	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_14707_9807	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_6390_9974	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_4691_12067	4	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_17448_14122	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_17213_14575	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_26742_16490	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_12490_16505	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_7751_22068	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_16110_6542	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_6640_9328	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_17577_12359	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_16433_12706	2	0	0	0	0	0	0	0	0	0	0	0	0	1	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_25056_22893	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_13110_1977	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_15016_5261	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_16565_10623	3	0	0	0	0	0	0	0	0	2	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_7067_15061	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_17584_23187	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_24102_23570	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_17772_24108	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_14657_1748	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_10993_5398	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_6981_7811	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_10149_9721	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_5639_10651	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_25701_15655	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_27603_17495	7	0	0	0	0	0	0	0	0	1	0	0	0	1	0	2	1	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_15422_19940	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_26393_21929	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_23532_22350	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_18507_22576	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_15720_29081	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_19379_3300	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_16562_5570	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_21019_6342	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22227_7717	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_21004_8857	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_16997_10935	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_28271_11256	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11352_13241	3	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_28152_16274	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_13463_16561	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_12631_18383	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_12800_22855	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_18393_28178	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_15115_2610	3	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_20783_5301	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_18673_6259	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_4525_9290	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_23168_9750	3	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_24485_10653	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_12446_11570	3	0	2	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_5393_13669	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_4057_19587	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_11145_23790	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_17435_3730	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_17216_4292	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_18566_5886	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_16615_9462	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_24635_10372	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_24618_11376	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_4609_11919	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_24873_12639	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_28218_15620	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_7946_17808	1	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_22279_21530	2	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_16977_22333	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_18085_25907	4	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_3782_8109	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_13609_10025	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_25551_18277	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_21197_21065	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16143_23069	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14159_27572	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14993_16910	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12608_3163	3	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	1	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12866_5545	3	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_8412_8356	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_8361_9953	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_27900_10832	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_23908_15902	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_24460_16041	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_20010_17589	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_23750_17910	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_7754_25974	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_13037_4245	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_20402_11270	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_13776_17053	2	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23895_17769	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17143_19056	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_17602_8360	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_21138_11762	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_12311_16407	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_9374_22772	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22826_4799	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_25979_21288	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_15151_22654	2	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_19777_24988	2	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_5528_8801	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_10545_13535	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_24891_17279	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_21848_23651	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_13873_2429	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_12332_11456	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_28526_15940	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_21244_15970	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14370_23848	3	0	0	0	0	0	0	0	0	0	2	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_11020_6991	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_3671_10080	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_12151_14526	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_9215_20566	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_24315_7382	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17088_10563	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_22492_13115	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_8577_13143	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_25186_13251	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_29167_14391	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_26616_17589	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_25219_18794	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_24287_19984	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_26069_24105	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17713_26784	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_17009_11320	2	0	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_15129_15739	6	0	0	0	0	0	0	0	0	0	6	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_17550_17269	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_8849_21176	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_8141_21233	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_5848_24084	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_6317_24602	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_22736_6885	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_23531_7580	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18186_10299	6	0	0	0	0	0	0	0	0	0	2	2	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_15601_12718	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11370_15670	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_15967_2074	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_11281_2740	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_25861_7699	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_22462_8142	5	0	0	0	0	0	0	0	0	0	0	1	1	2	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_4659_12375	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19148_12981	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_17274_13189	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_26510_16371	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19795_24825	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_15543_24961	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13342_26294	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_25342_9197	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_27837_14578	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_16110_14718	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19529_17837	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22144_24942	4	0	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_22980_4780	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_7947_10616	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_12402_11394	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_23950_13306	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_22555_16416	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17322_21438	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_8178_6298	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_18868_8410	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_14252_8710	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_10683_16772	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_9710_20744	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_26214_23057	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_14512_24313	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11591_8763	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_24918_13396	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_21922_24386	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_20753_25954	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_16795_27764	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_14075_7093	4	0	0	0	0	0	0	0	0	0	4	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_4913_14332	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_4792_21056	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_14589_9498	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_21901_10432	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15139_15368	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_6171_20541	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_22749_25576	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_22746_5360	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_20527_14577	41	0	3	0	0	0	0	0	0	0	1	0	0	6	0	7	7	0	2	15" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_20577_15921	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_21459_16802	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_8106_16988	3	0	0	0	0	0	0	0	0	0	1	1	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_15186_22937	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_18656_26472	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_18354_8608	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_13016_9270	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_21893_17093	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_23464_17422	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_16172_11937	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_22756_12283	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_5881_15408	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_17961_17141	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_7645_19778	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_20601_20789	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_18874_21200	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_27773_22176	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_12744_24921	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_17324_3447	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_19574_4320	3	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_18721_12153	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_18300_12673	2	0	0	0	0	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_10394_17608	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_18345_18844	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_5497_7982	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_9717_11591	8	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	2	4" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_8652_13094	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_27146_22098	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_15516_28587	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_12821_2190	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_15523_6202	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_24817_7965	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_13555_13333	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_28308_17388	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_25352_17924	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_4814_18702	3	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_22131_23386	2	0	0	0	0	0	0	0	0	0	1	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_16258_26585	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_7963_5767	2	0	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26207_7695	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_6888_8092	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10008_8529	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_24725_10550	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_3905_12418	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_4873_16721	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26291_17310	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_8991_5702	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_7165_8175	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_15093_8390	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_25465_9646	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_7399_10515	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_5333_12504	6	0	0	0	0	0	0	0	0	0	6	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_11516_17364	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_26782_20607	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_7343_26333	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_22532_10009	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_21881_13315	13	0	0	0	0	0	0	0	0	0	2	0	0	3	6	1	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_10056_13609	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_14273_14388	1	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_16268_21350	2	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_8877_3935	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_20589_8650	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_26634_17611	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_25610_23225	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16914_26242	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_19690_27437	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_10298_5101	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_12417_5953	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_26098_19093	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_15600_19124	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_13035_5452	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_6514_8778	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_4372_14342	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_24229_16660	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_22842_22813	5	0	0	0	0	0	0	0	0	0	0	1	1	3	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_16201_28240	2	0	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_15789_5235	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_7613_8427	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_4327_10073	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_10691_10850	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_8585_16521	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_22299_24998	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_14219_3778	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_7287_9945	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_8310_14174	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_11090_20196	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_19617_4733	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_17636_10033	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_3847_15449	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_20020_16264	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_3005_18956	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_7807_21999	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_17052_3882	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_23853_7456	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_3548_11064	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_4609_18391	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_27878_20060	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_4622_21981	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_23381_22618	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_19357_22811	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10061_3691	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_11391_5528	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10511_13856	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_2644_15119	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_11375_17687	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_27208_20174	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_9239_22168	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_23966_22243	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18681_22857	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_24171_25394	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24109_8015	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8925_8850	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24068_9219	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8621_15527	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_14735_26068	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_16853_2840	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_26450_7049	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15637_12992	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_23003_13541	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_27941_14131	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18108_15259	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_16185_17163	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18810_18264	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17818_21151	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13999_26329	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18844_27717	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_19891_5082	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_6250_5987	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_15521_13523	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_21999_17538	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_6400_18545	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_18978_23806	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_23416_24696	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_14865_2720	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10073_3247	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9636_6179	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_7981_6310	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_17991_13659	2	0	0	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_19464_17673	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_20972_19668	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_13563_20676	2	0	0	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_24714_23272	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_14734_11233	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_16174_11502	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_16063_11536	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_14703_18340	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13105_21336	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20503_21801	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18739_8131	3	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18510_8776	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_20440_13060	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_6495_13602	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_6141_22497	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22942_23318	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8492_23708	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_7676_25736	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_7920_25776	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_10482_5509	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_16339_18404	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_21947_18735	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_8442_23370	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_16703_5011	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_25497_17323	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_18298_21125	2	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_16606_4023	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_9222_10640	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_13384_15039	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_24284_21559	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11687_23017	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_15669_23137	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_18888_26858	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_18172_10876	3	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_4696_10935	20	0	5	0	0	0	0	0	0	0	0	0	1	7	0	0	0	0	0	7" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_23298_12028	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_11018_16527	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_9988_18586	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_20074_25538	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_25408_18555	4	0	0	0	0	0	0	0	0	0	0	2	0	0	1	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15118_19690	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_10776_6955	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_5691_11023	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_12440_13627	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_25956_17550	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_19274_24278	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_22868_26146	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_18324_28644	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_17002_3531	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_28168_19224	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_5824_20256	3	0	0	0	0	0	0	0	0	0	0	1	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_16642_25679	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_19974_4752	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_27871_9754	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_23551_13901	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_21817_19761	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_10602_19971	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_10681_7135	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_10445_7921	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_25599_13567	3	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_20708_17882	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_19808_24467	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_16975_4772	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_17263_11534	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_11463_19602	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_9619_23737	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_9093_4236	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_24260_7949	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_23722_9668	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_9811_9982	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_6265_10463	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26969_15672	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_22900_6448	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_8012_12345	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_14931_21754	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_17959_5140	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_8388_7715	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_11892_8192	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_14201_8661	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_24035_9289	2	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_23230_12044	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_20216_27116	1	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_25878_10062	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18517_10721	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_8786_10844	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_15683_17782	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_27414_13728	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_25912_14594	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22379_19804	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_17902_26485	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_11491_13931	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_20592_3815	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_19763_16198	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_6949_16896	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_13842_19281	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_20923_14281	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_8066_16367	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_5527_16685	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_15986_21920	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_7832_8780	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_25872_9951	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_13116_15821	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_6365_24079	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_23570_6782	2	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_10864_8146	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_10494_10758	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_2986_15638	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14609_6975	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14855_8462	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17014_10530	2	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13330_10638	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14814_10944	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17376_13100	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_25975_18317	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15744_23114	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_24650_7997	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_12028_24705	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_22806_22503	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_16033_3594	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19457_3875	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_6268_10349	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_27297_13091	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13321_19992	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18175_23385	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9210_8620	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_5058_10624	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_24795_10657	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_16474_12783	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_10680_16017	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_20223_25753	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_17579_3448	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12732_5062	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_5706_8003	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_17714_13657	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_17415_24873	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_9943_6220	4	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	1	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_21537_12875	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_22564_6206	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_6882_12684	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_10589_14796	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_10122_15632	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_17039_19920	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15636_26203	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_23085_20010	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_14648_14535	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_16534_14769	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_4030_17110	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_10705_11765	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_7624_17316	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_5933_20471	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_14981_25247	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_13011_10787	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_18804_14594	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_17603_21659	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_26207_14420	6	0	0	0	0	0	0	0	0	0	0	0	4	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_10010_14992	2	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_23990_16750	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_24147_19370	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_8304_3695	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_28641_13458	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_19706_18823	2	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_8777_8861	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_2702_12868	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11736_17751	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_24625_19641	2	0	0	0	0	0	0	0	0	0	0	0	1	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_6545_8894	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_20319_11310	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_9164_14402	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_14482_15634	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_19649_18554	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_5702_10845	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_18278_17470	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_18555_17761	1	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_3758_10627	10	0	2	0	0	0	0	0	0	0	0	0	0	3	2	1	0	1	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_7971_11346	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_8995_24296	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14900_7576	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_3890_17221	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_18945_9651	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_28341_20456	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_11900_6285	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15090_18340	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23929_25607	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_9733_12197	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_26583_8053	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_17032_8072	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_13967_11089	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_27189_20696	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_20841_7623	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_27354_15573	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_23011_5924	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_3757_10082	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_5351_11203	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24372_12546	28	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	28" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_13648_13972	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13879_1994	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_17627_6261	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_4449_10146	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_3713_15410	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_9431_18167	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21712_25652	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_22050_3053	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_14408_20028	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_10387_25149	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11506_10347	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9943_15231	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_27130_19681	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_22274_20191	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9141_21408	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_24306_21531	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_12062_8139	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_21002_11725	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_10719_18379	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_12649_18573	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19332_23681	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9368_6991	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_27304_8277	3	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_27446_10971	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23881_16039	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10573_19345	8	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	1	0	4" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18058_25781	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13551_2912	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13320_3436	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_28391_18939	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_12255_19099	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17292_21391	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_22604_4240	3	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12782_8845	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_24666_20800	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_18975_9767	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_7856_11382	3	0	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_6640_14822	2	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_18899_8185	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_25226_10311	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_26015_12877	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_19668_20827	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17102_4744	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_5552_10582	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_16184_19905	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_23846_21173	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_18053_21392	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_8153_5482	2	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_14104_6500	2	0	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_7268_7252	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_23336_11976	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_27195_14074	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_14096_15016	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_10803_20222	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_18358_2270	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_11790_16027	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_22647_16398	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_10122_16816	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_22189_18381	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_20252_6302	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_6882_14781	2	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_11269_19733	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_15226_21168	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_14530_23810	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_9416_24199	2	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_11932_12993	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_7822_24159	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_21066_7506	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_19248_7551	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_8291_8111	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_13579_10781	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_12166_13729	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_5139_21514	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_23078_24856	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_18460_8495	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_23259_13531	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_22954_15571	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_13923_24399	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_11924_15441	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_11757_15493	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_25899_22034	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_22451_22884	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_15143_1651	2	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_28476_17852	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_4499_18914	1	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_23703_5129	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16641_11299	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_7373_14922	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_18106_21085	4	0	0	0	0	0	0	0	0	0	0	0	0	1	2	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_14915_26251	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_13258_9769	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_12551_11293	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_11222_12600	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_13900_15927	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_4170_17056	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_7847_17999	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_13319_18244	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_24841_18959	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_10390_20786	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_16656_25520	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14928_26203	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_21608_2864	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_8349_14557	3	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12270_15236	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_28451_19405	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_16540_20675	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_26302_22051	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_7862_4687	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_15705_5052	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24923_8045	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_27270_10834	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24270_11650	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_6527_16493	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23422_20961	5	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	3	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_10756_24942	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23231_4481	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_25694_18609	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_12786_23414	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_9050_25130	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_18275_26860	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_21706_6039	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_14010_12420	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_28698_12913	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_23262_25144	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_15436_27017	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_15496_5008	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_13819_9393	8	0	0	0	0	0	0	0	0	0	0	0	0	5	0	3	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_9318_10158	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_16665_12409	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_25332_17915	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_13057_19114	2	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_13878_20553	7	0	0	0	0	0	0	0	0	0	0	0	0	4	0	0	2	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_22866_20909	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_27163_22727	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18422_23262	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_23186_26439	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_13208_3584	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_18196_9880	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_9608_10994	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_2798_11023	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_24246_18294	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_14610_28830	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12004_2617	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15277_6482	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_20095_8247	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18711_9702	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_27797_16769	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_28129_18678	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_2564_19006	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21383_22868	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_13950_25143	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_20933_6700	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_17207_6739	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_16441_6974	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_25950_7889	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_13772_8177	5	0	0	0	0	0	0	0	0	0	0	0	0	4	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_23503_22763	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_15637_23152	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_22982_24025	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9419_4295	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_21753_6188	9	0	0	0	0	0	0	0	0	0	0	0	0	9	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_23738_7093	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_12887_9342	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18981_20025	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9624_20695	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_9306_24571	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13032_6958	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18700_9502	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_4490_10708	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_22863_10725	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_14224_15638	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_6747_17324	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_6843_21123	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_9810_23014	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_24017_25976	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_12965_2018	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9915_4609	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_6688_5293	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18296_7709	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_16123_8166	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10520_9402	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8934_14248	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_28534_17138	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22494_17866	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8059_18290	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_15498_19369	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_3483_21724	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18158_25538	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_7036_14239	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_4780_15092	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_8034_16374	3	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_23114_22755	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_22573_22946	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_9650_25321	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_16491_2474	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_12357_13497	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_25051_20557	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_27740_10743	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_26747_11121	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_15482_15086	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_10962_17123	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_4125_17157	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_28700_17194	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_19006_21979	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_17104_22073	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_19878_24663	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_15244_25817	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_20064_5598	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_16989_10433	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_8408_16485	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_19264_18585	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_9326_20908	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_16818_5464	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_18070_5688	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15982_24425	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_25719_12422	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_15051_17101	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_18284_2758	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_24787_25663	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_16667_9805	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_7255_14963	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_20651_22709	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_13526_22864	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_16258_1585	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_19519_2320	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_6453_7009	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_19279_7836	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_27058_11840	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_8160_14127	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_21370_14363	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_24964_16600	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_19976_22044	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_11412_26175	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_14780_27058	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_13351_28335	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_17635_9975	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_27183_12779	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_17618_20533	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_16323_3522	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_15598_4100	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_12205_7359	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_12498_13580	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_21247_15845	3	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_13174_19348	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_8528_6078	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_8454_7758	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_12749_8826	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_12022_9082	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_16774_9147	6	0	0	0	0	0	0	0	0	0	0	0	0	2	0	1	0	1	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26347_9344	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_21038_9589	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11313_13873	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_26672_16660	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_20105_17530	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21249_7964	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_28097_13665	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21308_17042	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_13714_22583	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_18276_27021	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_13275_27560	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_19280_28263	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_18928_10051	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_12634_10967	2	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_18715_18416	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_17062_26446	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_10524_26952	2	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_4190_14354	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_16182_25089	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_19172_4834	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_7684_6622	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_18640_14309	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_27632_18343	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_22654_26891	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_10606_3096	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_20588_5990	4	0	0	0	0	0	0	0	0	0	0	0	0	0	4	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_18476_6578	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_8057_17803	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_17151_24507	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_21018_4980	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_6877_11063	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23025_12099	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_9992_20296	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_22535_20381	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_5363_6161	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_15955_6621	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_28442_12734	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_21550_17093	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_26777_15267	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_7191_15338	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22580_21773	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_8770_12045	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_27927_21606	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_13493_4507	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_16625_10134	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8693_13422	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_18240_15644	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15560_14481	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21962_17433	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_19644_17655	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_18205_23899	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_13586_2804	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_28850_15369	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_19252_26342	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_15982_3044	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_12814_18299	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_16472_4147	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20973_15490	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21607_13167	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10193_19783	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_26942_22032	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_25025_23939	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_12238_5807	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_25412_7972	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13838_9027	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_9500_21244	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17218_22721	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_16924_2551	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_14650_8724	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_23398_14571	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_26372_21137	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_20055_23053	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_16102_5245	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_19435_2222	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_17227_9135	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_11929_10165	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_6381_12893	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_17795_17991	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_17870_18881	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_24305_5134	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_24795_12844	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_19416_13506	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_3878_16016	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_18816_26393	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_23285_7924	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17127_12464	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_26879_8110	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_4339_17322	3	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_23299_14833	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_27865_15182	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_12449_21256	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_15244_28739	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_14413_2449	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_12163_4647	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_29414_16449	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_13599_17305	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_12889_27699	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_22417_22191	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_11494_8595	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_27658_13780	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_17191_16655	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10029_17181	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_9492_17806	7	0	0	0	0	0	0	0	0	0	0	0	0	0	7	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_16978_21215	2	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_26633_11062	1	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_7006_17302	2	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11952_13519	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11538_27185	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_26403_8487	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_12197_8664	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23849_8759	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_14019_23841	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_10463_16723	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_23455_4853	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_23502_7683	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_24555_20879	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_6775_23397	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18644_12052	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_2714_17713	10	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	10" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_17670_1881	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8914_20463	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_11326_21178	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_16714_23075	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_14688_5568	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_7275_13443	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_16691_26561	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_25344_22139	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18755_1824	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_5981_8948	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_28222_11771	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_19287_12246	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_24926_20157	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_21719_22520	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_23986_14034	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_14260_20208	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_27071_8397	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_4902_9630	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_3366_17894	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_6909_12719	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17437_26807	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_25569_21727	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_24176_5781	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_25987_20682	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_22154_4269	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_19605_13841	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_21156_7257	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_9580_7709	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_24670_14227	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_23373_7365	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_17640_11873	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_13773_19305	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_13406_20132	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_25850_8754	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_15661_13758	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_26075_20260	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_25923_16574	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_10631_18447	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_9362_22992	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_23067_17619	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_17711_14171	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_11460_16881	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_7333_20307	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_25145_5016	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10585_20149	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_13419_4336	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_25427_14638	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_16137_10353	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_20979_14273	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_15011_15369	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_10382_22128	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11823_22466	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_23742_6714	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_7350_12102	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_23689_6395	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_14415_16643	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_20257_16697	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15601_18021	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_6004_8081	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_28406_9787	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_25959_18524	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_17965_25181	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_9130_13455	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_21227_17435	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_21905_17218	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10305_19494	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_16960_19773	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_12799_6921	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_11295_11901	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_26818_15061	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_21458_15751	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_4756_16948	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_20810_3468	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_8324_8698	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_15641_10799	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_22680_22915	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_20712_23782	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_12315_7486	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_25166_7929	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_26033_16351	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_7963_16361	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_20322_27077	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_20872_8285	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_21353_8415	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18378_12213	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_23441_23046	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_13632_26764	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18521_9535	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_17095_9759	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_28094_17280	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13971_21290	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_10248_22431	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_11237_4361	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_17281_4476	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_6229_5326	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_15409_10721	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_23027_15678	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_19698_19740	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9024_20163	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_18444_24197	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_24924_5766	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_6223_8694	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17527_9958	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_22000_19843	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_10248_4992	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_16184_16848	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_15051_3502	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_22490_8729	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_7735_17880	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_16979_24828	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_25692_5505	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_26030_6786	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_15684_7042	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_23034_13380	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_7943_8043	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_22379_16929	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_19379_4847	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_8679_15164	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_6366_20022	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_10624_8294	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_22934_10355	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_20507_18628	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_11818_4055	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_18302_14612	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_7561_16231	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_26358_22652	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_12975_25037	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_7813_4701	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_18108_6799	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_25240_9047	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_23522_9652	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_22930_12602	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_5187_18269	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_22245_24576	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_7971_11807	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_10343_15424	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_17860_28348	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_15112_7605	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_10335_11049	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_18144_24421	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_4494_9088	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_19864_14092	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_21647_20109	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_10324_8129	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_22474_15335	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_26629_16356	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_24745_24299	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_6774_6343	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_8406_20325	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_23234_7134	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_24150_14758	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_7927_15626	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_18369_27790	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23153_7273	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_5336_9821	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_18791_11806	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_10112_8040	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_22795_14196	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_21768_26571	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_4549_14740	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21619_15325	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_22113_16232	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_22982_25824	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_10106_14274	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_11092_24430	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_13372_27170	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_26249_6887	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19563_8007	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_20185_10282	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_28261_14681	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_11760_14772	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_22706_5018	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_8550_6381	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_25131_8732	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_11189_15616	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13266_17804	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_7871_10547	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_8408_13436	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_27613_20155	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_10592_12969	4	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	4	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_20457_11366	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_7538_13991	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_11226_20980	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_12620_23717	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_23117_11899	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_24314_8984	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_19623_12472	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_23721_15987	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_11592_18531	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_19846_20892	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_17074_12360	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_13415_4791	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_24286_16039	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_7386_6808	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_5448_14619	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_11983_2593	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_12861_15386	10	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	7" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_22608_17863	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_11664_24324	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_4808_12955	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_13340_15811	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_27296_22083	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_5882_22900	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_18033_2621	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_8681_8228	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_18589_17554	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_5459_18973	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_7344_20185	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_26352_22378	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_22214_8324	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_23336_26415	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_14383_15110	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_19892_23978	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_16375_7276	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_7699_18608	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12977_5827	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21850_8958	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_4126_16552	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_6766_19046	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_21794_19898	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_12096_22703	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_12881_9910	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_25638_13115	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_5281_15326	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_16971_16405	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_27925_17349	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_2989_20122	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_20414_23391	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_9372_3141	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_11975_6040	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_25479_14916	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_8826_23586	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_13201_9116	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9494_11127	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_16967_16702	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_16841_5683	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_17219_12550	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_10104_19430	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_18381_13991	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_5906_19319	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_27141_19101	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_7853_22683	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_12547_24481	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_8663_16101	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_23590_17131	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_24218_17682	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_7116_11773	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_13267_15722	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_25509_16535	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_27387_19532	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_11282_7357	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_7483_16182	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_19509_25673	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_14454_5355	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_15232_28922	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_15065_8580	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_6753_10007	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2109_17004_17030	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_18060_5720	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_21476_15790	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_25915_23958	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_19511_20526	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_14544_23278	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_13608_21597	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_17014_4226	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_19251_6041	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0" << endl;
out << "M00967_43_000000000-A3JHG_1_1101_20702_8361	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_14586_9284	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1102_11158_17229	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_12856_12076	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_19321_18100	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1103_9350_18825	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_12015_3840	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_24088_7145	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_18319_13388	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_19151_18692	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1104_19580_19072	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_7593_10522	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_23258_10532	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_17706_11167	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_20476_12712	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_14935_17739	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_17574_20335	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_15558_26326	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1105_11242_27278	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_16075_4721	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22705_6123	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_25330_8604	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_24378_9680	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_24084_10563	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1106_22389_22397	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_11459_12453	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_6826_23774	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1107_21734_25420	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_10822_18112	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1108_21553_18669	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_6553_9924	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_20113_10044	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_8799_10155	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1109_17950_22790	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_23557_12709	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1110_10169_13508	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1111_10018_24068	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_10723_2426	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_29269_13288	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1112_18506_25911	3	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	3" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_19158_8422	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_13987_16790	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_26517_18281	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1113_18370_24961	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_10874_10296	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_9080_19413	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_14939_20476	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_1114_21630_21181	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_13911_2481	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_24119_7738	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_24905_9024	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_14159_9619	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_26453_14067	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2101_26265_15434	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_19730_4917	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_25992_9452	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_15674_19713	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_14775_20254	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2102_17791_24913	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_9230_6170	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2103_11391_8155	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_11063_11086	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2104_26300_11226	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_17259_2092	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_15512_14920	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2105_25960_21171	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_25081_10002	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_28405_10515	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_5950_12743	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2106_13686_15465	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_23657_12369	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_14468_14971	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_10874_20271	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2107_23641_20458	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_22703_14521	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2108_10549_22228	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_8800_5111	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_25043_10243	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_24113_10963	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2" << endl;
out << "M00967_43_000000000-A3JHG_1_2110_20350_23961	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_4991_9664	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2111_12365_24314	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2112_25043_23608	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_8502_12671	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2113_6721_20524	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_22468_4333	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
out << "M00967_43_000000000-A3JHG_1_2114_6705_14284	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1" << endl;
    
    
    out.close();
    
    return "count.temp";
}
/***********************************************************************/

